<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 6cc64d37551c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:32 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
define('WE_DOCUMENT_ROOT', (string) realpath(__DIR__ . '/../../../../'));
require_once(WE_DOCUMENT_ROOT . '/webEdition/we/include/we.inc.php');
we_html_tools::protect();

$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', 0, 2);

$out = [];
switch(($cmd = we_base_request::string('we_cmd', '', 0))){
	case '':
		exit();
	case 'editSource':
		try{
			$doc = we_contents_root::getSessionDocument($we_transaction);
			assert($doc instanceof we_document_files);
			$filename = preg_replace('|/' . $doc->Filename . '.*$|', '/' . $doc->Filename . $doc->Extension, $doc->Path);
		} catch (Exception $ex){
			$filename = we_base_request::_(we_base_request::FILE, 'we_cmd', '', 1);
		}

		$ct = we_base_request::string('we_cmd', '', 3);
		$tmp = we_base_request::_(we_base_request::RAW_CHECKED, 'we_cmd', '###EDITORPLUGIN:EMPTYSTRING###', 4);

		$source = ($tmp === '###EDITORPLUGIN:EMPTYSTRING###' ? $_SESSION['weS']['we_data'][$we_transaction][0]['elements']['data']['dat'] : $tmp);

		$out = [
			'cmd' => $cmd,
			'session' => session_id(),
			'sessionName' => session_name(),
			'transaction' => $we_transaction,
			'filename' => $filename,
			'ct' => $ct,
			'source' => base64_encode($source),
			'charset' => 'UTF-8'
		];

		break;
	case 'editFile':
		$we_doc = we_contents_root::getSessionDocument($we_transaction);

		$filename = $we_doc->Path;
		$we_ContentType = $we_doc->ContentType;


		$tmp_file = TEMP_DIR . basename($filename);

		if(file_exists(($file = $we_doc->getElementS('data')))){
			copy($file, WE_DOCUMENT_ROOT . $tmp_file);
		} else {
			t_e($tmp_file . ' not exists in ' . __FILE__ . ' on line ' . __LINE__);
		}

		$out = [
			'cmd' => $cmd,
			'session' => session_id(),
			'sessionName' => session_name(),
			'transaction' => $we_transaction,
			'ua' => $_SERVER['HTTP_USER_AGENT'],
			'lang' => $_SERVER['HTTP_ACCEPT_LANGUAGE'] ?? '',
			'enc' => $_SERVER['HTTP_ACCEPT_ENCODING'] ?? '',
			'filename' => $filename,
			'ct' => $ct,
			'tmp' => we_http_util::getServerUrl(true) . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_base_showTemp&file=' . str_replace(WEBEDITION_DIR, '', $tmp_file)
		];

		break;
	case 'setSource':
		$doc = we_contents_root::getSessionDocument($we_transaction);
		$doc->setElementS('data', we_base_request::_(we_base_request::RAW_CHECKED, 'we_cmd', '', 2));
		$doc->saveInSession($we_transaction);
		$out = [
			'cmd' => $cmd,
			'transaction' => $we_transaction,
		];

		break;
	case 'reloadContentFrame':
		$out = [
			'cmd' => $cmd,
			'transaction' => $we_transaction,
		];

		break;
	case 'setBinary':
		if(isset($_FILES['uploadfile']) && ($we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction', 0))){
			$we_ContentType = we_base_request::string('contenttype');
			$we_doc = we_contents_root::getSessionDocument($we_transaction);

			$tempName = TEMP_PATH . we_base_file::getUniqueId();
			move_uploaded_file((string) $_FILES['uploadfile']['tmp_name'], $tempName);
			we_base_file::insertIntoCleanUp($tempName, 3600);

			if($we_ContentType === we_base_ContentTypes::IMAGE){
				$we_doc->setElementS('data', $tempName, 'image');
				try{
					$dim = we_contents_thumbnail::getimagesize($tempName);
					$we_doc->setElementI('width', $dim[0], 'attrib');
					$we_doc->setElementI('height', $dim[1], 'attrib');
				} catch (we_exception_image $e){

				}
			} else {
				$we_doc->setElementS('data', $tempName);
			}

			$we_doc->saveInSession($we_transaction);
		}
		break;
	default:
		t_e('error', "command '" . we_base_request::string('we_cmd', '', 0) . "' not known!");
}

we_html_tools::headerCtCharset('text/html');

echo we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'weplugin_cmd.js', '', ['id' => 'loadVarWeplugin_cmd', 'data-weplugin' => setDynamicVar($out)]), we_html_element::body());
