<?php
declare(strict_types=1);

/**
 * if hook execution is enabled this function will be executed
 * when publishing a document, template, object or class
 * The array $param has all information about the respective document.
 *
 * IMPORTANT!
 * Copy this file to the custom/hooks folder when doing any changes. This file needs to be renamed to
 * weCustomHook_user_preLogin.inc.php
 *
 * Files in the sample_hooks folder are not executed and are not update-safe and will be overwritten by the next webEdition update
 *
 *
 * @param array{hookHandler:we_hook_base,type:string,user:array} $param
 * Use and install https://github.com/adldap/adLDAP/ in $MYLIB_PATH.
 */
class we_hook_user_preLogin extends we_hook_base{
	/** contains modifyable Username, password, PLEASE NEVER SAVE plaintext password */
	protected $user;

	/** @var string */
	protected $type;

	public function __construct(we_hook_handler $handler, array $params){
		parent::__construct($handler, $params);
		$this->user = &$this->params['user'];
		$this->type = $this->params['type'];
	}

	public function execute(): bool{
		$pwdCheck = &$this->params['checkPassword']; //!!IMORTANT if this is set to false, password is not compared with the saved password

		switch($this->type){
			case 'login':
				if(true /* use this to deactivate LDAP authentication */){
					$pwdCheck = false; // don't check webEdition password

					$DB_WE = DB_WE::$inst;

					// Authenticate username and password against LDAP service of ActiveDirectory. webEdition user password does not matter.
					$authOK = false;
					// Only authenticate via LDAP/AD if username provided
					if(strlen($this->user['Username']) > 0){
						@include_once($MYLIB_PATH . '/src/adLDAP.php'); // load and initialize AD object
						if(class_exists('adLDAP') && isset($MYCOMPANY_adLDAP)){
							try{
								$MYCOMPANY_adLDAP = new adLDAP(
									[
									'account_suffix' => '@ad.mycompany.de',
									'base_dn' => 'ou=myou,dc=ad,dc=mycompany,dc=de',
									'admin_username' => 'lalaadminusername',
									'admin_password' => 'lalaadminpassword',
									'domain_controllers' => ['ad.mycompany.de'],
									]
								);
							} catch (adLDAPException $e){
								// server might just be down for the moment. warning-level error only
								t_e('warning', "LDAP exception while authenticating:\n" . $e->toString());
							}

							// actual authentication
							if($MYCOMPANY_adLDAP->user()->authenticate($this->user['Username'], $this->user['password']) === true){
								//t_e('warning', 'DEBUG: LDAP authentication successful.');
								$authOK = true;
							} else {
								t_e('warning', sprintf('LDAP authenticating for existing webEdition user (%1$s) failed because of wrong credentials', $DB_WE->escape($this->user['Username']))); // never log unescaped user data to DB or mail ;)
							}
						} else {
							t_e('warning', 'LDAP library adLDAP not available when trying to log in.');
						}
					}

					if(!$authOK){
						$this->hookHandler->setErrorString('LDAP authentication denied');
					}
					//return $authOK;
				}

				break;
			case 'resetPassword':
			//FIXME: not implemented yet
		}

		$this->hookHandler->setErrorString('Login denied');
		return false;
	}

}
