<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 546bf5b43dc2 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-22 00:48 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_contents_objectFile extends we_contents_objectBase{
	public const BACKUP_XML_TAG = 'we:object';
	public const ERROR_NOT_SAME_CLASS = -3;

	/** @var int */
	var $TableID = 0;

	/** @var int */
	var $rootDirID = 0;

	/** @var string */
	var $RootDirPath = '/';

	/** @var int[] */
	var $AllowedClasses = [];

	/** @var array */
	var $DefArray = [];

	/** @var string */
	var $Url = '';

	/** @var int */
	var $TriggerID = 0;

	/** @var array{ID:int,ParentID:int,Filename:string,ContentType:string,Creation_Date:string,Mod_Date:string,Path:string,CreatorID:int,ModifierID:int,RestrictOwners:int,Owners:string,OwnersReadOnly:string,DefaultCategory:string,DefaultParentID:int,DefaultText:string,DefaultValues:array,DefaultDesc:string,DefaultTitle:string,DefaultKeywords:string,DefaultUrl:string,DefaultUrlfield0:string,DefaultUrlfield1:string,DefaultUrlfield2:string,DefaultUrlfield3:string,DefaultTriggerID:string,Workspaces:int[],DefaultWorkspaces:string,Templates:int[],Text:string,WorkspacesTemplates:array<int,int>} */
	protected $classData;

	/** @var bool */
	private $DefaultInit = false; // this flag is set when the document was first initialized with default values e.g. from Doc-Types

	/** @var string database table in which the object is stored */
	public $Table = OBJECT_FILES_TABLE;

	/** @var string ContentType of the Object  */
	public $ContentType = we_base_ContentTypes::OBJECT_FILE;

	public function __construct(int $id = 0){
		parent::__construct();
		if(!isset($GLOBALS['WE_IS_DYN'])){
			$this->AllowedClasses = we_users_util::getAllowedClasses();
		}
		if($id){
			$this->initByID($id);
		}
	}

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		$this->PublWhenSave = false;
		$this->IsTextContentDoc = true;
		$this->persistent_slots = array_merge($this->persistent_slots, [
			'CSS' => we_base_request::INTLISTA,
			'DefArray' => we_base_request::SERIALIZED,
			'AllowedClasses' => we_base_request::INTLISTA,
			'Workspaces' => we_base_request::INTLISTA,
			'RootDirPath' => we_base_request::STRING,
			'rootDirID' => we_base_request::INT,
			'TableID' => we_base_request::INT,
			'Category' => we_base_request::INTLISTA,
			'IsSearchable' => we_base_request::BOOL,
			'Language' => we_base_request::STRING,
			'Url' => we_base_request::STRING,
			'TriggerID' => we_base_request::INT,
			'classData' => we_base_request::SERIALIZED
		]);

		if(isWE()){
			if(we_modules_info::isActive(we_modules_info::CUSTOMER) && we_base_permission::hasPerm([we_base_perm::CUSTOMER_CAN_EDIT_CUSTOMERFILTER])){
				$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_WEBUSER;
			}
			$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_PROPERTIES, we_base_constants::WE_EDITPAGE_INFO, we_base_constants::WE_EDITPAGE_CONTENT,
				we_base_constants::WE_EDITPAGE_WORKSPACE, we_base_constants::WE_EDITPAGE_PREVIEW, we_base_constants::WE_EDITPAGE_VARIANTS, we_base_constants::WE_EDITPAGE_VERSIONS,
				we_base_constants::WE_EDITPAGE_SCHEDULER]);
		}
		$this->DefArray = $this->getDefaultValueArray();
		$this->objectFileInit();
	}

	/**
	 *
	 * @param int $classID
	 * @param string $formname
	 * @param int[] $categories
	 * @param int $parentid
	 * @param int $objID
	 * @param bool $wewrite
	 * @return \we_contents_objectFile
	 * @throws we_exception_base
	 */
	public static function initObject(int $classID, string $formname = we_base_constants::DEFAULT_FORM, array $categories = [], int $parentid = 0, int $objID = 0, bool $wewrite = false): we_contents_objectFile{
		$session = we_tag_sessionStart::$started;

		if((!$session) || (!isset($_SESSION['weS']['we_data']['we_object_session' . $formname])) || $wewrite){
			we_tag_write::$objElements[$formname] = $wof = new we_contents_objectFile();
			$wof->we_new();
			if($objID){
				$wof->initByID($objID);
				if(!$wof->TableID){
					throw new we_exception_base('no class definition found');
				}
			} else {
				$wof->TableID = $classID;
				$wof->setRootDirID(true);
				$wof->resetParentID();
				$wof->restoreDefaults();
			}
			if(($wewrite || !$objID)){
				$wof->Category = $categories ?: $wof->Category;
				if($parentid){
					// check if parentid is in correct folder ...
					$parentfolder = new we_folder_objectFile($parentid);

					if(($wof->ParentPath == $parentfolder->Path) || str_starts_with($parentfolder->Path . '/', $wof->ParentPath)){
						$wof->ParentID = $parentfolder->ID;
						$wof->Path = $parentfolder->Path . '/' . $wof->Filename;
					}
				}
			}

			if($session){
				$wof->saveInSession('we_object_session' . $formname);
			}
		} else {
			if($objID){
				we_tag_write::$objElements[$formname] = $wof = new we_contents_objectFile();
				$wof->initByID($objID);
			} else {
				$wof = we_contents_root::getSessionDocument('we_object_session' . $formname);
				assert($wof instanceof we_contents_objectFile);
				we_tag_write::$objElements[$formname] = $wof;
			}
			$wof->TableID = $classID ?: $wof->TableID;
			$wof->Category = $categories ?: $wof->Category;
		}

		$wof->DefArray = $wof->DefArray ?: $wof->getDefaultValueArray(); //bug #7426

		if(($ret = we_base_request::_(we_base_request::URL, 'we_returnpage'))){
			$wof->setElementS('we_returnpage', $ret);
		}

		if(isset($_REQUEST['we_ui_' . $formname]) && is_array($_REQUEST['we_ui_' . $formname])){
			foreach($_REQUEST['we_ui_' . $formname] as $n => $v){
				$v = we_base_util::rmPhp($v);
				$ret = $wof->convertElemFromRequest('', $v, (string) $n);
				if(is_int($ret)){
					$wof->setElementI((string) $n, $ret);
				} else {
					assert(is_string($ret));
					$wof->setElementS((string) $n, $ret);
				}
			}
		}
		if(isset($_REQUEST['we_ui_' . $formname . '_categories'])){
			$wof->Category = path_to_idA(we_base_request::_(we_base_request::WEFILELISTA, 'we_ui_' . $formname . '_categories'), CATEGORY_TABLE);
		}
		if(isset($_REQUEST['we_ui_' . $formname . '_Category'])){
			$_REQUEST['we_ui_' . $formname . '_Category'] = (is_array($_REQUEST['we_ui_' . $formname . '_Category']) ?
				implode(',', $_REQUEST['we_ui_' . $formname . '_Category']) :
				implode(',', array_filter(explode(',', (string) $_REQUEST['we_ui_' . $formname . '_Category']))));
		}
		foreach($wof->persistent_slots as $slotname => $type){
			if(we_base_request::string('we_ui_' . $formname . '_' . $slotname, we_base_request::NOT_VALID_S) !== we_base_request::NOT_VALID_S){
				$v = we_base_request::_($type, 'we_ui_' . $formname . '_' . $slotname);
				$ret = is_string($v) ? $wof->convertElemFromRequest('', $v, $slotname) : $v;
				$wof->{$slotname} = $ret;
			}
		}

		we_document_image::checkAndPrepare($formname, we_tag_write::$objElements[$formname]);
		we_document_video::checkAndPrepare($formname, we_tag_write::$objElements[$formname]);
		we_document_audio::checkAndPrepare($formname, we_tag_write::$objElements[$formname]);
		we_document_other::checkAndPrepare($formname, we_tag_write::$objElements[$formname]);

		if($session){
			$wof->saveInSession('we_object_session' . $formname);
		}
		return $wof;
	}

	function makeSameNew(array $keep = []): void{
		$this->DefaultInit = true;
		parent::makeSameNew(array_merge($keep, ['Category', 'TableID', 'rootDirID', 'RootDirPath', 'Workspaces', 'IsSearchable', 'Url', 'TriggerID']));
		$this->objectFileInit(true);
		$this->DefaultInit = false;
	}

	public function we_rewrite(): bool{
		$this->setLanguage();
		$this->setUrl();

		return ($this->DB_WE->query('UPDATE ' . $this->DB_WE->escape($this->Table) . ' SET Url="' . $this->DB_WE->escape($this->Url) . '" WHERE ID=' . intval($this->ID))) && parent::we_rewrite();
	}

	private function getObjectRootPathOfObjectWorkspace(): int{
		$rootID = fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE TableID=' . $this->TableID . ' AND IsClassFolder=1');
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return $rootID;
		}
		$ws = implode(',', we_users_util::get_ws(OBJECT_FILES_TABLE));
		if(!$ws){
			return $rootID;
		}
		$lastPath = '';
		$lastLen = PHP_INT_MAX;
		$all = $this->DB_WE->getAllFirstq('SELECT ID,Path FROM ' . OBJECT_FILES_TABLE . ' WHERE IsFolder=1 AND TableID=' . $this->TableID . ' AND ID IN(' . $ws . ') ORDER BY LENGTH(Path)', false);
		'@phan-var array<int,string> $all';
		foreach($all as $id => $path){
			if(count($all) === 1){
				return $id;
			}
			if($lastPath){
				$len = we_base_util::getMaxLenCompare($lastPath, $path . '/');
				if($len < $lastLen){
					$lastLen = $len;
					//$lastID = $db->f('ID');
					$lastPath = substr($lastPath, 0, $lastLen);
				}
			} else {
				$lastPath = $path . '/';
				//$lastID = $db->f('ID');
				$lastLen = strlen($lastPath);
			}
		}
		if(!$lastPath){
			return $rootID;
		}
		//find last /, so we have the dir on top of all ws
		$lastPath = substr($lastPath, 0, strrpos($lastPath, '/') ?: 0);

		if($lastPath === '/'){
			return $rootID;
		}
		return fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE IsFolder=1 AND TableID=' . $this->TableID . ' AND Path="' . $this->DB_WE->escape($lastPath) . '"');
	}

	protected function formCopyDocument(): string{
		$idname = 'we_' . $this->Name . '_CopyID';
		$rootDirId = $this->getObjectRootPathOfObjectWorkspace();
		$but = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.forms[0].elements['" . $idname . "'].value,table:'" . $this->Table . "',targetID:'" . $idname . "',jsCmd:'copyDocumentSelect',rootDirID:'" . $rootDirId . "',filter:'" . $this->ContentType . "'});");
		return we_html_element::hidden($idname, $this->CopyID) . $but;
	}

	function copyDoc(int $id): bool{
		if(!$id){
			return false;
		}

		$doc = new we_contents_objectFile();
		$doc->InitByID($id, '', self::LOAD_TEMP_DB);
		$doc->setRootDirID(true);
		if($this->ID == 0){
			foreach(array_keys($this->persistent_slots) as $name){
				$this->{$name} = $doc->{$name} ?? '';
			}
			$this->Creation_Date = new DateTime();
			$this->CreatorID = we_users_user::getID();
			$this->DefaultInit = true;
			$this->rootDirID = $doc->rootDirID;
			$this->RootDirPath = $doc->RootDirPath;
			$this->ID = 0;
			$this->OldPath = '';
			$this->Publish_Date = null;
			$this->Filename .= '_copy';
			$this->Path = $this->getPath();
			$this->OldPath = $this->Path;
		}
		$this->elements = $doc->elements;
		foreach(array_keys($this->elements) as $n){
			$this->elements[$n]['cid'] = 0;
		}
		$this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
		$this->Category = $doc->Category;
		$this->documentCustomerFilter = $doc->documentCustomerFilter;
		return true;
	}

	function restoreWorkspaces(): void{
		if(!$this->TableID){ // WORKARROUND for bug 4631
			$acs = we_users_util::getAllowedClasses();
			$this->TableID = $acs ? $acs[0] : 0;
			//$ac = implode(',', $acs);
		}
		$foo = getHash('SELECT Workspaces,DefaultWorkspaces FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($this->TableID));
		$defwsCSVArray = $foo ? we_base_request::filterIntList($foo['DefaultWorkspaces']) : [];
		$owsCSVArray = $foo ? we_base_request::filterIntList($foo['Workspaces']) : [];
		$this->Workspaces = [];

// loop throgh all default workspaces
		foreach($defwsCSVArray as $defWs){
// loop through each object workspace
			foreach($owsCSVArray as $ows){
				if(we_users_util::in_workspace((int) $defWs, [$ows], FILE_TABLE)){ // if default workspace is within object workspace
					$this->Workspaces[] = $defWs;
				}
			}
		}
	}

	function setRootDirID(bool $doit = false): void{
		if($this->TableID && ($this->InWebEdition || $doit)){
			['Path' => $this->RootDirPath, 'ID' => $this->rootDirID] = getHash('SELECT o.Path,`of`.ID FROM ' . OBJECT_FILES_TABLE . ' `of` JOIN ' . OBJECT_TABLE . ' o ON o.ID=`of`.TableID WHERE `of`.IsClassFolder=1 AND `of`.ParentID=0 AND o.ID=' . intval($this->TableID));
		}
	}

	function resetParentID(): void{
		if($this->ParentPath === '/' || (!str_starts_with($this->ParentPath . '/', $this->RootDirPath . '/'))){
			$this->setParentID($this->rootDirID);
		}
// adjust to bug #376 regarding workspace
		$workspaceRootDirId = $this->getObjectRootPathOfObjectWorkspace();
		$this->ParentPath = id2path($workspaceRootDirId, OBJECT_FILES_TABLE);
		$this->ParentID = $workspaceRootDirId;
	}

	function restoreDefaults(bool $makeSameNewFlag = false): void{
		$this->DefaultInit = true;
		if(!$makeSameNewFlag){
			$this->resetParentID();
		}
		$this->Owners = [];
		$this->OwnersReadOnly = [];
		$this->RestrictOwners = false;
		$this->Category = [];
		$this->Filename = '';
		$this->IsSearchable = true;
		$this->restoreWorkspaces();
		$this->elements = [];
		$hash = getHash('SELECT Owners,OwnersReadOnly,RestrictOwners,DefaultCategory,DefaultText,DefaultValues,DefaultTriggerID FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($this->TableID), $this->DB_WE);
		if($hash){
// fix - the class access permissions should not be applied
			$this->Category = we_base_request::filterIntList($hash['DefaultCategory']);
			if($hash['DefaultText']){
				$text = $hash['DefaultText'];
				$regs = [];
				if(preg_match('/%unique([^%]*)%/', $text, $regs)){
					$anz = ($regs[1] ? intval($regs[1]) : 16);
					$unique = substr(md5(uniqid(__FUNCTION__, true)), 0, min($anz, 32));
					$text = preg_replace('/%unique[^%]*%/', $unique, $text);
				}
				if(str_contains($text, '%ID%')){
//FIXME: this is NOT safe!!! Insert entry, and update afterwards
					$id = 1 + fI('SELECT max(ID) FROM ' . OBJECT_FILES_TABLE);
					$text = str_replace('%ID%', (string) $id, $text);
				}
				$this->Filename = strtr($text, ['%d%' => date('d'),
					'%j%' => date('j'),
					'%m%' => date('m'),
					'%y%' => date('y'),
					'%Y%' => date('Y'),
					'%n%' => date('n'),
					'%h%' => date('H'),
					'%H%' => date('H'),
					'%g%' => date('G'),
					'%G%' => date('G'),
				]);
			}

			if($hash['DefaultValues']){
				$vals = we_unserialize($hash['DefaultValues']);
				if(isset($vals['WE_CSS_FOR_CLASS'])){
					$this->CSS = is_array($vals['WE_CSS_FOR_CLASS']) ? $vals['WE_CSS_FOR_CLASS'] : we_base_request::filterIntList($vals['WE_CSS_FOR_CLASS']);
				}
				if(is_array($vals)){
					foreach($vals as $name => $field){
						'@phan-var string $name';
						if(is_array($field)){
							$foo = explode('_', $name);
							if(in_array(($type = $foo[0]), array_keys(self::DEFAULTS_VALUES))){
								unset($foo[0]);
								$name = implode('_', $foo);
								$n = ($type == self::TYPE_OBJECT ? we_listview_base::OBJECT_FIELD_PREFIX . $name : ($name ?? ''));

								$defaultField = self::DEFAULTS_VALUES[$type]['defField'];
								$defaultValue = $field[$defaultField] ?? self::DEFAULTS_VALUES[$type]['defaults'][$defaultField][0];

								if($type === self::TYPE_SELECT){
									$defaultField = we_contents_objectBase::DEFAULT_S;
									$defaultValue = $vals[self::TYPE_SELECT . '_' . $name]['selectKeys'][(int) $defaultValue] ?? '';
								}

								switch($defaultField){
									case we_contents_objectBase::DEFAULT_B:
										if($type === self::TYPE_DATE){ // FIXME: process exceptions in switch($type)
											$this->setElementI($n, $defaultValue ? time() : 0);
											break;
										}
										$this->setElementB($n, (bool) $defaultValue);
										break;
									case we_contents_objectBase::DEFAULT_I:
										$this->setElementI($n, (int) $defaultValue);
										break;
									case we_contents_objectBase::DEFAULT_F:
										$this->setElementF($n, (float) $defaultValue);
										break;
									case we_contents_objectBase::DEFAULT_HTML:
									case we_contents_objectBase::DEFAULT_S:
										$this->setElementS($n, (string) $defaultValue);
										break;
									case we_contents_objectBase::DEFAULT_AI:
									case we_contents_objectBase::DEFAULT_AS:
										// FIXME: multiobject value is still saved as csv on OBJECT_FILES_TABLE, while default is AI!
										if($type === self::TYPE_MULTIOBJECT){ // FIXME: process exceptions in switch($type)
											$this->setElementS($n, implode(',', $defaultValue));
											break;
										}
										$this->setElementA($n, (array) $defaultValue);
										break;
									case we_contents_objectBase::DEFAULT_HREF:
									case we_contents_objectBase::DEFAULT_LINK:
										$this->setElementA($n, (array) $defaultValue, SERIALIZE_JSON);
										break;
								}
							}
						} else {
							switch($name){
								// FIXME: make these defaults generic too
								case 'WE_CSS_FOR_CLASS':
									$this->setElementA($name, $field);
									break;
								case 'WorkspaceFlag':
									$this->setElementB($name, (bool) $field);
									break;
								default:
									$this->setElementS($name, (string) $field);
							}
						}
					}
				}
			}
		}
		$this->setTypeAndLength();
	}

	protected function check_requiredFields(): string{
		foreach($this->DefArray as $n => $v){
			if(!is_array($v) || empty($v['required'])){
				continue;
			}
			[$type, $name] = explode('_', (string) $n, 2);
			switch($type){
				case self::TYPE_MULTIOBJECT://FIXME: why is this still serialized?!
					$temp = we_unserialize($this->getElementS($name)) ?? [];
					$array = array_filter($temp['objects'] ?? $temp);
					if(empty($array)){
						return $name;
					}
					continue 2;
				case self::TYPE_OBJECT:
					$val = $this->getFieldValue($name, $type);
					if($val == '0'){
						return fS('SELECT Text FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($name));
					}
					continue 2;
				case self::TYPE_CHECKBOX:
				case self::TYPE_IMG:
					$val = $this->getFieldValue($name, $type);
					if(empty($val)){
						return $name;
					}
					continue 2;
				default:
					$val = $this->getFieldValue($name, $type);
					if(!$val){
						return $name;
					}
			}
		}
		return '';
	}

	protected function isVariantNamesValid(): bool{
		$variationFields = we_base_variants::getAllVariationFields($this);

		if(!empty($variationFields)){
			$i = 0;
			while($this->issetElement(we_base_variants::PREFIX . $i)){
				if(!trim($this->getElementS(we_base_variants::PREFIX . $i++))){
					return false;
				}
			}
		}

		return true;
	}

	/* must be called from the editor-script. Returns a filename which has to be included from the global-Script */

	function editor(we_base_jsCmd $jsCmd = null): we_editor_base{
		switch($this->EditPageNr){
			default:
				$_SESSION['weS']['EditPageNr'] = $this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
			case we_base_constants::WE_EDITPAGE_PROPERTIES:
			case we_base_constants::WE_EDITPAGE_WORKSPACE:
				return new we_editor_properties($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_INFO:
				return new we_editor_info_objectFile($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_CONTENT:
				return new we_editor_contentobjectFile($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_PREVIEW:
				return new we_editor_showObject($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_SCHEDULER:
				return new we_editor_scheduler($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_VARIANTS:
				return new we_editor_variants($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_WEBUSER:
				return new we_editor_weDocumentCustomerFilter($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_VERSIONS:
				return new we_editor_versions($this, $jsCmd);
		}
	}

	public function formPath(bool $disablePath = false, bool $notSetHot = false, string $extra = ''): string{
		$rootDirId = $this->getObjectRootPathOfObjectWorkspace();
		if(!$this->ParentID){
			$this->ParentID = $rootDirId;
			$this->ParentPath = id2path($rootDirId, OBJECT_FILES_TABLE);
		}
		$this->setUrl();
		return
			$this->formInput('', 'Filename', g_l('modules_object', '[objectname]'), 0, 255, ['onchange' => "pathOfDocumentChanged(true);"]) . we_html_element::br() .
			$this->formDirChooser($rootDirId) . we_html_element::br() .
			$this->formIsSearchable() . we_html_element::br() .
			g_l('modules_object', '[seourl]') . ': ' . $this->Url . we_html_element::br() .
			$this->formTriggerDocument($this->TriggerID);
	}

	protected function formIsSearchable(): string{
		return we_html_forms::checkboxWithHidden($this->IsSearchable, 'we_' . $this->Name . '_IsSearchable', g_l('weClass', '[IsSearchable]'), false, 'defaultfont', '_EditorFrame.setEditorIsHot(true);');
	}

	public function formClass(): string{
		return ($this->ID ?
			'<span class="defaultfont">' . $this->classData['Filename'] . '</span>' :
			$this->formSelectFromDB('TableID', OBJECT_TABLE, 'ID', 'Text', '', 'IsFolder=0' . ($this->AllowedClasses ? ' AND ID IN(' . implode(',', $this->AllowedClasses) . ')' : '') . ' ORDER BY Path ', 1, [
				$this->TableID], false, [
				'onchange' => "if(_EditorFrame.getEditorDocumentId() != 0){we_cmd('reload_editpage');}else{we_cmd('restore_defaults');};_EditorFrame.setEditorIsHot(true);"]));
	}

	public function formClassId(): string{
		return '<span class="defaultfont">' . $this->TableID . '</span>';
	}

	/**
	 *
	 * @param int $tableID
	 * @param bool $contentOnly
	 * @param we_database_base $db
	 * @return array<int,array{table:string,name:string,type:string,len:int,flags:int}>
	 */
	static function getSortedTableInfo(int $tableID, bool $contentOnly, we_database_base $db/* , bool $checkVariants = false */): array{
		if(!$tableID){
			return [];
		}

		$tableInfo = $db->metadata(OBJECT_X_TABLE . $tableID);
		'@phan-var array<string,array{table:string,name:string,type:string,len:int,flags:int}> $tableInfo';
		$tableInfo2 = [];
		foreach($tableInfo as $arr){
			$names = explode('_', $arr['name']);
			switch($names[0]){
				case 'variant':
					if($names[1] === we_base_variants::ELEMENT_NAME){
						break;
					}
				//no break
				case self::TYPE_INPUT:
				case self::TYPE_TEXT:
				case self::TYPE_INT:
				case self::TYPE_FLOAT:
				case self::TYPE_DATE:
				case self::TYPE_IMG:
				case self::TYPE_OBJECT:
				case self::TYPE_MULTIOBJECT:
				case self::TYPE_SELECT:
				/* if($checkVariants){
				  $variantdata = $arr;
				  } */
				default:
					$tableInfo2[] = $arr;
			}
		}

		if(!$contentOnly){
			return $tableInfo2;
		}

		/* if($checkVariants && isset($variantdata) && is_array($variantdata)){
		  $tableInfo2[] = $variantdata;
		  } */

		return $tableInfo2;
	}

	/**
	 *
	 * @param we_base_jsCmd $jsCmd
	 * @param string $name
	 * @param string $type
	 * @param array<string,mixed> $attribs default values from class
	 * @üaram array{table:string,name:string,type:string,len:int,flags:int,default:string,pos:int,type_short:string,flags_name:string} $dbFields
	 * @param bool $editable
	 * @return string
	 */
	function getFieldHTML(we_base_jsCmd $jsCmd, string $name, string $type, array $attribs, array $dbFields, bool $editable = true): string{
		switch($type){
			default:
			case self::TYPE_INPUT:
				return $this->getInputFieldHTML($name/* , $dbFields */, $editable);
			case self::TYPE_COUNTRY:
				return $this->getCountryFieldHTML($name, $editable);
			case self::TYPE_LANGUAGE:
				return $this->getLanguageFieldHTML($name, $editable);
			case self::TYPE_HREF:
				return $this->getHrefFieldHTML($name, $attribs, $editable);
			case self::TYPE_LINK:
				return $this->getLinkFieldHTML($name, $editable);
			case self::TYPE_TEXT:
				return $this->getTextareaHTML($name, $attribs, $editable);
			case self::TYPE_IMG:
				return $this->getImageHTML($name, $editable);
			case self::TYPE_BINARY:
				return $this->getBinaryElementHTML($name, $editable, new we_document_other(), self::TYPE_BINARY, [we_base_ContentTypes::APPLICATION, we_base_ContentTypes::PDF]);
			case self::TYPE_VIDEO:
				return $this->getBinaryElementHTML($name, $editable, new we_document_video(), self::TYPE_VIDEO, [we_base_ContentTypes::VIDEO]);
			case self::TYPE_AUDIO:
				return $this->getBinaryElementHTML($name, $editable, new we_document_audio(), self::TYPE_AUDIO, [we_base_ContentTypes::AUDIO]);
			case self::TYPE_DATE:
				return $this->getDateFieldHTML($name, $editable);
			case self::TYPE_CHECKBOX:
				return $this->getCheckboxFieldHTML($name, $editable);
			case self::TYPE_INT:
				return $this->getIntFieldHTML($name, $dbFields, $editable);
			case self::TYPE_FLOAT:
				return $this->getFloatFieldHTML($name, $dbFields, $editable);
			case self::TYPE_OBJECT:
				return $this->getObjectFieldHTML($jsCmd, $name, $editable);
			case self::TYPE_MULTIOBJECT:
				return $this->getMultiObjectFieldHTML($jsCmd, $name, $editable);
			case self::TYPE_COLLECTION:
				return $this->getCollectionFieldHTML($name, $editable);
			case self::TYPE_SELECT:
				return $this->getSelectFieldHTML($name, $editable);
			case self::TYPE_SHOPVAT:
				return $this->getShopVatFieldHtml($name, $attribs, $editable);
			case self::TYPE_SHOPCATEGORY:
				return $this->getShopCategoryFieldHtml($name, $attribs, $editable);
			case 'OF':
				return '';
		}
	}

	public function getElementByType(string $name, string $type): string{
		switch($type){
			case self::TYPE_TEXT:
			case self::TYPE_INPUT:
			case self::TYPE_COUNTRY:
			case self::TYPE_LANGUAGE:
				return $this->getElementS($name);
			case self::TYPE_HREF:
				return parent::getHrefByArray($this->getElementA($name));
			case self::TYPE_LINK:
				return $this->getLinkFieldHTML($name, false);
			case self::TYPE_DATE:
				return $this->getElementS($name);
			case self::TYPE_FLOAT:
			case self::TYPE_INT:
				return strlen($this->getElementS($name)) ? $this->getElementS($name) : '';
			case self::TYPE_SELECT:
				return $this->getElementS($name);
			default:
				return $this->getElementS($name);
		}
	}

	function getFieldsHTMLString(we_base_jsCmd $jsCmd, bool $editable): string{
		$ret = '';
		foreach($this->getFieldsHTML($jsCmd, $editable)->getContent() as $content){
			assert($content instanceof we_html_multiIconBoxContentLine);
			$ret = ($content->getHtml() ? $content->getHtml() . we_html_element::br(2) : '');
		}
		return $ret;
	}

	function getFieldsHTML(we_base_jsCmd $jsCmd, bool $editable): we_html_multiIconBoxContent{
		$parts = new we_html_multiIconBoxContent();
		if(empty($this->TableID)){
			return $parts;
		}
		$dv = $this->getDefaultValueArray();

		$tableInfo_sorted = self::getSortedTableInfo($this->TableID, true, $this->DB_WE);
		$fields = $regs = [];
		foreach($tableInfo_sorted as $cur){
			if(in_array($cur['name'], ['OF_ID', 'OF_DocumentTable'], true)){
				continue;
			}
			if(preg_match('/(.+?)_(.*)/', $cur['name'], $regs)){
				$fields[] = ['name' => $regs[2], 'type' => $regs[1], 'db' => $cur];
			}
		}

		foreach($fields as $field){
			$realName = $field['type'] . '_' . $field['name'];
			$edMerk = $editable;
			if(!((!isset($dv[$realName]) || (isset($dv[$realName]) && empty($dv[$realName]['users']))) || we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) || we_users_util::isUserInUsers(we_users_user::getID(), we_base_request::filterIntList($dv[$realName]['users'])))){
				$editable = false;
			}
			$c2 = $this->getFieldHTML($jsCmd, $field['name'], $field['type'], ($dv[$realName] ?? []), $field['db'], $editable);
			$parts->add((new we_html_multiIconBoxContentLine('', $c2))->id($realName));
			$editable = $edMerk;
		}
		return $parts;
	}

	private function formatDescription(string $name): string{
		[$type, $tmp] = explode('_', $name, 2);
		$realname = $type . '_' . we_base_variants::stripVariantPrefix($tmp);

		if(empty($this->DefArray[$realname]['editdescription'])){
			return '';
		}

		$desc = $this->DefArray[$realname]['editdescription'];

		return we_html_element::div(['class' => 'weObjectDescription'], (str_contains($desc, '<script') ?
				$desc :
				str_replace("\n", we_html_element::br(), $desc)
		));
	}

	private function getHeadline(string $type, string $name): string{
		$realname = we_base_variants::stripVariantPrefix($name);

		return we_html_element::div(['class' => 'weObjectHeadline' . (empty($this->DefArray[$type . '_' . $realname]['required']) ? '' : ' required')], $realname);
	}

	private function getSelectFieldHTML(string $name, bool $editable = true): string{
		$realName = we_base_variants::stripVariantPrefix($name);
		$vals = empty($this->DefArray[self::TYPE_SELECT . '_' . $realName]['selectKeys']) ? [] :
			array_combine($this->DefArray[self::TYPE_SELECT . '_' . $realName]['selectKeys'], $this->DefArray[self::TYPE_SELECT . '_' . $realName]['selectValues']);

		$element = $this->getElementS($name);
		if(!$editable){
			return $this->getPreviewView($name, $vals[$element] ?? '');
		}
		$isRequired = !empty($this->DefArray[self::TYPE_SELECT . '_' . $name]['required']);
		if($isRequired){
			$vals = ['' => '---'] + $vals;
		}

		return $this->getInputContainer($this->getHeadline(self::TYPE_SELECT, $name), $this->formatDescription(self::TYPE_SELECT . '_' . $name) . $this->formSelectFromArray(self::TYPE_SELECT, $name, $vals, '', 1, false, [
					'onchange' => '_EditorFrame.setEditorIsHot(true);',
					($isRequired ? 'required' : '') => 'required'
				]), 'selectElementField');
	}

	private function getObjectFieldHTML(we_base_jsCmd $jsCmd, string $ObjectID, bool $editable = true): string{
		//FIXME: this is bad matching text instead of id's
		$foo = getHash('SELECT `of`.Text,`of`.ID FROM ' . OBJECT_FILES_TABLE . ' `of` WHERE `of`.IsClassFolder=1 AND `of`.TableID=' . $ObjectID);
		$name = $foo['Text'] ?? '';
		$pid = empty($foo['ID']) ? 0 : $foo['ID'];

		$textname = 'we_' . $this->Name . '_txt[' . we_listview_base::OBJECT_FIELD_PREFIX . $ObjectID . '_path]';
		$idname = 'we_' . $this->Name . '_' . self::TYPE_OBJECT . '[' . we_listview_base::OBJECT_FIELD_PREFIX . $ObjectID . '#' . self::ETYPE_NUMBER . ']';
		$myid = $this->getElementI(we_listview_base::OBJECT_FIELD_PREFIX . $ObjectID);
		$spath = $this->getElementS(we_listview_base::OBJECT_FIELD_PREFIX . $ObjectID . '_path');
		['Path' => $path, 'IsPublished' => $npubl] = getHash('SELECT Path,IsPublished FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $myid) ?: [
			'Path' => $spath, 'IsPublished' => 1];
		if($path === ''){
			$myid = 0;
			$npubl = true;
		}
		$ob = new we_contents_objectFile($myid);
		if($myid){
			$ob->DefArray = $ob->getDefaultValueArray();
		}

		if(!$editable){
			return (new we_html_multiIconBox((new we_html_multiIconBoxContent())
						->add((new we_html_multiIconBoxContentLine('<span style="cursor: pointer;" class="weObjectHeadline">' . ($ob->getText() ?: $name) . '</span>' . ((bool) $npubl ? '' : ' <span class="weObjectHeadline" style="color:red">' . g_l('modules_object', '[not_published]') . '</span>'), ($myid ? $ob->getFieldsHTMLString($jsCmd, false) : '')))->rightHead()->fold())
				))->get();
		}

		$cmd = 'we_object_file_objectlinkChange,' . we_base_WE::$transaction . ',' . we_contents_object::QUERY_PREFIX . $ObjectID;

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId($textname . we_base_file::getUniqueId(), '/' . $name)
			->setContentType(array_merge(we_base_ContentTypes::FOLDER_TYPES, [we_base_ContentTypes::OBJECT_FILE]))
			->setInput($textname, $path)
			->setMaxResults(10)
			->setResult($idname, (string) $myid)
			->setSelector(we_gui_suggest::DocSelector)
			->setTable(OBJECT_FILES_TABLE)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $idname . "'].value,table:WE().consts.tables.OBJECT_FILES_TABLE,targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'" . $cmd . "',rootDirID:'" . $pid . "',filter:WE().consts.contentTypes.OBJECT_FILE,open_doc:" . (we_base_permission::hasPerm([
						we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS]) ? 0 : 1) . '})'))
			->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, ($myid ? "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.OBJECT_FILES_TABLE,'" . $myid . "',WE().consts.contentTypes.OBJECT_FILE);" : ''), '', '', ($myid ? false : true)))
			->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value=0;document.we_form.elements['" . $textname . "'].value='';top.we_cmd('we_object_file_reloadEntry','" . we_base_WE::$transaction . "','" . we_contents_object::QUERY_PREFIX . $ObjectID . "')"))
			->setWidth(443);

		//FIXME: THIS IS REALLY odd
		return we_html_element::div(['class' => 'weObjectHeadline'], $name .
				(empty($this->DefArray[we_contents_object::QUERY_PREFIX . $ObjectID]['required']) ? '' : '*') .
				((bool) $npubl ? '' : we_html_element::span(['style' => 'color:red'], ' ' . g_l('modules_object', '[not_published]')))) .
			we_html_element::div(['class' => 'weObjectContent'], $this->formatDescription(we_contents_object::QUERY_PREFIX . $ObjectID) .
				$weSuggest->getHTML()) .
			(we_editor_functions::isSeeMode() ?
			(new we_html_multiIconBox((new we_html_multiIconBoxContent())->add((new we_html_multiIconBoxContentLine('', ($myid ? $ob->getFieldsHTMLString($jsCmd, false) : '')))->fold())))->get() : '');
	}

	private function getCollectionFieldHTML(string $name, bool $editable = true): string{
		$collectionID = $this->getElementI($name);
		$path = fS('SELECT Path FROM ' . VFILE_TABLE . ' WHERE ID=' . $collectionID . ' AND IsFolder=0');
		$collectionID = ($path ? $collectionID : 0);

		$textname = 'we_' . $this->Name . '_txt[' . $name . '_path]';
		$idname = 'we_' . $this->Name . '_' . self::TYPE_COLLECTION . '[' . $name . '#' . self::ETYPE_NUMBER . ']';

		if(!$editable){
			return $this->getPreviewView($name, $path . ' (ID: ' . $collectionID . ')');
		}

		$cmd = 'we_object_file_objectlinkChange,' . we_base_WE::$transaction . ',' . we_contents_object::QUERY_PREFIX . $name;

		$buttons = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $idname . "'].value,table:WE().consts.tables.VFILE_TABLE,targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'" . $cmd . "',rootDirID: 0,filter:WE().consts.contentTypes.COLLECTION,open_doc:" . (we_base_permission::hasPerm([
					we_base_perm::WORK_COLLECTIONS_CAN_SEE]) ? 0 : 1) . '})') .
			(we_base_permission::hasPerm([we_base_perm::WORK_COLLECTION_NEW]) ?
			we_html_button::create_button(we_html_button::ADD_COLLECTION, "javascript:we_cmd('edit_new_collection','write_back_to_opener," . $idname . "," . $textname . "','',-1,WE().consts.tables.VFILE_TABLE);", '', '', false, false) : ''
			) .
			we_html_button::create_button(we_html_button::OPEN_TO_EDIT, ("javascript:var cid=document.we_form.elements['" . $idname . "'].value;if(cid != '0'){WE().layout.weEditorFrameController.openDocument(WE().consts.tables.VFILE_TABLE,cid,WE().consts.contentTypes.COLLECTION);}"), '', '', ($collectionID ? false : true)) . // FIXME: set disabled=true|false on select
			we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value=0;document.we_form.elements['" . $textname . "'].value='';top.we_cmd('we_object_file_reloadEntry','" . we_base_WE::$transaction . "','" . we_contents_object::QUERY_PREFIX . $collectionID . "')");

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setNoAutoInit(true) // autosuggest is deactivated
			->setAcId($textname)
			->setContentType([we_base_ContentTypes::COLLECTION])
			->setInput($textname, $path)
			->setMaxResults(10)
			->setResult($idname, (string) $collectionID)
			->setSelector(we_gui_suggest::DocSelector)
			->setTable(VFILE_TABLE)
			->setWidth(396);

		return $this->getInputContainer($this->getHeadline('collection', $name), $this->formatDescription('collection_' . $name) . we_html_tools::formElementTable('', $weSuggest->getHTML(), $buttons), 'collectionField');
	}

	private function getMultiObjectFieldHTML(we_base_jsCmd $jsCmd, string $name, bool $editable = true): string{
		$temp = we_unserialize($this->getElementS($name));
		$objects = array_merge($temp['objects'] ?? $temp); // reindex array
		$max = empty($this->DefArray[self::TYPE_MULTIOBJECT . '_' . $name]['max']) ? 0 : intval($this->DefArray[self::TYPE_MULTIOBJECT . '_' . $name]['max']);
		$show = min(($max ?: PHP_INT_MAX), count($objects));
		$isSEEM = we_editor_functions::isSeeMode();

		if(!$show && !$editable){
			return $this->getPreviewView($name, '');
		}

		$classid = $this->DefArray[self::TYPE_MULTIOBJECT . '_' . $name]['tableid'];

		if($editable){
			['ID' => $rootDirID, 'Path' => $rootDirPath] = getHash('SELECT `of`.ID,`of`.Path FROM ' . OBJECT_FILES_TABLE . ' `of` WHERE `of`.IsClassFolder=1 AND `of`.TableID=' . intval($classid)) ?: [
				'ID' => 0, 'Path' => '/'];

			$weSuggest = we_gui_suggest::getInstance();
			$weSuggest->setContentType([we_base_ContentTypes::FOLDER_OBJECTFILE, we_base_ContentTypes::OBJECT_FILE])
				->setMaxResults(10)
				->setSelector(we_gui_suggest::DocSelector)
				->setTable(OBJECT_FILES_TABLE);
//multiobjects don't care for the order, so don't show buttons for ordering here
			/* $upbutDis = (new we_html_button(we_html_button::DIRUP))->disable(true)->get();
			  $downbutDis = (new we_html_button(we_html_button::DIRDOWN))->disable(true)->get(); */
			$plusbutDis = (new we_html_button(we_html_button::LIST_ADD))->disable(true)->get();
			$parts = new we_html_multiIconBoxContent();
			$paths = id2pathA($objects, OBJECT_FILES_TABLE);
			unset($paths[0]);
			for($f = 0; $f < $show; $f++){
				$textname = 'we_' . $this->Name . '_txt[' . $name . '_path' . $f . ']';
				$idname = 'we_' . $this->Name . '_' . self::TYPE_MULTIOBJECT . '[' . $name . '_default' . $f . ']';

				//$path = $this->getElementS(we_listview_base::OBJECT_FIELD_PREFIX . $name . '_path');
				$myid = intval($objects[$f]);
				$path = $paths[$myid] ?? $rootDirPath;

				$weSuggest->setAcId($textname . we_base_file::getUniqueId(), $rootDirPath)
					->setInput($textname, $path)
					->setResult($idname, (string) $myid)
					->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $idname . "'].value,table:WE().consts.tables.OBJECT_FILES_TABLE,targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'" . 'fieldMultiobject_selectMultiobject_callback,' . ($isSEEM ? 'isSEEM' : '') . ',' . self::TYPE_MULTIOBJECT . '_' . $name . "',rootDirID:'" . $rootDirID . "',filter:WE().consts.contentTypes.OBJECT_FILE,open_doc:" . (we_base_permission::hasPerm([
								we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS]) ? 0 : 1) . "})"))
					->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('we_object_file_multiobjectDelete','" . we_base_WE::$transaction . "','" . self::TYPE_MULTIOBJECT . '_' . $name . "','" . ($f) . "')"))
					->setCreateButton((empty($max) || (count($objects) < $max) ?
							we_html_button::create_button(we_html_button::LIST_ADD, "javascript:we_cmd('we_object_file_multiobjectInsert','" . we_base_WE::$transaction . "','" . self::TYPE_MULTIOBJECT . '_' . $name . "','" . ($f) . "')") :
							$plusbutDis
					))
					->setOpenButton(($myid ?
							we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.OBJECT_FILES_TABLE," . intval($myid) . ",WE().consts.contentTypes.OBJECT_FILE);") :
							we_html_button::create_button(we_html_button::OPEN_TO_EDIT, '', '', '', true)
					))
				/* ->setAdditionalButton(($f > 0 ?
				  we_html_button::create_button(we_html_button::DIRUP, "javascript:we_cmd('object_up_multiobject_at_object','" . we_base_WE::$transaction . "','" . self::TYPE_MULTIOBJECT . '_' . $name . "','" . ($f) . "')") :
				  $upbutDis
				  ) .
				  ($f < count($objects) - 1 ?
				  we_html_button::create_button(we_html_button::DIRDOWN, "javascript:we_cmd('we_object_file_multiobjectDown','" . we_base_WE::$transaction . "','" . self::TYPE_MULTIOBJECT . '_' . $name . "','" . ($f) . "')") :
				  $downbutDis
				  )) */
				;

				$parts->add((new we_html_multiIconBoxContentLine('', $weSuggest->getHTML(false)))->noline()->class('multiobject'));

				if($isSEEM && $myid){
					$ob = new we_contents_objectFile($myid);
					$ob->DefArray = $ob->getDefaultValueArray();
					$parts->add((new we_html_multiIconBoxContentLine('', $ob->getFieldsHTMLString($jsCmd, false)))->fold()->noline());
				}
			}
			$weSuggest->reset();
			$this->setElementS($name, implode(',', $objects), self::TYPE_MULTIOBJECT);

			return $this->getInputContainer($this->getHeadline(self::TYPE_MULTIOBJECT, $name), $this->formatDescription(self::TYPE_MULTIOBJECT . '_' . $name) . (new we_html_multiIconBox($parts))->get() .
					(empty($max) || count($objects) < $max ?
					we_html_button::create_button(we_html_button::LIST_ADD, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('we_object_file_multiobjectInsert','" . we_base_WE::$transaction . "','" . self::TYPE_MULTIOBJECT . '_' . $name . "','" . (max(0, ($f - 1))) . "')") :
					we_html_button::create_button(we_html_button::LIST_ADD, '', '', '', true)), 'multiObjectField'
			);
		}

		$parts = new we_html_multiIconBoxContent();
		for($f = 0; $f < $show; $f++){
			if(!$objects[$f]){
				continue;
			}
			$ob = new we_contents_objectFile((int) $objects[$f]);
			$ob->DefArray = $ob->getDefaultValueArray();

			$parts->add((new we_html_multiIconBoxContentLine($ob->getText(), $ob->getFieldsHTMLString($jsCmd, false)))->fold());
		}

		$this->setElementS($name, implode(',', $objects), self::TYPE_MULTIOBJECT);

		return (new we_html_multiIconBox($parts))->get();
	}

	private function getInputContainer(string $headline, string $content, string $type, string $extra = ''): string{
		return $headline . we_html_element::div(['class' => 'weObjectContent ' . $type], $content) .
			($extra ?
			we_html_element::div(['class' => 'weObjectTypeInfo'], $extra) :
			'');
	}

	private function getShopVatFieldHtml(string $name, array $attribs, bool $editmode = true): string{
		if($editmode && we_modules_info::isActive(we_modules_info::SHOP)){

			$shopVats = we_shop_vats::getAllShopVATs();

			$values = [];
			foreach($shopVats as $shopVat){
				$values[$shopVat->id] = $shopVat->vat . '% - ' . $shopVat->getNaturalizedText() . ' (' . $shopVat->territory . ')';
			}

			$val = $this->getElementS($name) ?: $attribs['default'];

			return $this->getInputContainer($this->getHeadline('shopVat', 'shopvat'), $this->formatDescription('shopVat_shopVat') . we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_SHOPVAT . '[' . $name . ']', $values, 1, $val), 'shopVatField');
		}

		try{
			$weShopVat = we_shop_vats::getShopVATById($this->getElementI($name));
		} catch (we_exception_shop $e){
			try{
				$weShopVat = we_shop_vats::getStandardShopVat();
			} catch (we_exception_shop $e){
				t_e('Shop exception', $e);
				return '';
			}
		}

		return $this->getPreviewView($name, (string) $weShopVat->vat);
	}

	private function getShopCategoryFieldHtml(string $name, array $attribs, bool $editmode = true): string{
		if($editmode){
			$values = ['0' => ' '];
			if($attribs['shopcatLimitChoice']){
				$cats = we_category_category::we_getCatsFromIDs(we_base_request::filterIntList($attribs['default']), '', '', 'Path');
				foreach($cats as $cat){
					$values[] = [$cat->ID => $cat->Path];
				}
				$input = we_html_tools::select('dummy', $values, 1, 0, false, ['disabled' => 'disabled']) .
					we_html_element::hidden('we_' . $this->Name . '_' . self::TYPE_SHOPCATEGORY . ' [' . $name . ']', $attribs['default']);
			} else {
				foreach(we_shop_category::getShopCatsFromDir() as $cat){
					$values[$cat->ID] = $cat->Path;
				}
				$input = we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_SHOPCATEGORY . '[' . $name . ']', $values, 1, ($this->getElementS($name) ?: $attribs['default']));
			}

			return $this->getInputContainer($this->getHeadline('_shopCategory', '_shopcategory'), $this->formatDescription('_shopCategory__shopCategory') . $input, 'shopCategoryField');
		}
		try{
			$val = we_shop_category::getShopCatFieldByID($this->getElementI($name), $this->Category, $attribs['shopcatField'], ($attribs['shopcatShowPath'] != 'false'), $attribs['shopcatRootdir'], true);
			return $this->getPreviewView($name, $val);
		} catch (we_exception_shop $e){
			return $this->getPreviewView($name, $e->getMessage());
		}
	}

	private function getHrefFieldHTML(string $name, array $attribs, bool $editmode = true): string{
		$hrefArr = $this->getElementA($name);
		if(!$editmode){
			return $this->getPreviewView($name, parent::getHrefByArray($hrefArr));
		}

		$directory = !(isset($attribs['hrefdirectory']) && $attribs['hrefdirectory'] === 'true');
		$file = !(isset($attribs['hreffile']) && $attribs['hreffile'] === 'false');

		$attr = ' size="20" ';

		$int = (bool) ($hrefArr['int'] ?? false);
		$intID = (int) (!empty($hrefArr['intID']) ? $hrefArr['intID'] : 0);
		$intPath = $intID ? id2path($intID) : '';
		$extPath = $hrefArr['extPath'] ?? '';
		$int_elem_Name = 'we_' . $this->Name . '_' . self::TYPE_HREF . '[' . $name . we_base_link::MAGIC_INT_LINK . ']';
		$intPath_elem_Name = 'we_' . $this->Name . '_vars[' . $name . we_base_link::MAGIC_INT_LINK_PATH . ']';
		$intID_elem_Name = 'we_' . $this->Name . '_' . self::TYPE_HREF . '[' . $name . we_base_link::MAGIC_INT_LINK_ID . ']';
		$ext_elem_Name = 'we_' . $this->Name . '_' . self::TYPE_HREF . '[' . $name . we_base_link::MAGIC_INT_LINK_EXTPATH . ']';
		switch($attribs['hreftype'] ?? ''){
			case we_base_link::TYPE_INT:
				$out = self::hrefRow($intID_elem_Name, $intID, $intPath_elem_Name, $intPath, $attr, $int_elem_Name, false, true, $file, $directory);
				break;
			case we_base_link::TYPE_EXT:
				$out = self::hrefRow('', 0, $ext_elem_Name, $extPath, $attr, $int_elem_Name, false, true, $file, $directory);
				break;
			default:
				$out = self::hrefRow($intID_elem_Name, $intID, $intPath_elem_Name, $intPath, $attr, $int_elem_Name, true, $int, $file, $directory) .
					self::hrefRow('', 0, $ext_elem_Name, $extPath, $attr, $int_elem_Name, true, $int, $file, $directory);
		}
		return $this->getInputContainer($this->getHeadline('href', $name), $this->formatDescription('href_' . $name) . '<table>' . $out . '</table>', 'hrefField');
	}

	private function getLinkFieldHTML(string $name, bool $editmode = true): string{
		$link = $this->getElementA($name) ?: ['ctype' => 'text', 'type' => we_base_link::TYPE_EXT, 'href' => '#', 'text' => g_l('global', '[new_link]')];
		$img = new we_document_image();
		$startTag = $this->getLinkStartTag($link, [], $this->ParentID, $this->Path, $img);
		$content = parent::getLinkContent($link, $img) ?: g_l('global', '[new_link]');

		return $this->getInputContainer($this->getHeadline('link', $name), $this->formatDescription('link_' . $name) . ($startTag ? $startTag . $content . '</a>' : $content) . ($editmode ? (
				we_html_button::create_button(we_html_button::LINK_EDIT, "javascript:we_cmd('edit_link_at_object',{transaction:'" . we_base_WE::$transaction . "',identifier:'link_" . $name . "',fieldname:'" . $name . "'})") .
				we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('we_object_file_linkDelete','" . we_base_WE::$transaction . "', 'link_" . $name . "')")
				) : ''), 'linkField');
	}

	private function getPreviewView(string $name, string $content): string{
		return '<div class="weObjectHeadline">' . $name . '</div>' .
			($content !== '' ? '<div class="weObjectContent">' . $content . '</div>' : '');
	}

	/**
	 * @param string $name
	 * @param bool $editable
	 * @return string
	 */
	private function getInputFieldHTML(string $name/* , array $dbFields */, bool $editable = true): string{
		if(!$editable){
			return $this->getPreviewView($name, $this->getElementS($name));
		}
		$len = $this->getElementLen($name);
		$extra = g_l('modules_object', '[length]') . ': ' . $len;
		return $this->getInputContainer($this->getHeadline('input', $name), $this->formatDescription('input_' . $name) . we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_INPUT . '[' . $name . ']', $this->getElementS($name), $len, [
					'onchange' => "_EditorFrame.setEditorIsHot(true);"], 'text'), 'textField', $extra);
	}

	private function getCountryFieldHTML(string $name, bool $editable = true): string{
		$lang = explode('_', we_base_WE::$Language);
		$langs = we_base_country::getWELangs();
		$langcode = (string) (array_search($lang[0], $langs, true) ?: reset($langs));

		if(!$editable){
			return '<div class="weObjectHeadline">' . $name . '</div>' .
				($this->getElementS($name) != '--' || $this->getElementS($name) ? '<div class="defaultfont">' . we_base_country::getTranslation($this->getElementS($name), we_base_country::TERRITORY, $langcode) . '</div>' :
				'');
		}

		$countrycode = array_search($langcode, we_base_country::getWECountries(), true) ?: 'DE';
		$countryselect = new we_html_select(['name' => 'we_' . $this->Name . '_' . self::TYPE_LANGUAGE . '[' . $name . ']', 'style' => 'width:620px;', 'class' => 'wetextinput',
			'onchange' => "_EditorFrame.setEditorIsHot(true);"]);

		$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));

		foreach($topCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);
		$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
		foreach($shownCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);
		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $langcode . '_' . $countrycode . '.UTF-8');
		asort($topCountries, SORT_LOCALE_STRING);
		asort($shownCountries, SORT_LOCALE_STRING);
		if($oldLocale){
			setlocale(LC_ALL, $oldLocale);
		}
		if(WE_COUNTRIES_DEFAULT){
			$countryselect->addOption('--', WE_COUNTRIES_DEFAULT);
		}
		foreach($topCountries as $countrykey => $countryvalue){
			$countryselect->addOption((string) $countrykey, (string) $countryvalue);
		}

		if(!empty($topCountries) && !empty($shownCountries)){
			$countryselect->addOption('-', '----', ['disabled' => 'disabled']);
		}

		foreach($shownCountries as $countrykey => $countryvalue){
			$countryselect->addOption((string) $countrykey, (string) $countryvalue);
		}

		$countryselect->selectOption($this->getElementS($name));

		return $this->getInputContainer($this->getHeadline('country', $name), $this->formatDescription('country_' . $name) . $countryselect->getHtml(), 'countryField');
	}

	private function getLanguageFieldHTML(string $name, bool $editable = true): string{
		if(!$editable){
			$langcode = array_search(we_base_WE::$Language, we_base_country::getWELangs(), true) ?: '';
			return '<div class="weObjectPreviewHeadline">' . $name . '</div>' .
				($this->getElementS($name) != '--' || $this->getElementS($name) ? we_html_element::div([], we_base_country::getTranslation($this->getElementS($name), we_base_country::LANGUAGE, $langcode)) :
				'');
		}
		$frontendL = we_base_WE::$Langs['FrontendLanguages'];
		foreach($frontendL as &$lcvalue){
			$lccode = explode('_', $lcvalue);
			$lcvalue = $lccode[0];
		}
		$languageselect = new we_html_select(['name' => 'we_' . $this->Name . '_' . self::TYPE_LANGUAGE . '[' . $name . ']', 'style' => 'width:620px;', 'class' => 'wetextinput',
			'onchange' => "_EditorFrame.setEditorIsHot(true);"]);
		if(!$this->DefArray['language_' . $name]['required']){
			$languageselect->addOption('--', '');
		}

		foreach(we_base_country::g_l_a('languages', '') as $languagekey => $languagevalue){
			if(in_array($languagekey, $frontendL, true)){
				$languageselect->addOption((string) $languagekey, (string) $languagevalue);
			}
		}
		$languageselect->selectOption($this->getElementS($name));
		return $this->getInputContainer($this->getHeadline('language', $name), $this->formatDescription('language_' . $name) . $languageselect->getHtml(), 'languageField');
	}

	private function getCheckboxFieldHTML(string $name, bool $editable = true): string{
		if(!$editable){
			return $this->getPreviewView($name, g_l('global', ($this->getElementB($name) ? '[yes]' : '[no]')));
		}
		return $this->getInputContainer($this->getHeadline('checkbox', $name), $this->formatDescription('checkbox_' . $name) . we_html_forms::checkboxWithHidden($this->getElementB($name), 'we_' . $this->Name . '_' . self::TYPE_CHECKBOX . '[' . $name . '#' . self::ETYPE_NUMBER . ']', '', false, '', "_EditorFrame.setEditorIsHot(true);"), 'checkboxField');
	}

	/**
	 *
	 * @param string $name
	 * @param array{table:string,name:string,type:string,len:int,flags:int,default:string,pos:int,type_short:string,flags_name:string} $dbFields
	 * @param bool $editable
	 * @return string
	 */
	private function getIntFieldHTML(string $name, array $dbFields, bool $editable = true): string{
		if(!$editable){
			return $this->getPreviewView($name, (string) $this->getElementI($name));
		}
		$attr = [
			'onchange' => "_EditorFrame.setEditorIsHot(true);",
			'step' => '1',
		];
		switch($dbFields['type']){
			case we_database_base::TYPE_TINYINT:
				$attr['max'] = $dbFields['flags'] && MYSQLI_UNSIGNED_FLAG ? '255' : '127';
				break;
			case we_database_base::TYPE_SMALLINT:
				$attr['max'] = $dbFields['flags'] && MYSQLI_UNSIGNED_FLAG ? '65535' : '32767';
				break;
			case we_database_base::TYPE_MEDIUMINT:
				$attr['max'] = $dbFields['flags'] && MYSQLI_UNSIGNED_FLAG ? '16777255' : '8388607';
				break;
			case we_database_base::TYPE_INT:
				$attr['max'] = $dbFields['flags'] && MYSQLI_UNSIGNED_FLAG ? '4294967295' : '2147483647';
				break;
			case we_database_base::TYPE_BIGINT:
				$attr['max'] = $dbFields['flags'] && MYSQLI_UNSIGNED_FLAG ? '18446744073709551615' : '9223372036854775807';
				break;
		}
		$attr['min'] = $dbFields['flags'] && MYSQLI_UNSIGNED_FLAG ? '0' : '-' . ($attr['max']);
		$extra = g_l('modules_object', '[field][int]') . ' (' . $attr['min'] . '-' . $attr['max'] . ')';
		return $this->getInputContainer($this->getHeadline('int', $name), $this->formatDescription('int_' . $name) . we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_INT . '[' . $name . '#' . we_contents_base::ETYPE_NUMBER . ']', (string) $this->getElementI($name), strlen((string) ($this->getElementLen($name))), $attr, 'number'), 'numberField', $extra);
	}

	/**
	 * @param string $name
	 * @param array{table:string,name:string,type:string,len:int,flags:int,default:string,pos:int,type_short:string,flags_name:string} $dbFields
	 * @param bool $editable
	 * @return string
	 */
	private function getFloatFieldHTML(string $name, array $dbFields, bool $editable = true): string{
		if(!$editable){
			return $this->getPreviewView($name, $this->getElementS($name));
		}
		$extra = g_l('modules_object', '[field][float]');
		$attr = [
			'onchange' => "_EditorFrame.setEditorIsHot(true);",
			'step' => 'any',
		];
		if($dbFields['flags'] && MYSQLI_UNSIGNED_FLAG){
			$attr['min'] = '0';
		}
		return $this->getInputContainer($this->getHeadline('float', $name), $this->formatDescription('float_' . $name) . we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_FLOAT . '[' . $name . ']', (string) (strlen($this->getElementS($name)) ? $this->getElementS($name) : 0.0), $this->getElementLen($name), $attr, 'number'), 'floatField', $extra);
	}

	private function getDateFieldHTML(string $name, bool $editable = true): string{
		if(!$editable){
			return $this->getPreviewView($name, date(g_l('date', '[format][default]'), $this->getElementI($name)));
		}
		$d = $this->getElementI($name);
		return $this->getInputContainer(
				$this->getHeadline('date', $name), $this->formatDescription('date_' . $name) .
				we_html_tools::getNewDateInput('we_' . $this->Name . '_' . self::TYPE_DATE . '_' . $name . '[%s]', $d ? new DateTime(date('Y-m-d H:i:s', $d)) : null, we_html_tools::FORMAT_DATE_TIME, true, false),
																																																															'dateField'
		);
	}

	private function getTextareaHTML(string $name, array $attribs, bool $editable = true): string{
		if(!$editable){
			return $this->getPreviewView($name, (string) $this->getFieldByVal($this->getElementS($name), 'txt', $attribs));
		}
		//	send charset which might be determined in template
		$value = $this->getElementS($name);
		$attr = array_merge($attribs, [
			'wysiwyg' => $attribs['wysiwyg'] ?? false,
			'width' => $attribs['width'] ?? 620,
			'height' => $attribs['height'] ?? 200,
			'rows' => 10,
			'cols' => 60,
			'commands' => preg_replace('/ *, */', ',', isset($attribs['commands']) && $attribs['commands'] ? $attribs['commands'] : COMMANDS_DEFAULT),
			'inlineedit' => !empty($attribs['inlineedit']),
			'tinyparams' => $attribs['tinyparams'] ?? '',
			'templates' => $attribs['templates'] ?? '',
			'gallerytemplates' => $attribs['gallerytemplates'] ?? '',
			'class' => $attribs['class'] ?? '',
			'classes' => $attribs['cssClasses'] ?? ''
		]);
		$removefirstparagraph = !empty($attribs['removefirstparagraph']);

		return $this->getInputContainer($this->getHeadline('text', $name), $this->formatDescription('text_' . $name) . we_html_forms::weTextarea('we_' . $this->Name . '_' . self::TYPE_TEXT . '[' . $name . ']', $value, $attr, !((!empty($attribs['classes'])) || $this->getDocumentCss()), false, $removefirstparagraph, false, $name), 'textareaField');
	}

	private function getImageHTML(string $name, bool $editable = true): string{
		$id = $this->getElementI($name);
		if(!id2path($id)){
			$id = 0;
			$this->setElementI($name, 0);
		}
		$img = new we_document_image($id);
// handling thumbnails for this image
// identifying default thumbnail of class:
		$defvals = $this->DefArray;
		$realName = we_base_variants::stripVariantPrefix($name);
		$thumbID = fI('SELECT ID FROM ' . THUMBNAILS_TABLE . ' WHERE ID=' . ($defvals['img_' . $realName]['defaultThumb'] ?? 0)) ?: 0;
		if($img->ID && $img->fileExists()){
			$img->setElementI('thumbnailid', $thumbID, 'thumb');
		}

		if(!$editable){
			return $this->getPreviewView($name, $img->getHtml());
		}

// creating thumbnail only if it really exists:

		$fname = 'we_' . $this->Name . '_' . self::TYPE_IMG . '[' . $name . '#' . self::ETYPE_NUMBER . ']';

		return $this->getInputContainer($this->getHeadline('img', $name), $this->formatDescription('img_' . $name) . '<input type=hidden name="' . $fname . '" value="' . $this->getElementI($name) . '" />' .
// show thumbnail of image if there exists one:
				$img->getPreviewImg() .
				we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.FILE_TABLE," . ($id ?: 0) . ",WE().consts.contentTypes.IMAGE)", '', '', ($id ? false : true)) .
				we_html_button::create_button(we_html_button::IMAGE_SELECT, "javascript:we_cmd('we_selector_image',{id:'" . ($id ?: ($this->DefArray['img_' . $name]['defaultdir'] ?? 0)) . "',table:WE().consts.tables.FILE_TABLE,targetID:'" . $fname . "',jsCmd:'" . 'we_object_file_reloadEntry,' . we_base_WE::$transaction . ',img_' . $name . ',setScrollTo' . "',rootDirID:'" . ($this->DefArray['img_' . $name]['rootdir'] ?? 0) . "',startID: " . ($this->DefArray['img_' . $name]['defaultdir'] ?? 0) . "})") .
				we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('we_object_file_imageRemove','" . we_base_WE::$transaction . "','img_" . $name . "');setScrollTo();", '', '', ($id ? false : true)), 'imageField'
		);
	}

	private function getBinaryElementHTML(string $name, bool $editable, we_document_binary $doc, string $type, array $ct): string{
		$id = $this->getElementI($name);
		$doc->initByID($id);

		if(!$editable){
			return $this->getPreviewView($name, $doc->getHtml());
		}
		$fname = 'we_' . $this->Name . '_' . self::TYPE_IMG . '[' . $name . '#' . self::ETYPE_NUMBER . ']';

		$content = '<input type="hidden" name="' . $fname . '" value="' . (string) $this->getElementI($name) . '" />' . $doc->getHtml() .
			we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:'" . ($id ?: ($this->DefArray[$type . "_" . $name]['defaultdir'] ?? 0)) . "',table:WE().consts.tables.FILE_TABLE,targetID:'" . $fname . "',jsCmd:'we_object_file_reloadEntry," . we_base_WE::$transaction . "," . $type . "_" . $name . "',rootDirID:" . (!empty($this->DefArray[$type . '_' . $name]['rootdir']) ? $this->DefArray[$type . '_' . $name]['rootdir'] : 0) . ",filter:'" . implode(',', $ct) . "'})") .
			we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('we_object_file_imageRemove','" . we_base_WE::$transaction . "','" . $type . '_' . $name . "')");

		return $this->getInputContainer($this->getHeadline($type, $name), $this->formatDescription($type . '_' . $name) . $content, 'fileField');
	}

	/**
	 *
	 * @return array<string,mixed>
	 */
	public function getDefaultValueArray(): array{
		if(!$this->TableID){//e.g. quick commands
			//t_e('no table id');
			return [];
		}
		if(empty($this->classData)){
			$h = getHash('SELECT * FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($this->TableID));
			$h['DefaultValues'] = we_unserialize($h['DefaultValues']);
			$h['Workspaces'] = we_base_request::filterIntList($h['Workspaces']);
			$h['Templates'] = we_base_request::filterIntList($h['Templates']);
			$h['WorkspacesTemplates'] = (empty($h['Workspaces']) || empty($h['Templates']) ? [] : array_combine($h['Workspaces'], $h['Templates']));
			$this->classData = $h;
		}

		$ret = $this->classData['DefaultValues'];
		assert(is_array($ret));
		return $ret;
	}

	public function canMakeNew(): bool{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return true;
		}
		$ac = we_users_util::getAllowedClasses();
		return !empty($ac) && $ac[0] !== -1;
	}

	/**
	 *
	 * @param int[] $ClassWs
	 * @param bool $all
	 * @return array<int,string>
	 */
	public static function getPossibleWorkspaces(array $ClassWs, bool $all = false): array{
		$userWs = we_users_util::get_ws(FILE_TABLE);
// wenn User Admin ist oder keine Workspaces zugeteilt wurden
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) || ((!$userWs) && $all)){
// alle ws, welche in Klasse definiert wurden zurueckgeben
			return $ClassWs ? id2pathA($ClassWs, FILE_TABLE) : [];
		}
// alle UserWs, welche sich in einem der ClassWs befinden zurueckgeben
		$out = [];
		foreach($ClassWs as $ws){
			if(we_users_util::in_workspace((int) $ws, $userWs, FILE_TABLE)){
				$out[] = (int) $ws;
			}
		}

		return $out ? id2pathA($out, FILE_TABLE) : [];
	}

	private function formWorkspaces(we_base_jsCmd $jsCmd): string{
		$classWsTmpl = $this->classData['WorkspacesTemplates'];

		$values = self::getPossibleWorkspaces($this->classData['Workspaces']);
//    remove not existing workspaces and templates
		$arr = $this->Workspaces ? id2pathA($this->Workspaces, FILE_TABLE) : [];

		//only dirs which are in class
		$newArr = $arr ? array_map('intval', array_intersect(array_keys($arr), array_keys($values))) : [];
		sort($newArr);

		$this->Workspaces = $newArr;
		//list only dirs not already selected
		$values = array_diff_key($values, array_flip($newArr));

		$addbut = (empty($values) ?
			'' :
			we_html_tools::select(md5(uniqid(__FUNCTION__)), (['' => g_l('global', '[add_workspace]')] + $values), 1, '', false, ['onchange' => '_EditorFrame . setEditorIsHot(true);
		we_cmd(\'object_add_workspace\',this.options[this.selectedIndex].value);'])
			);

		$obj = new we_chooser_multiDirAndTemplate($newArr, 'object_del_workspace', $addbut, $classWsTmpl);

		$obj->setEditable(true);

		return $obj->get($jsCmd);
	}

	function del_workspace(int $id): void{
		if($id < 0){
			return;
		}
		if(($pos = array_search($id, $this->Workspaces)) !== false){
			unset($this->Workspaces[$pos]);
		}
	}

	function ws_from_class(): void{
		$this->Workspaces = $this->classData['Workspaces'];
	}

	private function getTemplateFromWorkspace(int $parentID): int{
		$classWsTmpl = $this->classData['WorkspacesTemplates'];
		if(empty($classWsTmpl) || empty($this->Workspaces)){
			return 0;
		}
		//if the parent is set as workspace in object, this is fast
		if(isset($classWsTmpl[$parentID]) && in_array($parentID, $this->Workspaces)){
			return (int) $classWsTmpl[$parentID];
		}
		//search for set workspaces
		$ws = fI('SELECT ID FROM ' . FILE_TABLE . ' AS parent WHERE ID IN (' . implode(',', $this->Workspaces) . ') AND CONCAT(parent.Path,"/")=(
SELECT LEFT(Path,LENGTH(parent.Path)+1) FROM ' . FILE_TABLE . ' WHERE ID=' . $parentID . ') ORDER BY Path DESC LIMIT 1');

		//return the template for found workspace, or for root if set - 0 otherwise
		return (int) ($ws && isset($classWsTmpl[$ws]) ? $classWsTmpl[$ws] : ($classWsTmpl[0] ?? 0));
	}

	function getTemplateID(int $parentID): int{
		//noting found, use first of class-Template
		return (int) ($this->getTemplateFromWorkspace($parentID) ?: reset($this->classData['WorkspacesTemplates']) ?: 0);
	}

	/**
	 *
	 * @param string $elementName
	 * @param string $type
	 * @return string
	 */
	function getFieldValue(string $elementName, string $type): string{
		switch($type){
			case self::TYPE_HREF:
				return parent::getHrefByArray($this->getElementA($elementName));
			case self::TYPE_LINK:
				$link = $this->getElementA($elementName);
				return (empty($link) ?
					'' :
					self::getLinkContent($link, new we_document_image())
					);
			case self::TYPE_CHECKBOX:
				return /* $this->getElementS($elementName) ?: */ (string) ($this->getElementI($elementName) ?: '');
			case self::TYPE_SELECT:
				if(!$this->DefArray || !is_array($this->DefArray)){
					$this->DefArray = $this->getDefaultValueArray();
				}
				$options = array_combine($this->DefArray[self::TYPE_SELECT . '_' . $elementName]['selectKeys'], $this->DefArray[self::TYPE_SELECT . '_' . $elementName]['selectValues']);
				// FIXME: why do we save numeric selected keys to bdid? we should save all selected keys to dat!
				$selected = $this->getElementS($elementName) ?: (string) $this->getElementI($elementName);
				return array_key_exists($selected, $options) ? $options[$selected] : '';
			case self::TYPE_DATE:
			case self::TYPE_IMG:
				return (string) $this->getElementI($elementName);
			case self::TYPE_OBJECT:
				return $this->getElementS(we_listview_base::OBJECT_FIELD_PREFIX . $elementName);
			default:
				return $this->getElementS($elementName);
		}
	}

	function setTitleAndDescription(): void{
		$fields = [self::DESCRIPTION_FIELD => 'DefaultDesc', self::TITLE_FIELD => 'DefaultTitle', self::KEYWORD_FIELD => 'DefaultKeywords'];

		foreach($fields as $key => $field){
			if(empty($this->classData[$field])){
				continue;
			}
			$regs = explode('_', (string) $this->classData[$field], 2);
			if(!empty($regs[0]) && !empty($regs[1])){
				$this->setElementS($key, $this->getFieldValue($regs[1], $regs[0]));
			}
		}
	}

	public function setUrl(): void{
		if(!$this->TableID){
			return;
		}
		$max = count(self::URL_FIELDS);
		$regs = $urlfield = [];
		$text = (string) $this->classData['DefaultUrl'];
		if(empty($text)){
			$this->Url = '';
		}
		for($i = 0; $i <= $max; ++$i){
			if(!empty($this->classData['DefaultUrlfield' . $i])){
				preg_match('/(.+?)_(.*)/', (string) $this->classData['DefaultUrlfield' . $i], $regs);
				$cur = $urlfield[$i] = (isset($regs[1]) && $regs[1] !== '' && isset($regs[2]) && $regs[2] !== '' ?
					$this->getFieldValue($regs[2], $regs[1]) : '');
			} else {
				$cur = '';
			}
			if($i > 0 && preg_match('/%urlfield' . $i . '(\d+)%/', $text, $regs)){
				$anz = (!$regs[1] ? self::URL_FIELDS['urlfield' . $i] : intval($regs[1]));
				$text = preg_replace('/%urlfield' . $i . '\d+%/', trim(substr($cur, 0, $anz)), $text);
			}
		}
		if(!isset($urlfield[0]) || $urlfield[0] == ''){
			$urlfield[0] = time();
		}

		if(preg_match('/%urlunique([^%]*)%/', $text, $regs)){
			$anz = (!$regs[1] ? 16 : intval($regs[1]));
			$unique = substr(md5(uniqid(__FUNCTION__, true)), 0, min($anz, 32));
			$text = preg_replace('/%urlunique[^%]*%/', $unique, $text);
		}
		$tstamp = (int) $urlfield[0];
		$text = strtr($text, [
			'%ID%' => $this->ID,
			'%locale%' => $this->Language,
			'%language%' => substr($this->Language, 0, 2),
			'%country%' => substr($this->Language, 4, 2),
			'%d%' => $this->Creation_Date->format('d'),
			'%j%' => $this->Creation_Date->format('j'),
			'%m%' => $this->Creation_Date->format('m'),
			'%y%' => $this->Creation_Date->format('y'),
			'%Y%' => $this->Creation_Date->format('Y'),
			'%n%' => $this->Creation_Date->format('n'),
			'%g%' => $this->Creation_Date->format('G'),
			'%G%' => $this->Creation_Date->format('G'),
			'%h%' => $this->Creation_Date->format('H'),
			'%H%' => $this->Creation_Date->format('H'),
			'%Md%' => $this->Mod_Date->format('d'),
			'%Mj%' => $this->Mod_Date->format('j'),
			'%Mm%' => $this->Mod_Date->format('m'),
			'%My%' => $this->Mod_Date->format('y'),
			'%MY%' => $this->Mod_Date->format('Y'),
			'%Mn%' => $this->Mod_Date->format('n'),
			'%Mg%' => $this->Mod_Date->format('G'),
			'%MG%' => $this->Mod_Date->format('G'),
			'%Mh%' => $this->Mod_Date->format('H'),
			'%MH%' => $this->Mod_Date->format('H'),
			'%Fd%' => date('d', $tstamp),
			'%Fj%' => date('j', $tstamp),
			'%Fm%' => date('m', $tstamp),
			'%Fy%' => date('y', $tstamp),
			'%FY%' => date('Y', $tstamp),
			'%Fn%' => date('n', $tstamp),
			'%Fg%' => date('G', $tstamp),
			'%FG%' => date('G', $tstamp),
			'%Fh%' => date('H', $tstamp),
			'%FH%' => date('H', $tstamp),
			'%DirSep%' => '/',
			'%Filename%' => trim($this->Filename),
			'%Parent%' => fS('SELECT Text FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($this->ParentID)),
			'%PathIncC%' => ltrim(strtr($this->Path, [$this->Filename => '']), '/'),
			'%PathNoC%' => strtr($this->Path, [$this->Filename => '', $this->classData['Path'] => '']),
			]
		);

		//remove duplicate "//" which will produce errors and transform URL to lowercase
		$text = preg_replace('|\.+$|', '', strtr((string) (OBJECTSEOURLS_LOWERCASE ? mb_convert_case($text, MB_CASE_LOWER, 'UTF-8') ?? '' : $text), [' ' => '-', '//' => '/']));
		$text = (URLENCODE_OBJECTSEOURLS ?
			strtr(urlencode(str_replace(we_base_util::SEO_FILTER_CHARS, '', $text)), ['%2F' => '/']) :
			we_base_util::convertDisallowedChars($text));
		$this->Url = substr($text, 0, 256);
	}

	public function insertAtIndex(array $only = [], array $fieldTypes = []): bool{
		if(!($this->IsSearchable && $this->Publish_Date)){
			$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=' . $this->TableID . ' AND ID=' . intval($this->ID));
			return true;
		}

		$this->setTitleAndDescription();
		$text = '';
		foreach($this->nextElement('') as $k){
			$k = (string) $k;
			switch($this->getElementType($k)){
				default:
				case self::TYPE_OBJECT:
				case self::TYPE_MULTIOBJECT:
				case self::TYPE_LANGUAGE:
				case self::TYPE_HREF:
				case self::TYPE_IMG:
					//not handled
					break;
				case self::TYPE_DATE:
					if(($v = $this->getElementI($k))){
						$text .= ' ' . date(g_l('date', '[format][default]'), $v);
					}
					break;
				case self::TYPE_INT:
					if(($v = $this->getElementI($k))){
						$text .= ' ' . $v;
					}
					break;
				case self::TYPE_FLOAT:
					if(($v = $this->getElementS($k))){
						$text .= ' ' . ((float) $v);
					}
					break;

				case self::TYPE_SELECT://FIXME: select returns the key not the value
				case self::TYPE_INPUT:
				case 'txt':
				case self::TYPE_TEXT:
					if(($v = $this->getElementS($k))){
						if(we_base_util::isSerialized($v)){
							//link/href
							$tmp = we_unserialize($v);
							if(isset($tmp['text'])){
								$text .= ' ' . $tmp['text'];
							}
						} else {
							$text .= ' ' . $v;
						}
					}
					break;
			}
		}
		$text = self::formatIndexText($text);

		if(!$text){
			$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=' . $this->TableID . ' AND ID=' . intval($this->ID));
			//no need to keep an entry without relevant data in the index
			return true;
		}

		$ws = array_unique($this->Workspaces) ?: [0];

		foreach($ws as $w){
			if(!$this->DB_WE->query('REPLACE INTO ' . INDEX_TABLE . ' SET ' . we_database_base::arraySetter([
						'ID' => $this->ID,
						'ClassID' => $this->TableID,
						'WorkspaceID' => $w,
						'Text' => $text,
						'Category' => implode(',', array_filter(array_unique($this->Category))) ?: null,
						'Title' => self::formatIndexText($this->getElementS(self::TITLE_FIELD)) ?: null,
						'Description' => self::formatIndexText($this->getElementS(self::DESCRIPTION_FIELD)) ?: null,
						'Keywords' => self::formatIndexText($this->getElementS(self::KEYWORD_FIELD)) ?: null,
						'Language' => $this->Language,
				]))){
				return false;
			}
		}
		return true;
	}

	private function setLanguage(string $language = ''): void{
		$this->Language = $language ?: $this->Language;
	}

	private function setPublishTime(?DateTime $time): bool{
		$this->Publish_Date = $time;
		return $this->DB_WE->query('UPDATE ' . OBJECT_FILES_TABLE . ' SET ' . we_database_base::arraySetter(['Publish_Date' => $time]) . ' WHERE ID=' . $this->ID);
	}

	private function markAsPublished(): bool{
		return $this->setPublishTime(new DateTime());
	}

	private function markAsUnPublished(): bool{
		return $this->setPublishTime(null);
	}

	private function findElementType(string $k): string{
		foreach(array_keys($this->DefArray) as $n){
			$regs = explode('_', (string) $n, 2);
			$testtype = $regs[0];
			if(isset($regs[1])){
				$fieldname = $regs[1];
				if($k == $fieldname){
					return $testtype;
				}
			}
		}
		return '';
	}

	/**
	 *
	 * @param string $type
	 * @param string $v
	 * @param string $k
	 * @return mixed
	 */
	protected function convertElemFromRequest(string $type, string $v, string $k){
		$type = $type ?: $this->findElementType($k);
		switch($type){
			case self::TYPE_TEXT:
				return we_base_util::rmPhp($v);
			case self::TYPE_INPUT:
				return !empty($this->DefArray[$type . '_' . $k]['forbidhtml']) && $this->DefArray[$type . '_' . $k]['forbidhtml'] === true ? we_base_util::rmHTML($v, true) : we_base_util::rmPhp($v);
			case self::TYPE_FLOAT:
				return (float) $v;
			case self::TYPE_INT:
			case self::TYPE_CHECKBOX:
				return (int) $v;
			default:
				return we_base_util::rmHTML($v, true);
		}
	}

	private function correctWorkspaces(): void{
		if(!$this->Workspaces){
			return;
		}
		$this->Workspaces = array_merge((in_array(0, $this->Workspaces) ? [0] : []),
																						DB_WE::$inst->getAllq('SELECT ID FROM ' . FILE_TABLE . ' WHERE ID IN(' . implode(',', $this->Workspaces) . ') AND IsFolder=1', true)
		);
	}

	protected function isPathInvalid(): bool{
		return parent::isPathInvalid() || $this->ParentID === 0 || $this->ParentPath === '/' || !str_starts_with($this->Path, $this->RootDirPath);
	}

	public function parseTextareaFields(string $rebuildMode = 'default'): void{
		foreach($this->nextElement(self::TYPE_TEXT) as $name){
			$name = (string) $name;
			$text = $this->getElementS($name);
			$this->MediaLinks = array_merge($this->MediaLinks, we_wysiwyg_editor::getAllMediaLinks($text, $name));
			$text = we_wysiwyg_editor::parseThumbnailsWidthHeight($text);
			$this->setElementS($name, $text);
		}
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		if(intval($this->TableID) == 0){
			return false;
		}
		$this->errMsg = '';
		$insertToHistory = true;

		if($this->isPathInvalid()){
			return false;
		}
		if(empty($this->classData)){
			t_e('should not happen');
			$this->getDefaultValueArray();
		}

		we_base_variants::correctModelFields($this);
		$this->TriggerID = $this->TriggerID ?:
			(fI('SELECT TriggerID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($this->ParentID)) ?:
			(int) $this->classData['DefaultTriggerID']);
		$resaveWeDocumentCustomerFilter = true;
		$this->correctWorkspaces();

		if(!$skipHook){
			$hook = new we_hook_handler('preSave', ['obj' => $this, 'resave' => $resave]);
			//check if doc should be saved
			if(!$hook->executeHook()){
				$this->errMsg = $hook->getErrorString();
				return false;
			}

			//check for warnings and notices
			if(($tmp = $hook->getWarningString())){
				$this->warningMsg[] = $tmp;
			}
			if(($tmp = $hook->getNoticeString())){
				$this->noticeMsg[] = $tmp;
			}
		}

		if((!$this->ID || $resave)){
			$insertToHistory = false;
			$resaveWeDocumentCustomerFilter = false;
			if((!parent::we_save($resave, true)) || $resave || (!$this->we_republish())){
				return false;
			}
		}
		$this->Mod_Date = new DateTime();
		$this->ModifierID = !we_base_WE::$schedulerActive ? we_users_user::getID() : 0;
		$this->wasUpdate = true;
		$this->setUrl();

		$this->unregisterMediaLinks(false);
		$this->parseTextareaFields('temp');
		$this->registerMediaLinks(true);

		if($resave === false && $resaveWeDocumentCustomerFilter){
			$this->saveDocumentCustomerFilter();
		}

		if(!$this->Publish_Date){
			$insertToHistory = false;
			if(!we_contents_root::we_save(true)){
				return false;
			}
			if(!$resave){
				we_base_delete::deleteTemporaryContent($this->ID, $this->Table);
			}
		}
		$a = $this->saveTmp();
// version
		if($this->isVersioned()){
			we_versions_version::save($this);
		}
		if(LANGLINK_SUPPORT){
			if(!is_array($this->LangLinks)){
				$this->LangLinks = [];
			}
			$a = $a && $this->setLanguageLink();
		}

// hook
		if(!$skipHook){
			$hook = new we_hook_handler('save', ['obj' => $this, 'resave' => $resave]);
			//check if doc should be saved
			if(!$hook->executeHook()){
				$this->errMsg = $hook->getErrorString();
				return false;
			}

			//check for warnings and notices
			if(($tmp = $hook->getWarningString())){
				$this->warningMsg[] = $tmp;
			}
			if(($tmp = $hook->getNoticeString())){
				$this->noticeMsg[] = $tmp;
			}
		}

		if($insertToHistory){
			we_base_history::insertIntoHistory($this);
		}

		return $a;
	}

	function registerMediaLinks(bool $temp = false, bool $linksReady = false): bool{
		//register media in fields type link
		if(!$linksReady){
			$dv = $this->getDefaultValueArray();
			foreach(array_keys($dv) as $k){
				if(str_starts_with($k, 'link_')){
					$name = str_replace('link_', '', $k);
// IMI!
//					$link = $this->getElementA($name);
					$link = [];
					if(isset($link['type']) && isset($link['id']) && isset($link['img_id'])){ //FIXME: $link should be an object so we can check class
						if($link['type'] === 'int' && $link['id']){
							$this->MediaLinks['link[name=' . $name . ']'] = $link['id'];
						}
						if($link['img_id']){
							$this->MediaLinks['link[name=' . $name . ']'] = $link['img_id'];
						}
					}
				}
			}
		}

		return parent::registerMediaLinks($temp, $linksReady);
	}

	function ModifyPathInformation(int $parentID): void{
		$this->setParentID($parentID);
		$this->Path = $this->getPath();
		$this->wasUpdate = true;
		$this->savePersistentSlotsToDB(['Filename', 'Path', 'ParentID']);
		$this->saveTmp();
		$this->insertAtIndex();
		$this->modifyChildrenPath(); // only on folders, because on other classes this function is empty
	}

	function hasWorkspaces(): bool{
		return !empty($this->classData['Workspaces']);
	}

	function setTypeAndLength(): void{
		if(!$this->TableID){
			return;
		}
		$db = $this->DB_WE;
		$tableInfo = $db->metadata(OBJECT_X_TABLE . intval($this->TableID));
		'@phan-var array<string,array{table:string,name:string,type:string,len:int,flags:int}> $tableInfo';
		$regs = [];
		foreach($tableInfo as $cur){
			if(preg_match('/(.+?)_(.*)/', $cur['name'], $regs)){
				[, $type, $name] = $regs;
				switch($type){
					case 'OF':
						continue 2;
					default:
						$this->issetElement($name) || $this->elements[$name] = [];
						$this->setElementLen($name, intval($cur['len']));
						$this->setElementType($name, $type);
				}
			}
		}
	}

	protected function we_load(int $from = self::LOAD_MAIN_DB): bool{
		$found = false;
		switch($from){
			case self::LOAD_TEMP_DB:
				$found = parent::we_load($from);
				if($found){
//fixed: at least TableID must be fetched
					break;
				}
				$from = self::LOAD_MAIN_DB;
//no break;
			case self::LOAD_MAIN_DB:
				$found = parent::we_load($from);
				break;
		}
		$this->getDefaultValueArray();
		$this->setTitleAndDescription();
		$this->getLinkedObjects();
// init Customer Filter !!!!
		return $found;
	}

	private function getUniqueIDsAndFixNames(): void{ //FIXME: remove in cardada
		if(!is_array($this->DefArray) || empty($this->DefArray)){
			return;
		}
		$newDefArr = $this->getDefaultValueArray();
		foreach($newDefArr as $n => $v){
			if(is_array($v) &&
				isset($v['uniqueID']) &&
				($oldName = $this->DefArrayNameNotEqual($n, $v['uniqueID']))){
				$foo = explode('_', $n);
				unset($foo[0]);
				$nn = implode('_', $foo);
				$foo = explode('_', $oldName);
				unset($foo[0]);
				$no = implode('_', $foo);
				$this->elements[$nn] = $this->elements[$no] ?? '';
				unset($this->elements[$no]);
			}
		}
	}

	private function DefArrayNameNotEqual(string $name, string $uniqueID): string{
		foreach($this->DefArray as $n => $v){
			if(is_array($v) && isset($v['uniqueID']) && $v['uniqueID'] == $uniqueID){
				return (string) ($n === $name ? '' : $n);
			}
		}
		return '';
	}

	public function we_publish(bool $DoNotMark = false, bool $saveinMainDB = true, bool $skipHook = false): bool{
		if(!$skipHook){
			$hook = new we_hook_handler('prePublish', [$this]);
			//check if doc should be saved
			if(!$hook->executeHook()){
				$this->errMsg = $hook->getErrorString();
				return false;
			}

			//check for warnings and notices
			if(($tmp = $hook->getWarningString())){
				$this->warningMsg[] = $tmp;
			}
			if(($tmp = $hook->getNoticeString())){
				$this->noticeMsg[] = $tmp;
			}
		}
		$old = $this->Publish_Date;
		$this->oldCategory = we_base_request::filterIntList(fS('SELECT Category FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->ID)));

		if(!($saveinMainDB ? we_contents_root::we_save(true) : $this->we_save($DoNotMark))){
			return false;
		}
		if($DoNotMark){
			$this->unregisterMediaLinks(true, false);
			$this->parseTextareaFields('main');
			// TODO: we should try to throw out obsolete elements from temporary! but this affects static docs only!
			// TODO: when doing rebuild media link test all elements against template!
			$this->registerMediaLinks(); // last param: when rebuilding static docs do not delete temp entries!
		} else {
			if(!$this->markAsPublished()){
				return false;
			}

			$this->unregisterMediaLinks();
			$this->registerMediaLinks(false, true);
		}
		//hook
		if(!$skipHook){
			$hook = new we_hook_handler('publish', ['obj' => $this, 'prePublishTime' => $old]);
			//check if doc should be saved
			if(!$hook->executeHook()){
				$this->errMsg = $hook->getErrorString();
				return false;
			}

			//check for warnings and notices
			if($hook->getWarningString() != ''){
				$this->warningMsg[] = $hook->getWarningString();
			} elseif($hook->getNoticeString() != ''){
				$this->noticeMsg[] = $hook->getNoticeString();
			}
		}
		if(!$DoNotMark){
			we_base_delete::deleteTemporaryContent($this->ID, $this->Table);
		}
		//if($oldUrl != $this->Url || !$wasPublished || $this->oldCategory != $this->Category){
		$this->rewriteNavigation();
		//}
		return $this->insertAtIndex();
	}

	public function we_unpublish(bool $skipHook = false): bool{
		$oldPublished = $this->Publish_Date;
		if(!$this->ID || !$this->markAsUnPublished()){
			return false;
		}

		/* version */
		if($this->isVersioned()){
			we_versions_version::save($this, 'unpublished');
		}
		/* hook */
		if(!$skipHook){
			$hook = new we_hook_handler('unpublish', [$this]);
			//check if doc should be saved
			if(!$hook->executeHook()){
				$this->errMsg = $hook->getErrorString();
				return false;
			}

			//check for warnings and notices
			if($hook->getWarningString()){
				$this->warningMsg[] = $hook->getWarningString();
			} elseif($hook->getNoticeString()){
				$this->noticeMsg[] = $hook->getNoticeString();
			}
		}
//clear navigation cache to see change if object in navigation #6916
		$this->rewriteNavigation();

		$ret = $this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=' . $this->TableID . ' AND ID=' . intval($this->ID));

		// if document was modified before unpublishing, the actual version is in tblTemporaryDoc: we unregister temp=0
		// otherwise we have nothing to do
		if($ret && $oldPublished && ($this->Mod_Date > $oldPublished)){
			$this->unregisterMediaLinks(true, false);
		}

		return $ret;
	}

	public function we_republish(bool $rebuildMain = true): bool{
		return ($this->Publish_Date && $this->Mod_Date <= $this->Publish_Date ?
			$this->we_publish(true, $rebuildMain) :
			$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=' . $this->TableID . ' AND ID=' . intval($this->ID))
			);
	}

	private function objectFileInit(bool $makeSameNewFlag = false): void{
		if($this->ID){
			$this->setRootDirID();
			$oldTableID = fI('SELECT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($this->ID));
			if($oldTableID != $this->TableID){
				$this->resetParentID();
			}
			if(($def = $this->getDefaultValueArray())){
				if(isset($def['WE_CSS_FOR_CLASS'])){
					$this->CSS = is_array($def['WE_CSS_FOR_CLASS']) ? $def['WE_CSS_FOR_CLASS'] : we_base_request::filterIntList($def['WE_CSS_FOR_CLASS']);
				}
			}
		} else if(we_base_WE::$we_EDITOR && (!$this->ID)){
			if($this->DefaultInit == true){
				if(!$this->TableID){
					$this->AllowedClasses = we_users_util::getAllowedClasses();
					$this->TableID = $this->AllowedClasses[0];
				}
				if($this->TableID){
					$this->setRootDirID();
					if(!$makeSameNewFlag){
						$this->resetParentID();
					}
					$this->restoreDefaults($makeSameNewFlag);
				}
			} else {
				$initWeDocumentCustomerFilter = ($this->ParentID ? false : true);

				$this->setRootDirID();
				$this->checkAndCorrectParent();
				if($initWeDocumentCustomerFilter){
// get customerFilter of parent Folder
					$tmpFolder = new we_folder_objectFile($this->rootDirID);
					$this->documentCustomerFilter = $tmpFolder->documentCustomerFilter;
					unset($tmpFolder);
				}
			}
		}
	}

	function getLinkedObjects(): void{
		if(!$this->TableID){
			return;
		}
		static $recursiveObjects = [];
		if(empty($recursiveObjects)){
			$recursiveObjects[] = $this->ID;
		}

		$linkObjects = [];
		$tableInfo = self::getSortedTableInfo($this->TableID, false, $this->DB_WE);
		$regs = [];
		foreach($tableInfo as $cur){
			if(in_array($cur['name'], ['OF_ID', 'OF_DocumentTable'], true)){
				continue;
			}
			if(preg_match('/' . self::TYPE_OBJECT . '_(.*)/', $cur['name'], $regs)){
				if(($id = $this->getElementI('we_' . $cur['name']))){
					$linkObjects[] = $id;
				}
			}
		}
		foreach($linkObjects as $id){
			if(in_array($id, $recursiveObjects)){
				continue;
			}
			$recursiveObjects[] = $id;
			$tmpObj = new we_contents_objectFile($id);
			array_pop($recursiveObjects);
			foreach($tmpObj->elements as $n => $elem){
				switch($n){
					case self::TITLE_FIELD:
					case self::DESCRIPTION_FIELD:
					case self::KEYWORD_FIELD:
						break;
					default:
						if($elem['type'] != self::TYPE_OBJECT && !$this->issetElement($n)){
							$this->elements[$n] = $elem;
						}
				}
			}
		}
	}

	protected function getContentData(string $table): bool{
		if(!$this->TableID){
			return false;
		}
		$tableInfo = self::getSortedTableInfo($this->TableID, false, $this->DB_WE);

		$hash = getHash('SELECT * FROM ' . OBJECT_X_TABLE . intval($this->TableID) . ' WHERE OF_DocumentTable="' . stripTblPrefix($table) . '" AND OF_ID=' . intval($this->ID));
		if(!$hash){
			return false;
		}
		foreach($tableInfo as $cur){
			$regs = explode('_', $cur['name'], 2);
			if(count($regs) > 1){
				[$type, $name] = $regs;
				switch($type){
					case 'OF':
						continue 2;
					case self::TYPE_INT:
					case self::TYPE_IMG:
					case self::TYPE_BINARY:
					case self::TYPE_AUDIO:
					case self::TYPE_VIDEO:
					case self::TYPE_DATE:
						$this->setElementI($name, (int) ($hash[$cur['name']] ?? 0), $type);
						break;
					case self::TYPE_FLOAT:
						$this->setElementF($name, (float) $hash[$cur['name']], $type);
						break;
					case self::TYPE_CHECKBOX:
						$this->setElementB($name, (bool) $hash[$cur['name']], $type);
						break; #
					case self::TYPE_OBJECT:
						$name = we_listview_base::OBJECT_FIELD_PREFIX . $name;
						$this->setElementI($name, (int) $hash[$cur['name']], $type);
						break;
					case self::TYPE_LINK:
					case self::TYPE_HREF:
						$this->setElementA($name, we_unserialize($hash[$cur['name']]), $type);
						break;
					default:
						$this->setElementS($name, (string) $hash[$cur['name']], $type);
				}
				$this->setElementLen($name, $cur['len']);
			}
		}

// add variant data if available
		$fieldname = 'variant_' . we_base_variants::ELEMENT_NAME;
		$elementName = we_base_variants::ELEMENT_NAME;
		if(isset($hash[$fieldname])){
			$this->setElementS($elementName, $hash[$fieldname], 'variant'); // this should be obsolete!
			$this->setElementLen($elementName, strlen($hash[$fieldname]));

			// FIXME: eliminate element we_base_variants::ELEMENT_NAME, we only need ELEMENT_NAME_ARRAY
			$this->setElementA(we_base_variants::ELEMENT_NAME_ARRAY, we_unserialize($hash[$fieldname]), 'variant');
		}

		return true;
	}

	protected function isFilenameEmpty(): bool{
		return ($this->Filename === '');
	}

	protected function isFilenameNotValid(): bool{
		return preg_match('/[^a-z0-9\._\-]/i', $this->Filename) ? true : false;
	}

	protected function isFilenameNotAllowed(): bool{
		return false;
	}

	protected function isFilenameDouble(): bool{
		return fB('SELECT 1 FROM ' . OBJECT_FILES_TABLE . ' WHERE ParentID=' . intval($this->ParentID) . ' AND Filename="' . $this->DB_WE->escape($this->Filename) . '" AND ID!=' . intval($this->ID));
	}

	public function checkFieldsOnSave(): string{
		if($this->urlDouble()){
			return sprintf(g_l('weEditor', '[objectFile][we_objecturl_exists]'), $this->Url);
		}
		return parent::checkFieldsOnSave();
	}

	private function urlDouble(): bool{
		$this->setUrl();

		return ($this->Url && fB('SELECT 1 FROM ' . OBJECT_FILES_TABLE . ' WHERE Url="' . DB_WE::$inst->escape($this->Url) . '" AND ID!=' . intval($this->ID)) );
	}

	function checkPathDiffAndCreate(): bool{
		return true;
	}

	protected function scheduleToBeforeNow(): bool{
		//FIXME: check schedarray!
		return false; //(we_modules_info::isActive(we_modules_info::SCHEDULER) && ($this->To < time()));
	}

	function publInScheduleTable(): ?DateTime{
		if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
			return we_schedule_scheduler::publInScheduleTable($this->ID, $this->Table, $this->schedArr, $this->DB_WE);
		}
		return null;
	}

	protected function writeDocument(): bool{
		return true; // do nothing;
	}

	protected function saveContentDataInDB(string $table): bool{
		if(intval($this->TableID) === 0){
			return false;
		}
		$fieldnames = $this->DB_WE->metaFieldNames(OBJECT_X_TABLE . intval($this->TableID));

		if(!$this->wasUpdate){
			$this->CreatorID = $this->CreatorID ?: we_users_user::getID();
		}

		$data = [];
		foreach($fieldnames as $cur){
			$regs = explode('_', (string) $cur, 2);
			if(count($regs) <= 1){
				continue;
			}
			$name = $regs[1];
			switch($regs[0]){
				case 'OF':
					switch($name){
						case 'ID':
							$data['OF_ID'] = $this->ID;
							break;
						case 'DocumentTable':
							$data['OF_DocumentTable'] = stripTblPrefix($table);
							break;
					}
					break;
				//quick fix for now
				case self::TYPE_FLOAT:
					$data[$cur] = $this->getElementF($name);
					break;
				case self::TYPE_INT:
				case self::TYPE_IMG:
				case self::TYPE_BINARY:
				case self::TYPE_AUDIO:
				case self::TYPE_VIDEO:
				case self::TYPE_DATE:
					$data[$cur] = $this->getElementI($name);
					break;
				case self::TYPE_CHECKBOX:
					$data[$cur] = $this->getElementB($name);
					break;
				case self::TYPE_OBJECT:
					$name = we_listview_base::OBJECT_FIELD_PREFIX . $name;
					$data[$cur] = $this->getElementI($name);
					break;
				case 'variant':
					$data[$cur] = we_serialize($this->getElementA(we_base_variants::ELEMENT_NAME_ARRAY), SERIALIZE_JSON);
					break;
				default:
//@phan-suppress-next-line PhanDeprecatedFunction
					$val = $this->getElement($name);
					$data[$cur] = is_array($val) ? we_serialize($val, SERIALIZE_JSON) : $val;
			}
		}

		$ret = ($this->DB_WE->query('REPLACE INTO ' . OBJECT_X_TABLE . intval($this->TableID) . ' SET ' . we_database_base::arraySetter($data)));
		return $ret;
	}

	private function saveTmp(): bool{
		if($this->Publish_Date && ($this->Mod_Date > $this->Publish_Date)){
			if(!$this->saveContentDataInDB(TEMPORARY_DOCTARGET)){
				return false;
			}
		}
		return $this->savePersistentSlotsToDB([
				'Path', 'Text', 'ParentID', 'CreatorID', 'ModifierID', 'RestrictOwners', 'Owners', 'OwnersReadOnly', 'Publish_Date', 'Mod_Date',
				'IsSearchable', 'Url', 'TriggerID']);
	}

	function getDocumentData(string $includepath = ''): string{
		extract($GLOBALS, EXTR_SKIP); // globalen Namensraum herstellen.
		$back = [we_base_WE::$we_doc, we_base_WE::$we_obj];
		we_base_WE::setWeObj($this);

		$doc = new we_document_triggerDocument();
		$doc->initByObj($this);
		$doc->InWebEdition = false;

		we_base_WE::setWeDoc($doc);

		$we_include = $includepath ?: $doc->getTemplatePath();
		ob_start();
		include($we_include);
		$contents = ob_get_clean() ?: '';
		[we_base_WE::$we_doc, we_base_WE::$we_obj] = $back;
		return $contents ?: '';
	}

	protected function setElementsFromHTTP(): void{
		parent::setElementsFromHTTP();
		if(!$_REQUEST){
			return;
		}
		$regs = [];
		$fields = [
			self::TYPE_HREF => false,
			self::TYPE_MULTIOBJECT => false,
			self::TYPE_IMG => false,
		];
		'@phan-var array<string,bool> $fields';

		foreach(array_keys($_REQUEST) as $n){
			if(preg_match('/^we_' . $this->Name . '_(' . self::TYPE_HREF . '|' . self::TYPE_MULTIOBJECT . '|' . self::TYPE_IMG . ')$/', $n, $regs)){
				$fields[$regs[1]] = true;
			}
		}
		if($fields[self::TYPE_HREF]){
			$empty = ['int' => 1, 'intID' => '', 'extPath' => ''];
			$hrefs = $match = [];
			foreach($_REQUEST['we_' . $this->Name . '_' . self::TYPE_HREF] as $k => $val){
				if(preg_match('|^(.+)' . we_base_link::MAGIC_INFIX . '(.+)$|', (string) $k, $match)){
					$hrefs[$match[1]][$match[2]] = $val;
				}
			}

			foreach($hrefs as $k => $v){
				//remove path
				$href = array_merge($empty, $v);
				unset($href['intPath']);
				$this->setElementA($k, $href, self::TYPE_HREF);
			}
		}

		if($fields[self::TYPE_IMG]){
			foreach($_REQUEST['we_' . $this->Name . '_' . self::TYPE_IMG] as $k => $val){
				$this->setElementI((string) $k, (int) $val, self::TYPE_IMG);
			}
		}

		if($fields[self::TYPE_MULTIOBJECT]){
			$multiobjects = [];
			foreach($this->nextElement(self::TYPE_MULTIOBJECT) as $k){
				$multiobjects[(string) $k] = [];
			}

			$match = [];
			foreach($_REQUEST['we_' . $this->Name . '_' . self::TYPE_MULTIOBJECT] as $k => $val){
				if(preg_match('|^(.+)_default(.+)$|', (string) $k, $match)){
					$multiobjects[$match[1]][$match[2]] = (int) $val;
				}
			}

			foreach($multiobjects as $realName => $data){
				ksort($data, SORT_NUMERIC);
				$this->setElementS($realName, implode(',', $data), self::TYPE_MULTIOBJECT);
			}
		}
	}

	function userCanSave(bool $ctConditionOk = false): bool{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return true;
		}
		if(!we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE])){
			return false;
		}
		if(!$this->RestrictOwners){
			return true;
		}

		$readers = array_keys(array_filter($this->OwnersReadOnly));
		return !we_users_util::isUserInUsers(we_users_user::getID(), $readers);
	}

	/**
	 * @return bool
	 * @desc	checks if the user has the right to see an objectfile
	 */
	function userHasPerms(): bool{
		return (we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE]) && (!$this->RestrictOwners) || we_users_util::isOwner($this->Owners) || we_users_util::isOwner([
				$this->CreatorID]));
	}

	/**
	 * checks if this object can have variants
	 *
	 * if paramter checkField is true, this function checks also, if there are
	 * already fields selected for the variants.
	 *
	 * @return boolean
	 */
	function canHaveVariants(bool $checkFields = false): bool{
		if(!$checkFields){
			return true;
		}
		$object = new we_contents_object($this->TableID);

		return !empty($object->getVariantFields());
	}

	/**
	 * @return array<string,array{type:string,content:string,attributes:array}> with the filed names as keys and attributes as values
	 */
	function getVariantFields(): array{
		$object = new we_contents_object($this->TableID);
		return $object->getVariantFields();
	}

	function downSelectAtObject(string $name, int $i): void{
		$old = we_unserialize($this->getElementS($name));
		$objects = $old['objects'] ?? $old;
		$temp = $objects[($i + 1)];
		$objects[($i + 1)] = $objects[$i];
		$objects[$i] = $temp;

		$this->setElementS($name, implode(',', $objects), self::TYPE_MULTIOBJECT);
	}

	function upSelectAtObject(string $name, int $i): void{
		$old = we_unserialize($this->getElementS($name));
		$objects = $old['objects'] ?? $old;
		$temp = $objects[($i - 1)];
		$objects[($i - 1)] = $objects[$i];
		$objects[$i] = $temp;
		$this->setElementS($name, implode(',', $objects), self::TYPE_MULTIOBJECT);
	}

	function addSelectToObject(string $name, int $pos): void{
		$amount = 1;
		$old = we_unserialize($this->getElementS($name));
		$objects = $old['objects'] ?? $old;
		for($i = count($objects) + $amount - 1; 0 <= $i; $i--){
			if(($pos + $amount) < $i){
				$objects[$i] = $objects[($i - $amount)];
			} else if(($pos < $i && $i <= ($pos + $amount)) || count($objects) === 0){
				$objects[$i] = 0;
			}
		}
		$this->setElementS($name, implode(',', $objects), self::TYPE_MULTIOBJECT);
	}

	function removeSelectFromObject(string $name, int $nr): void{
		$old = we_unserialize($this->getElementS($name));
		$objects = $old['objects'] ?? $old;
		for($i = 0; $i < count($objects) - 1; $i++){
			if($i >= $nr){
				$objects[$i] = $objects[($i + 1)];
			}
		}
		unset($objects[$i]);
		$this->setElementS($name, implode(',', $objects), self::TYPE_MULTIOBJECT);
	}

	function checkAndCorrectParent(): void{
		if($this->ParentID == 0){
			$this->resetParentID();
		}
		if(!str_starts_with($this->ParentPath . '/', $this->RootDirPath . '/')){
			$this->resetParentID();
		}
	}

	/**
	 *
	 * @return int[]
	 */
	protected function getNavigationFoldersForDoc(): array{
		$category = array_map('escape_sql_query', array_unique(array_filter(array_merge($this->Category, $this->oldCategory))));

		$queries = ['(Selection="' . we_navigation_navigation::SELECTION_STATIC . '" AND SelectionType="' . we_navigation_navigation::STYPE_OBJLINK . '"  AND LinkID=' . intval($this->ID) . ')',
			//FIXME: query should use ID, not parentID
			'(Selection="' . we_navigation_navigation::SELECTION_DYNAMIC . '" AND DynamicSelection="' . we_navigation_navigation::DYN_CLASS . '" AND ClassID=' . $this->TableID . ')'
		];
		if($category){
			//FIXME: query should use ID, not parentID
			$queries[] = '( Selection="' . we_navigation_navigation::SELECTION_DYNAMIC . '" AND DynamicSelection="' . we_navigation_navigation::DYN_CLASS . '" AND (CONCAT(",",Categories,",") REGEXP ",(' . implode('|', $category) . '),"))';
		}
		//@phan-suppress-next-line PhanPartialTypeMismatchReturn
		return $this->DB_WE->getAllq('SELECT DISTINCT ParentID FROM ' . NAVIGATION_TABLE . ' WHERE ' . implode(' OR ', $queries), true);
	}

	public static function getObjectHref(int $id, int $pid, string $path = '', bool $hidedirindex = false, bool $objectseourls = false, bool $inWebedition = false): string{
		if(!$id){
			return '';
		}

		$path = $path ?: we_http_util::getScriptName();

		$foo = getHash('SELECT `of`.Workspaces,`of`.TriggerID,f.IsPublished AS fPub FROM ' . OBJECT_FILES_TABLE . ' `of` LEFT JOIN ' . FILE_TABLE . ' f ON (`of`.TriggerID=f.ID AND f.IsDynamic=1) WHERE `of`.ID=' . $id);
		'@phan-var array{Workspaces:string,TriggerID:int,fPub:int} $foo';
		if(!$foo){
			return '';
		}
		if(!$foo['fPub']){
			//trigger document is not published - we have to find another one.
			$foo['TriggerID'] = 0;
		}

// check if object is published.
		if((!$inWebedition && !$foo['fPub'])){
			throw new we_exception_base('we_link_not_published');
		}

		$showLink = !empty($path = self::getNextDynDoc($path, $pid, explode(',', $foo['Workspaces'])));
		//note: /=0, so "0" is a valid wsp
		//note: /=0, so "0" is a valid wsp
		if(!$showLink && $foo['Workspaces'] !== ''){
			$wsp = we_base_request::filterIntList($foo['Workspaces']);
			if(we_users_util::in_workspace((int) $foo['TriggerID'], $wsp, FILE_TABLE)){
				if(!($path = id2path((int) $foo['TriggerID']))){
					return '';
				}
				$showLink = true;
			}
		}
		if($showLink){
			$pidstr = ($pid ? '?pid=' . $pid : '');
			$path_parts = pathinfo($path);

			if($objectseourls && show_SeoLinks() && ($objUrl = fS('SELECT Url FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $id))){
				return ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' .
					($hidedirindex && seoIndexHide($path_parts['basename']) ?
					'' :
					$path_parts['filename'] . '/' ) .
					$objUrl . $pidstr;
			}
			if($hidedirindex && !(we_base_WE::$editmode || we_base_WE::$mainDoc['editmode'] )){
				if(seoIndexHide($path_parts['basename'])){
					$path = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
				}
			}
			return $path . '?' . WEOBJ_PARAM . '=' . $id . str_replace('?', '&amp;', $pidstr);
		}
		if($foo['Workspaces']){
			$path = self::getNextDynDoc('', $pid, explode(',', $foo['Workspaces']));
			return ($path ? $path . '?' . WEOBJ_PARAM . '=' . $id . '&pid=' . $pid : '');
		}

		return '';
	}

	//Fix: #10219 leave this public while using in redirectSEOurls.php!
	public static function getNextDynDoc(string $path, int $pid, array $ws1): string{
		if($path && fI('SELECT IsDynamic FROM ' . FILE_TABLE . ' WHERE IsPublished=1 AND ContentType="' . we_base_ContentTypes::WEDOCUMENT . '" AND Path="' . DB_WE::$inst->escape($path) . '" LIMIT 1')){
			return $path;
		}

		return (we_users_util::in_workspace($pid, we_base_request::filterIntList($ws1), FILE_TABLE) ?
			fS('SELECT Path FROM ' . FILE_TABLE . ' WHERE IsPublished=1 AND ContentType="' . we_base_ContentTypes::WEDOCUMENT . '" AND IsDynamic=1 AND Path LIKE "' . id2path($pid, FILE_TABLE) . '%" ORDER BY CHAR_LENGTH(Path) LIMIT 1') :
			'');
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd): string{
		$parts = new we_html_multiIconBoxContent();
		switch($this->EditPageNr){
			default:
				$parts
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[path]'), $this->formPath(), we_html_multiIconBox::PROP_PATH)))
					->add((new we_html_multiIconBoxContentLine(g_l('modules_object', '[class]'), $this->formClass(), we_html_multiIconBox::PROP_CLASS))->noline()->fold()->hide(
							// No link to class in normal mode
							we_editor_functions::isNormalMode() || we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE])
					))
					->add((new we_html_multiIconBoxContentLine(g_l('modules_object', '[class]'), $this->formClass() . (we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE]) ? ' (<a href="javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.OBJECT_TABLE,' . $this->TableID . ',\'object\');">' . g_l('modules_object', '[class_id]') . ': ' . $this->formClassId() . '</a>)' : ''), we_html_multiIconBox::PROP_CLASS))->rightHead()->hide(we_editor_functions::isSeeMode()) // Link to class in normal mode
					)
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[language]'), $this->formLangLinks($this->rootDirID), we_html_multiIconBox::PROP_LANG))->fold())
					->add(we_modules_info::isActive(we_modules_info::CATEGORY) ? (new we_html_multiIconBoxContentLine(g_l('global', '[categorys]'), $this->formCategory($jsCmd), we_html_multiIconBox::PROP_CATEGORIES))->fold() : null)
					->add((new we_html_multiIconBoxContentLine(g_l('modules_object', '[copyObject]'), $this->formCopyDocument(), we_html_multiIconBox::PROP_COPY)))
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[owners]'), $this->formCreatorOwners($jsCmd), we_html_multiIconBox::PROP_USER))->fold())
				;
				break;
			case we_base_constants::WE_EDITPAGE_WORKSPACE:
				if($this->hasWorkspaces()){ // Show workspaces
					$parts
						->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[workspaces]'), $this->formWorkspaces($jsCmd), we_html_multiIconBox::PROP_WORKSPACE))->noline())
						->add((new we_html_multiIconBoxContentLine('', we_html_button::create_button('ws_from_class', "javascript:we_cmd('we_object_class_getClassWs');_EditorFrame.setEditorIsHot(true);"))))
					;
				} else { //	No workspaces defined
					$parts
						->add((new we_html_multiIconBoxContentLine('', g_l('modules_object', '[no_workspace_defined]'))));
				}
		}
		return (new we_html_multiIconBox($parts, 'PropertyPage'))->marginLeft()->get();
	}

	public static function makePIDTail(int $pid, int $cid): string{
		static $wsFlag = [];
		$parentIDs = [$pid];

		while($pid != 0){
			$pid = fI('SELECT ParentID FROM ' . FILE_TABLE . ' WHERE ID=' . $pid);
			$parentIDs[] = $pid;
		}
		if(isset($wsFlag[$cid])){
			$flag = $wsFlag[$cid];
		} else {
			$fooArr = we_unserialize(fS('SELECT DefaultValues FROM ' . OBJECT_TABLE . ' WHERE ID=' . $cid));
			$wsFlag[$cid] = $flag = ($fooArr['WorkspaceFlag'] ?? 1);
		}
		$pid_tail = [];
		if($flag){
			$pid_tail[] = '`of`.Workspaces=""';
		}
		$pid_tail[] = 'CONCAT(",",`of`.Workspaces,",") REGEXP ",(' . implode('|', $parentIDs) . '),"';
		return ' (' . implode(' OR ', $pid_tail) . ') ';
	}

	public function mustCheckEditPageArray(): bool{
		return true;
	}

	/** FIXME: remove in cardada
	 * @deprecated since barrhorn
	 */
	protected function initSerializedDat(array $sessDat): bool{
		if(empty($sessDat)){
			$this->Name = md5(uniqid(__FUNCTION__, true));
			return false;
		}

		if(isset($sessDat[1])){
			$this->elements = $sessDat[1];
		}
		$this->Name = md5(uniqid(__FUNCTION__, true));
		return true;
	}

	/** FIXME: remove in cardada
	 * @deprecated since barrhorn
	 */
	public function we_updaterTransferTempDoc(array $serData): void{
		$this->initSerializedDat($serData); //@phan-suppress-current-line PhanDeprecatedFunction
		$this->getPersistentSlotsFromDB('TableID,Publish_Date,Text,Path,ParentID');
		$this->getUniqueIDsAndFixNames();
		$this->setTypeAndLength();
		$this->saveContentDataInDB(TEMPORARY_DOCTARGET);
	}

	/**
	 *
	 * @return array{id:int,type:string,req:array}
	 */
	public function getShopArticleID(): array{
		return ['id' => $this->ID, 'type' => we_shop_shop::OBJECT, 'req' => []];
	}

	/**
	 * inits weDocumentCustomerFilter from db regarding the modelId
	 * is called from "we_textContentDocument::we_load"
	 * @see we_document_textContent::we_load
	 */
	protected function initWeDocumentCustomerFilterFromDB(): void{
		if(!we_modules_info::isActive(we_modules_info::CUSTOMER)){
			return;
		}
		$this->documentCustomerFilter = we_customer_documentFilter::getFilterByIdAndTable($this->ID, OBJECT_FILES_TABLE);
	}

	/**
	 * @return int[]
	 */
	public function getClassTemplates(): array{
		return we_base_request::filterIntList($this->classData['Templates']);
	}

	public function getFrameLoadCode(): string{
		return parent::getFrameLoadCode() .
			SIDEBAR_DISABLED || !$this->classData['SidebarDocID'] ? '' : "WE().layout.sidebar.open(''," . $this->classData['SidebarDocID'] . ")";
	}

}
