<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d3fd95ad9ec4 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-10 11:02 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_customer_frames extends we_modules_frame{
	/** @var array<string,string> */
	private $jsOut_fieldTypesByName;

	/** @var we_customer_view */
	protected $View;

	/** @var we_tree_customer */
	protected $Tree;

	/** @var int */
	private $advSearchFoundItems = 0;

	public function __construct(){
		$this->module = we_modules_info::CUSTOMER;
		parent::__construct();
		$this->showTreeHeader = true;
		$this->showTreeFooter = true;

		$this->Tree = new we_tree_customer($this->jsCmd);
		$this->View = new we_customer_view();
	}

	public function getHTML(string $what = '', int $mode = 0, int $step = 0): string{
		switch($what){
			case self::FOOTER:
				return $this->getHTMLEditorFooter([
						'main' => [
							we_html_button::FOOTER_SAVE => [[we_base_perm::CUSTOMER_EDIT, we_base_perm::CUSTOMER_NEW], ['we_customer_save']],
						],
						'optional' => [
							we_html_button::FOOTER_TRASH => [[we_base_perm::CUSTOMER_DELETE], ['we_customer_delete']]
						]
				]);
			case 'customer_admin':
				return $this->getHTMLCustomerAdmin();
			case 'branch_editor':
				return $this->getHTMLFieldEditor('branch', $mode);
			case 'field_editor':
				return $this->getHTMLFieldEditor('field', $mode);
			case 'sort_admin':
				return $this->View->getHTMLSortEditor($this);
			case 'settings':
				return $this->getHTMLSettings();
			case 'frameset':
				$this->View->Model->clearSessionVars();
				$this->View->settings->load();
				//FIXME: this causes the tree to load twice!
				return $this->getHTMLFrameset($this->Tree->getJSTreeCode(), ($sid = we_base_request::int('sid', -1)) !== -1 ? '&sid=' . $sid : '');
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	//TODO: move editor-body relatetd stuff to weCustomerView! => note dependencies on jsOut_fieldTypesByName
	function getHTMLFieldsSelect(string $branch): we_html_select{
		$select = new we_html_select(['name' => 'branch']);

		$fields_names = $this->View->Model->getFieldsNames($branch);
		$this->jsOut_fieldTypesByName = [];
		foreach($fields_names as $val){
			$tmp = $this->View->getFieldProperties(($branch ? $branch . '_' : '') . $val);
			$this->jsOut_fieldTypesByName[$val] = ($tmp['type'] ?? '');
		}
		if($fields_names){
			foreach($fields_names as $k => $field){
				$field = (string) $field;
				$select->addOption((string) $k, (we_customer_customer::isProperty($field) ?
						$this->View->settings->getPropertyTitle($field) :
						$field)
				);
			}
		}

		return $select;
	}

	protected function getHTMLEditorHeader(): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorHeader();
		}

		$tabs = new we_gui_tabs();

		$branches_names = $this->View->Model->getBranchesNames();

		$tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, false, '\'' . g_l('modules_customer', '[common]') . '\'', ['id' => "common", 'title' => g_l('modules_customer', '[common]')]);
		$allTabs = [g_l('modules_customer', '[common]') => 'common'];
		$branchCount = 0;
		foreach($branches_names as $branch){
			$tabs->addTab('', 'fa-object-group', false, '\'' . $branch . '\'', ['id' => 'branch_' . $branchCount, 'title' => $branch]);
			$allTabs[$branch] = 'branch_' . $branchCount;
			$branchCount++;
		}
		$tabs->addTab('', 'fa-object-ungroup', false, '\'' . g_l('modules_customer', '[other]') . '\'', ['id' => 'other', 'title' => g_l('modules_customer', '[other]')]);
		$tabs->addTab('', 'fa-list', false, '\'' . g_l('modules_customer', '[all]') . '\'', ['id' => 'all', 'title' => g_l('modules_customer', '[all]')]);
		$allTabs[g_l('modules_customer', '[other]')] = 'other';
		$allTabs[g_l('modules_customer', '[all]')] = 'all';
//((top.content.activ_tab=="' . g_l('modules_customer','[other]') . '") )

		if(we_modules_info::isActive(we_modules_info::SHOP)){
			$tabs->addTab('', 'fa-shopping-cart', false, '\'' . g_l('modules_customer', '[orderTab]') . '\'', ['id' => 'orderTab', 'title' => g_l('modules_customer', '[orderTab]')]);
			$allTabs[g_l('modules_customer', '[orderTab]')] = 'orderTab';
		}
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$tabs->addTab('', 'fa-file-o', false, '\'' . g_l('modules_customer', '[objectTab]') . '\'', ['id' => 'objectTab', 'title' => g_l('modules_customer', '[objectTab]')]);
			$allTabs[g_l('modules_customer', '[objectTab]')] = 'objectTab';
		}
		$tabs->addTab('', 'fa-file', false, '\'' . g_l('modules_customer', '[documentTab]') . '\'', ['id' => 'documentTab', 'title' => g_l('modules_customer', '[documentTab]')]);
		$allTabs[g_l('modules_customer', '[documentTab]')] = 'documentTab';

		$text = $this->View->Model->Username;

		//TODO: we have the following body in several modules!
		$body = we_html_element::body(['onresize' => 'weTabs.setFrameSize()', 'onload' => 'loaded();', 'id' => 'eHeaderBody',], we_html_element::div(['id' => 'main'], we_html_element::div([
						'id' => 'headrow'], we_html_element::b(str_replace(' ', '&nbsp;', g_l('modules_customer', '[customer]')) . ':&nbsp;') .
						we_html_element::span(['id' => 'h_path', 'class' => 'header_small'], '<b id="titlePath">' . str_replace(' ', '&nbsp;', $text) . '</b> (ID:' . $this->View->Model->ID . ')'
						)
					) .
					$tabs->getHTML()
				)
		);

		return $this->getHTMLDocument($body, we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
				we_html_element::jsScript(JS_DIR . 'initTabs.js') .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_top.js', '', ['id' => 'loadVarCustomerHeader', 'data-customerHeader' => setDynamicVar($allTabs)])
		);
	}

	protected function getHTMLEditorBody(): string{
		$hiddens = ['cmd' => 'we_customer_edit', 'pnt' => 'edbody', 'activ_sort' => 0];

		if(we_base_request::bool('home')){
			return $this->View->getHomeScreen();
		}

		$branch = we_base_request::string('branch', g_l('modules_customer', '[common]'));

		return $this->getHTMLDocument(we_html_element::body(
					['class' => 'weEditorBody', 'onload' => 'loaded=1', 'onunload' => 'doUnload()'], we_html_element::form(
						['name' => we_base_constants::WE_FORM, 'autocomplete' => 'off'], $this->View->getCommonHiddens($hiddens) .
						$this->View->getHTMLProperties($branch))), $this->View->getJSProperty());
	}

	protected function getHTMLTreeHeader(): string{
		$select = $this->View->getHTMLSortSelect();
		$select->setAttributes(['onchange' => 'applySort();', 'style' => 'width:150px']);
		$select->selectOption($this->View->settings->getSettings('default_sort_view'));

		return we_html_element::form(['name' => 'we_form_treeheader'], we_html_element::hiddens([
					'pnt' => 'treeheader',
					'pid' => '0',
					'cmd' => 'no_cmd']) .
				$select->getHtml() .
				we_html_button::create_button(we_html_button::RELOAD, "javascript:applySort();") .
				we_html_button::create_button(we_html_button::EDIT, "javascript:we_cmd('show_sort_admin')")
		);
	}

	protected function getHTMLTreeFooter(): string{
		return $this->getHTMLSearchTreeFooter();
	}

	private function getHTMLCustomerAdmin(): string{
		$branch = we_base_request::string("branch", g_l('modules_customer', '[other]'));
		$branch_select = we_base_request::string("branch", g_l('modules_customer', '[other]'));

		$select = $this->View->getHTMLBranchSelect(false);
		$select->setAttributes(['name' => "branch_select", 'class' => 'weSelect', 'onchange' => "selectBranch()", 'style' => "width:150px;"]);
		$select->selectOption($branch_select);

		$fields = $this->getHTMLFieldsSelect($branch);
		$fields->setAttributes(['name' => 'fields_select', 'size' => 15, 'onchange' => 'setFieldsButtons(this.selectedIndex,this.length);', 'style' => 'width:350px;height:250px;']);
		//$hiddens = rray('name' => "field", 'value' => ''));

		$buttons_table = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('open_add_field')") .
			we_html_button::create_button(we_html_button::EDIT, "javascript:we_cmd('open_edit_field')", '', '', true, false, '', false, '', '', 'editFieldButton') .
			we_html_button::create_button(we_html_button::DELETE, "javascript:we_cmd('delete_field')", '', '', true, false, '', false, '', '', 'deleteFieldButton') .
			we_html_button::create_button(we_html_button::DIRUP, "javascript:we_cmd('move_field_up')", '', '', true, false, '', false, '', '', 'moveFieldUpButton') .
			we_html_button::create_button(we_html_button::DIRDOWN, "javascript:we_cmd('move_field_down')", '', '', true, false, '', false, '', '', 'moveFieldDownButton');

		$table = (new we_html_table(['class' => 'default', 'width' => '500'], 4, 5))
			->setCol(0, 0, g_l('modules_customer', '[branch]'), ['class' => "defaultfont lowContrast", 'style' => 'padding-right:10px;'])
			->setCol(0, 2, g_l('modules_customer', '[branch_select]'), ['class' => "defaultfont lowContrast"])
			->setCol(1, 0, we_html_tools::textInput('branch', $branch, 0, ['style' => "width:350px;"]), ['style' => 'padding-right:10px;'])
			->setCol(1, 2, $select->getHtml(), ['style' => 'padding-right:10px;'])
			->setCol(1, 4, we_html_button::create_button(we_html_button::EDIT, "javascript:we_cmd('open_edit_branch')"), ['style' => 'padding-bottom:10px;'])
			->setCol(2, 0, g_l('modules_customer', '[fields]'), ['class' => "defaultfont lowContrast", 'style' => 'vertical-align:top;'])
			->setCol(3, 0, $fields->getHtml(), ['style' => 'vertical-align:top;padding-right:10px;'])
			->setCol(3, 2, $buttons_table, ['style' => 'vertical-align:top;']);

		return $this->getHTMLDocument(
				we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();'], we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_admin.js') .
					we_html_element::form(['name' => we_base_constants::WE_FORM], we_html_element::hiddens([
							'cmd' => 'switchBranch',
							'pnt' => 'customer_admin']) .
						we_html_tools::dialogLayout($table->getHtml(), g_l('modules_customer', '[field_admin]'), we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose()"))
					)
				)
		);
	}

	private function getHTMLFieldEditor(string $type, int $mode): string{
		$field = we_base_request::string('field', '');
		$branch = we_base_request::string('branch', g_l('modules_customer', '[other]'));

		$hiddens = we_html_element::hiddens([
				'pnt' => 'field_editor',
				'cmd' => 'no_cmd',
				'branch' => $branch,
				'art' => (string) $mode,
				($type === 'field' ? 'field' : '') => $field]);

		$cancel = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();");
		$curType = '';

		switch($type){
			case 'branch':
				$hiddens .= we_html_element::hidden('pnt', 'branch_editor');
				$edit = (new we_html_table(['width' => '300'], 1, 2))
					->setCol(0, 0, g_l('modules_customer', '[field_name]'), ['style' => 'vertical-align:middle;', 'class' => "defaultfont lowContrast"])
					->setCol(0, 1, we_html_tools::textInput("name", $branch), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont']);

				$save = we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_branch')");
				break;
			default:
				$hiddens .= we_html_element::hidden('pnt', 'field_editor');
				$field_props = $this->View->getFieldProperties($field);

				$types = new we_html_select(['name' => 'field_type', 'class' => 'weSelect', 'style' => "width:200px;", 'onchange' => 'setStatusEncryption(this.value);']);
				$types->addOptions(array_combine(array_keys(we_customer_settings::FIELD_TYPES), array_keys(we_customer_settings::FIELD_TYPES)));
				if(isset($field_props['type'])){
					$types->selectOption($field_props['type']);
					$curType = $field_props['type'];
				} else {
					$curType = 'input';
				}

				$enc = (new we_html_select(['name' => "field_encrypt", 'class' => "weSelect", 'style' => "width:200px;"]))
					->addOptions([0 => g_l('global', '[no]'), 1 => g_l('global', '[yes]')])
					->selectOption(!empty($field_props['encrypt']) ? 1 : 0);

				$edit = (new we_html_table(['width' => '300'], 5, 2))
					->setCol(0, 0, g_l('modules_customer', '[branch]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
					->setCol(0, 1, $branch, ['style' => 'vertical-align:middle;', 'class' => 'defaultfont'])
					->setCol(1, 0, g_l('modules_customer', '[field_name]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
					->setCol(1, 1, we_html_tools::textInput('name', ($field_props['name'] ?? '')), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont'])
					->setCol(2, 0, g_l('modules_customer', '[field_type]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
					->setCol(2, 1, $types->getHtml(), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont'])
					->setCol(3, 0, g_l('modules_customer', '[field_default]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
					->setCol(3, 1, we_html_tools::textInput("field_default", ($field_props['default'] ?? '')), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont'])
					->setCol(4, 0, g_l('modules_customer', '[encryptField]'), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont lowContrast'])
					->setCol(4, 1, $enc->getHtml(), ['style' => 'vertical-align:middle;', 'class' => 'defaultfont']);

				$save = we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_field')");
		}

		return
			$this->getHTMLDocument(
				we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();setStatusEncryption(\'' . $curType . '\');'], we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_admin.js') .
					we_html_element::form(['name' => we_base_constants::WE_FORM], $hiddens .
						we_html_tools::dialogLayout($edit->getHtml(), (
							$type === 'branch' ?
							(g_l('modules_customer', '[edit_branche]')) :
							g_l('modules_customer', ($mode === 2 ? '[edit_field]' : '[add_field]'))
							), we_html_button::position_yes_no_cancel($save, '', $cancel)
						)
					)
				)
		);
	}

	function getHTMLBox(string $content, string $headline = '', int $width = 100, int $height = 50, int $w = 25, int $vh = 0, int $ident = 0, int $space = 5, string $headline_align = "left", string $content_align = "left"): string{
		$table = (new we_html_table(['width' => $width . 'px', 'height' => $height . 'px', 'class' => 'default', 'style' => 'margin-left:' . $ident . 'px;margin-top:' . $vh . 'px;margin-bottom:' . ($w && $headline ? $vh : 0) . 'px;'], 1, 2))
			->setCol(0, 0, str_replace(' ', '&nbsp;', $headline), ['style' => 'vertical-align:middle;text-align:' . $headline_align . ';padding-right:' . $space . 'px;',
				'class' => 'defaultfont lowContrast'])
			->setCol(0, 1, $content, ['style' => 'vertical-align:middle;text-align:' . $content_align]);
		return $table->getHtml();
	}

	protected function getHTMLCmd(): string{
		if(($pid = we_base_request::int('pid', -1)) === -1){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$sortField = we_base_request::string('sort', we_base_request::NOT_VALID_S);

		if($sortField === we_base_request::NOT_VALID_S){
			$sortField = $this->View->settings->getSettings('default_sort_view');
		}
		we_tree_customer::setSort(empty($this->View->settings->SortView[$sortField]) ? '' : $sortField);

		$offset = we_base_request::int('offset', 0);

		if(we_base_request::string('error')){
			$this->jsCmd->addMsg(g_l('modules_customer', '[error_download_failed]'), we_base_util::WE_MESSAGE_ERROR);
		}

		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_tree_customer::getItems($pid, $offset, $this->Tree->default_segment), 'sorted' => false]);

		return $this->getHTMLDocument(
				we_html_element::body([], we_html_element::form(['name' => we_base_constants::WE_FORM], we_html_element::hiddens([
							'pnt' => 'cmd',
							'cmd' => 'no_cmd'])
					)
				)
		);
	}

	function getHTMLSearch(): string{//TODO: this is popup search editor: make separate frameset for popups!
		$colspan = 4;

		$mode = we_base_request::int('mode', 0);

		$hiddens = we_html_element::hiddens([
				'pnt' => 'search',
				'cmd' => 'search',
				'search' => '1',
				'mode' => (string) $mode
		]);

		$search_but = we_html_button::create_button(we_html_button::SEARCH, "javascript:we_cmd('search')");

		$search = (new we_html_table(['class' => 'default', 'width' => '550', 'height' => '50'], 4, 3))
			->setRowAttributes(0, ['style' => 'vertical-align:top'])
			->setCol(0, 0, g_l('modules_customer', '[search_for]'), ['class' => 'defaultfont', 'colspan' => '3', 'style' => 'padding-bottom: 3px;']);

		$select = new we_html_select(['name' => 'search_result', 'style' => 'width:550px;', 'onDblClick' => "WE().util.dialogTop(window).opener.top.content.we_cmd('we_customer_edit',document.we_form.search_result.options[document.we_form.search_result.selectedIndex].value)",
			"size" => 20]);

		if($mode){
			$this->getHTMLAddSearch($search, $select);
			$foundItems = $this->advSearchFoundItems;
		} else {
			$search->setCol(1, 0, we_html_tools::textInput('keyword', we_base_request::string('keyword', ''), 0, [
					], 'text', '550px'), ['style' => 'padding-bottom:5px;']
			);

			$sw = we_html_button::create_button(we_html_button::DIRRIGHT, "javascript:we_cmd('switchToAdvance')");

			$search->setCol(3, 0, we_html_element::div(['class' => 'defaultfont'], g_l('modules_customer', '[advanced_search]')) .
				$sw .
				$search_but, ['style' => 'text-align:right', 'colspan' => $colspan]
			);
			$hiddens .= we_html_element::hidden('count', 1);

			$max_res = we_customer_settings::MaxSearchResults;
			$result = (($k = we_base_request::string('keyword')) && we_base_request::bool('search') ?
				$this->View->getSearchResults($k, $max_res) :
				[]);

			$foundItems = count($result);

			foreach($result as $id => $text){
				$select->addOption((string) $id, (string) $text);
			}
		}

		$table = (new we_html_table(['width' => '550', 'height' => '50'], 3, 1))
			->setCol(0, 0, $search->getHtml())
			->setCol(1, 0, g_l('modules_customer', '[num_data_sets]') . ($foundItems ? ' (' . $foundItems . ')' : ''))
			->setCol(2, 0, $select->getHtml());
//need to call to have jsOut_fieldTypesByName filled
		$this->getHTMLFieldsSelect('');

		return $this->getHTMLDocument(
				we_html_element::body(['class' => 'weDialogBody', 'onload' => ($mode ? '' : 'document.we_form.keyword.focus();')], $this->View->getJSSearch() .
					we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_functions.js', /* (we_base_request::bool('mode') ? "setTimeout(lookForDateFields, 1);" : '') */ '', [
						'id' => 'loadVarCustomerFuntions', 'data-customerFunctions' => setDynamicVar($this->jsOut_fieldTypesByName)]) .
					we_html_element::form(['name' => we_base_constants::WE_FORM, 'onsubmit' => "we_cmd('search');return false;"], $hiddens .
						we_html_tools::dialogLayout(
							$table->getHtml(), g_l('modules_customer', '[search]'), we_html_button::position_yes_no_cancel('', we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();"))
						)
					)
				)
		);
	}

	private function getHTMLSettings(): string{
		if(we_base_request::string('cmd') === 'save_settings'){
			$this->View->processCommands($this->jsCmd);
			$closeflag = true;
		} else {
			$closeflag = false;
		}

		$default_sort_view_select = $this->View->getHTMLSortSelect();
		$default_sort_view_select->setAttributes(['name' => "default_sort_view", 'style' => 'width:200px;']);
		$default_sort_view_select->selectOption($this->View->settings->getSettings('default_sort_view'));

		$cur = 0;
		$table = (new we_html_table(['class' => 'default', 'style' => 'margin-right:10px;'], 5, 2))
			->setCol($cur, 0, g_l('modules_customer', '[default_sort_view]') . ":&nbsp;", ['style' => 'padding-right:30px;'])
			->setCol($cur, 1, $default_sort_view_select->getHtml())
			->setCol(++$cur, 0, g_l('modules_customer', '[start_year]') . ":&nbsp;", ['style' => 'padding-right:30px;'])
			->setCol($cur, 1, we_html_tools::textInput("start_year", $this->View->settings->getSettings('start_year')))
			->setCol(++$cur, 0, g_l('modules_customer', '[treetext_format]') . ":&nbsp;", ['style' => 'padding-right:30px;'])
			->setCol($cur, 1, we_html_tools::textInput("treetext_format", $this->View->settings->getSettings('treetext_format')));


		$default_order = (new we_html_select(['name' => 'default_order', 'style' => 'width:250px;', 'class' => 'weSelect']))
			->addOption('', g_l('modules_customer', '[none]'));

		foreach(we_customer_settings::ORDER_TABLE as $ord){
			$ordval = g_l('modules_customer', ($ord === 'ASC') ? '[ASC]' : '[DESC]');
			$default_order->addOption($ord, $ordval);
		}
		$default_order->selectOption($this->View->settings->getSettings('default_order'));

		$table
			->setCol(++$cur, 0, g_l('modules_customer', '[default_order]') . ':&nbsp;', ['style' => 'padding-right:30px;'])
			->setCol($cur, 1, $default_order->getHtml());

		$close = we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();");
		$save = we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_settings')");

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();'], we_html_element::form(['name' => we_base_constants::WE_FORM], we_html_tools::dialogLayout(
						we_html_element::hiddens(['pnt' => "settings", 'cmd' => '']) .
						$table->getHtml(), g_l('modules_customer', '[settings]'), we_html_button::position_yes_no_cancel($save, $close)
					)
				)
		);

		if($closeflag){
			$this->jsCmd->addCmd('close_dialog');
		}

		return $this->getHTMLDocument($body, we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/settings.js'));
	}

	public function getHTMLAddSearch(we_html_table $search, we_html_select $select): void{
		$count = we_base_request::int('count');

		$search_arr = [];

		$search_but = we_html_button::create_button(we_html_button::SEARCH, "javascript:we_cmd('search')");
		$colspan = 4;

		for($i = 0; $i < $count; $i++){
			if(we_base_request::string('branch_' . $i)){
				$search_arr[$i] = [
					'logic' => we_base_request::string('logic_' . $i),
					'branch' => we_base_request::string('branch_' . $i),
					'field' => we_base_request::string('field_' . $i),
					'operator' => we_base_request::int('operator_' . $i),
					'value' => we_base_request::string('value_' . $i),
				];
			}
		}

		$advsearch = new we_html_table([], 1, 4);
		$branch = $this->View->getHTMLBranchSelect();
		$branch->setOptionVT(1, g_l('modules_customer', '[other]'), g_l('modules_customer', '[other]'));

		$c = 0;

		foreach($search_arr as $i => $cur){
			$branch->selectOption($cur['branch']);
			$field = (!$cur['branch'] ?
				$this->getHTMLFieldsSelect(g_l('modules_customer', '[common]')) :
				$this->getHTMLFieldsSelect($cur['branch']));

			$field->selectOption($cur['field']);

			$branch->setAttributes(['name' => 'branch_' . $i, 'class' => 'weSelect', 'onchange' => "we_cmd('selectBranch')", 'style' => "width:145px"]);
			$field->setAttributes(['name' => 'field_' . $i, 'style' => "width:145px", 'class' => 'weSelect', /* 'onchange' => "isDateField(" . $i . ")" */]);

			if($i != 0){
				$advsearch->setCol($c, 0, we_html_tools::select('logic_' . $i, we_customer_view::SEARCH_LOGIC, 1, $cur['logic'], false, [], true, 70), ['colspan' => $colspan]);
				++$c;
			}
			$advsearch
				->setCol($c, 0, $branch->getHtml())
				->setCol($c, 1, $field->getHtml())
				->setCol($c, 2, we_html_tools::select('operator_' . $i, we_customer_view::SEARCH_OPERATORS, 1, $cur['operator'], false, [
						], true, 60))
				->setCol($c, 3, we_html_tools::getNewDateInput('value_' . $i, new DateTime($cur['value']), we_html_tools::FORMAT_DATE));
			++$c;
		}

		$advsearch->setCol($c, 0, we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('add_search')") .
			we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('del_search')")
			, ['colspan' => $colspan, 'style' => 'padding-top:5px;']
		);

		$search
			->setCol(1, 0, we_html_element::hidden('count', $count) . $advsearch->getHtml())
			->setCol(3, 0, "<table class='default'><tr><td>" .
				we_html_element::div(['class' => "defaultfont lowContrast"], g_l('modules_customer', '[simple_search]')) .
				we_html_button::create_button('fa:btn_direction_left,fa-lg fa-caret-left', "javascript:we_cmd('switchToSimple')") .
				$search_but
				. '</td><td>&nbsp;</td></tr></table>', ['colspan' => $colspan, 'style' => 'text-align:right;padding-top:5px;']
		);
		$max_res = we_customer_settings::MaxSearchResults;
		$result = ($search_arr && we_base_request::bool('search') ? self::getAdvSearchResults($search_arr, $max_res) : []);

		$this->advSearchFoundItems = count($result);

		foreach($result as $id => $text){
			$select->addOption((string) $id, (string) $text);
		}
	}

	/**
	 *
	 * @param array<int,array{logic:string,branch:string,field:string,operator:int,value:string}> $keywords
	 * @param int $res_num
	 * @return array
	 */
	private static function getAdvSearchResults(array $keywords, int $res_num): array{
		$where = '';

		foreach($keywords as $search){
			$search['field'] = str_replace(g_l('modules_customer', '[common]') . '_', '', $search['field']);
			$where .= ' ' . $search['logic'] . ' ' .
				$search['field'] . ' ' . we_customer_view::SEARCH_OPERATORS[$search['operator']] . " '" .
				(is_numeric($search['value']) ? (float) $search['value'] : DB_WE::$inst->escape((string) $search['value'])) .
				"'";
		}

		return DB_WE::$inst->getAllFirstq('SELECT ID,CONCAT(Username, " (",Forename," ",Surname,")") AS user FROM ' . CUSTOMER_TABLE . ' WHERE ' . (empty($where) ? 0 : $where) . ' ORDER BY Username LIMIT 0,' . $res_num, false);
	}

}
