<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d24ecdc64229 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-02 16:29 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_dialog_lang extends we_dialog_base{
	protected $isTinyDialog = true;

	protected function __construct(bool $noInternals = true){
		parent::__construct($noInternals);
		$this->onOkJsOnly = true;
		$this->changeableArgs = ['lang'];
		$this->dialogTitle = g_l('wysiwyg', '[language_title]');
		$this->noInternals = $noInternals;
		$this->defaultInit();
		$this->initByHttp();
	}

	protected function defaultInit(): void{
		$this->args = ['lang' => ''];
	}

	protected function initByHttp(): void{
		if(we_modules_info::isActive(we_modules_info::GLOSSARY) && we_base_request::bool('weSaveToGlossary') && !$this->noInternals){
			$Glossary = new we_glossary_glossary();
			$Glossary->Language = we_base_request::string('language', '');
			$Glossary->Type = we_glossary_glossary::TYPE_FOREIGNWORD;
			$Glossary->Text = trim(we_base_request::string('text'));
			$Glossary->Publish_Date = new DateTime();
			$Glossary->setAttribute('lang', we_base_request::string('we_dialog_args', '', 'lang'));
			$Glossary->setPath();

			if($Glossary->Text === '' || $Glossary->getAttribute('lang') === ''){
				$this->jsCmd->addMsg(g_l('modules_glossary', '[name_empty]'), we_base_util::WE_MESSAGE_ERROR);
			} else if($Glossary->pathExists($Glossary->Path)){
				$this->jsCmd->addMsg(g_l('modules_glossary', '[name_exists]'), we_base_util::WE_MESSAGE_ERROR);
			} else {
				$Glossary->save();
				$this->jsCmd->addMsg(g_l('modules_glossary', '[entry_saved]'), we_base_util::WE_MESSAGE_NOTICE);
				$this->jsCmd->addCmd('close');
			}
		}

		parent::initByHttp();
	}

	protected function getJs(): string{
		return parent::getJs() .
			we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'plugins/wexhtmlxtras/js/lang.js');
	}

	protected function getDialogContentHTML(): string{
		return '<table class="default">
<tr><td>' . $this->getLangField("lang", g_l('wysiwyg', '[language]'), 260) . '</td></tr>
</table>' .
			(we_modules_info::isActive(we_modules_info::GLOSSARY) && we_base_permission::hasPerm([we_base_perm::GLOSSARY_NEW]) && !$this->noInternals ?
			we_html_element::hiddens(['weSaveToGlossary' => '0',
				'language' => we_base_request::string('language', we_base_WE::$Langs['DefaultFrontendLanguage']),
				'text' => ''
			]) : ''
			);
	}

	protected function getDialogButtons(): string{
		$buttons = we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['we_dialog_args[lang]'].value='';weDoOk();");

		if(we_modules_info::isActive(we_modules_info::GLOSSARY) && we_base_permission::hasPerm([we_base_perm::GLOSSARY_NEW]) && !$this->noInternals){
			$buttons = we_html_button::create_button(we_html_button::ADD_TO_GLOSSARY, "javascript:weSaveToGlossaryFn();");
		}

		$buttons .= parent::getDialogButtons();

		return $buttons;
	}

}
