<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_import_wizardWE extends we_import_wizard{

	protected function getWizBody(): string{
		$formAttribs = [
			'name' => we_base_constants::WE_FORM,
			'enctype' => 'multipart/form-data'
		];

		$jsCmd = new we_base_jsCmd();
		$hiddens = [
			'pnt' => 'wizbody',
			'type' => we_exim_ExIm::TYPE_WE,
			'v[type]' => we_exim_ExIm::TYPE_WE,
			'step' => (string) $this->step,
			'mode' => (string) $this->mode,
			'button_state' => '0'
		];
		switch($this->step){
			case 2:
				$this->fileUploaderResp = new we_fileupload_resp_base();
				break;
			default:
				$this->fileUploader = new we_fileupload_ui_base('uploaded_xml_file', we_base_ContentTypes::getRealContentTypes(we_base_ContentTypes::XML), we_base_ContentTypes::getExtension(we_base_ContentTypes::XML));
				$this->fileUploader->setCmd('cmdFileselectOnclick', 'fileupload_doOnFileSelect');
				$this->fileUploader->setGenericFileName(TEMP_DIR . 'we_xml_' . we_fileupload::REPLACE_BY_UNIQUEID . '_w.xml');
		}
		switch($this->step){
			default:
			case 1:
				$content = $this->getStep1();
				break;
			case 2:
				$content = $this->getStep2($jsCmd);
				break;
			case 3:
				$content = $this->getStep3();
				break;
		}

		$doOnLoad = !we_base_request::bool('noload');
		return we_html_tools::getHtmlTop('', ($this->fileUploader ? $this->fileUploader->getCss() . $this->fileUploader->getJs() : '') .
				we_html_element::jsScript(JS_DIR . 'import_wizardWizbody.js') .
				$jsCmd->getCmds(), we_html_element::body(['class' => 'weDialogBody',
					'onload' => $doOnLoad ? "parent.wiz_next('wizbusy', WE().consts.dirs.WEBEDITION_DIR+'we_cmd.php?we_cmd[0]=import&pnt=wizbusy&mode=" . $this->mode . "&type=" . we_exim_ExIm::TYPE_CSV . "'); self.focus();" : ""
					], we_html_element::form($formAttribs, we_html_element::hiddens($hiddens) .
						$content
					)
				)
		);
	}

	private function getStep1(): string{
		$v = we_base_request::stringList('v');
		$doc_root = we_users_util::get_def_ws();
		$tmpl_root = we_users_util::get_def_ws(TEMPLATES_TABLE);
		$nav_root = we_users_util::get_def_ws(NAVIGATION_TABLE);

		$hdns = we_html_element::hiddens(['v[doc_dir_id]' => ($v['doc_dir_id'] ?? $doc_root),
				'v[tpl_dir_id]' => ($v['tpl_dir_id'] ?? $tmpl_root),
				'v[doc_dir]' => ($v['doc_dir'] ?? id2path($doc_root)),
				'v[tpl_dir]' => ($v['tpl_dir'] ?? id2path($tmpl_root, TEMPLATES_TABLE)),
				'v[import_from]' => ($v['import_from'] ?? 0),
				'v[navigation_dir_id]' => ($v['navigation_dir_id'] ?? $nav_root),
				'v[navigation_dir]' => ($v['navigation_dir'] ?? id2path($nav_root, NAVIGATION_TABLE)),
				'v[import_docs]' => $v['import_docs'] ?? 0,
				'v[import_templ]' => $v['import_templ'] ?? 0,
				'v[import_thumbnails]' => $v['import_thumbnails'] ?? 0,
				'v[import_objs]' => $v['import_objs'] ?? 0,
				'v[import_classes]' => $v['import_classes'] ?? 0,
				'v[restore_doc_path]' => $v['restore_doc_path'] ?? 1,
				'v[restore_tpl_path]' => $v['restore_tpl_path'] ?? 1,
				'v[import_dt]' => $v['import_dt'] ?? 0,
				'v[import_ct]' => $v['import_ct'] ?? 0,
				'v[import_binarys]' => $v['import_binarys'] ?? 0,
				'v[import_owners]' => $v['import_owners'] ?? 0,
				'v[owners_overwrite]' => $v['owners_overwrite'] ?? 0,
				'v[owners_overwrite_id]' => $v['owners_overwrite_id'] ?? 0,
				'v[owners_overwrite_path]' => $v['owners_overwrite_path'] ?? '/',
				'v[import_navigation]' => $v['import_navigation'] ?? 0,
				'v[rebuild]' => $v['rebuild'] ?? 1,
				'v[mode]' => $v['mode'] ?? 0,
				'v[btnState_next]' => 'enabled',
				'v[btnState_back]' => 'enabled'
		]);

		$importFromButton = (we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT])) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server', 'v[fserver]', 'all_Types', WE().util.dialogTop(window).wizbody.document.we_form.elements['v[fserver]'].value, 'setFormField,v[rdofloc],1,radio,0')") : '';
		$inputLServer = we_html_tools::textInput('v[fserver]', ($v['fserver'] ?? '/'), 255, ['readonly' => 'readonly'], 'text', '300px');
		$importFromServer = we_html_tools::formElementTable('', $inputLServer, $importFromButton);

		$inputLLocal = $this->fileUploader->getHTML();
		$importFromLocal = we_html_tools::formElementTable('', $inputLLocal);
		$rdoLServer = we_html_forms::radiobuttonV('lServer', $v['rdofloc'] ?? 'lServer', 'v[rdofloc]', g_l('import', '[fileselect_server]'));
		$rdoLLocal = we_html_forms::radiobuttonV('lLocal', $v['rdofloc'] ?? '', 'v[rdofloc]', g_l('import', '[fileselect_local]'));
		$importLocs = (new we_html_table(['class' => 'default'], 5, 1))
			->setCol(0, 0, $rdoLServer)
			->setCol(1, 0, $importFromServer)
			->setCol(3, 0, $rdoLLocal, ['style' => 'padding-top:4px;'])
			//$importLocs->setCol(4, 0, $this->fileUploader->getHtmlAlertBoxes());
			->setCol(4, 0, $importFromLocal);

		$fn_colsn = implode('', [
			we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_REPLACE, $v['collision'], 'v[collision]', g_l('import', '[replace]'), true, 'defaultfont', '', false, g_l('import', '[replace_txt]'), we_html_tools::TYPE_HELP),
			we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_RENAME, $v['collision'], 'v[collision]', g_l('import', '[rename]'), true, 'defaultfont', '', false, g_l('import', '[rename_txt]'), we_html_tools::TYPE_HELP),
			we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_SKIP, $v['collision'], 'v[collision]', g_l('import', '[skip]'), true, 'defaultfont', '', false, g_l('import', '[skip_txt]'), we_html_tools::TYPE_HELP)
		]);

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('import', '[import]'), $importLocs->getHTML()))
				->space(we_html_multiIconBox::SPACE_MED))
			->add((new we_html_multiIconBoxContentLine(g_l('import', '[file_collision]') . we_html_tools::alertAttentionBox(g_l('import', '[collision_txt]'), we_html_tools::TYPE_HELP), $fn_colsn))
			->space(we_html_multiIconBox::SPACE_MED)
		);

		$znr = -1;
		return $hdns . (new we_html_multiIconBox($parts, we_exim_ExIm::TYPE_WE))->marginLeft()
				->fold($znr, false, g_l('weClass', '[moreProps]'), g_l('weClass', '[lessProps]'))
				->headline(g_l('import', '[wxml_import]'))
				->get();
	}

	private function getStep2(we_base_jsCmd $jsCmd): string{
		$v = we_base_request::stringList('v');

		$hdnsBtnStates = we_html_element::hiddens([
				'v[btnState_next]' => ((isset($v['mode']) && $v['mode'] == 1) ? 'disabled' : 'enabled'),
				'v[btnState_back]' => 'enabled'
		]);

		try{
			if($v['rdofloc'] === 'lLocal'){
				$v['import_from'] = $this->fileUploaderResp->commitUploadedFile();
			}
		} catch (we_exception_file $e){
			$jsCmd->addMsg($e->getMessage(), we_base_util::WE_MESSAGE_ERROR);
			$jsCmd->addCmd('we_cmd', ['handle_event', 'previous']);
			return $hdnsBtnStates . $jsCmd->getCmds();
		}

		$hdns = we_html_element::hiddens([
				'v[type]' => $v['type'],
				'v[mode]' => $v['mode'] ?? 0,
				'v[fserver]' => $v['fserver'],
				'v[rdofloc]' => $v['rdofloc'],
				'v[import_from]' => $v['import_from'],
				'v[collision]' => $v['collision'] ?? 0,
		]);

		$import_file = WE_DOCUMENT_ROOT . $v['import_from'];
		if(we_backup_util::getFormat($import_file) != 'xml'){
			$jsCmd->addMsg(g_l('import', '[format_unknown]'), we_base_util::WE_MESSAGE_ERROR);
			$jsCmd->addCmd('we_cmd', ['handle_event', 'previous']);
			return $hdnsBtnStates . $jsCmd->getCmds();
		}
		$xml_type = we_backup_util::getXMLImportType($import_file);
		switch($xml_type){
			case 'backup':
				if(we_base_permission::hasPerm([we_base_perm::EXIM_IMPORT])){
					$jsCmd->addCmd('confirm_start_recoverBackup');
					return $jsCmd->getCmds();
				}
				$jsCmd->addMsg(g_l('import', '[backup_file_found]'), we_base_util::WE_MESSAGE_ERROR);
				$jsCmd->addCmd('we_cmd', ['handle_event', 'previous']);
				return $hdnsBtnStates . $jsCmd->getCmds();

			case 'customer':
				$jsCmd->addMsg(g_l('import', '[customer_import_file_found]'), we_base_util::WE_MESSAGE_ERROR);
				$jsCmd->addCmd('we_cmd', ['handle_event', 'previous']);
				return $hdnsBtnStates . $jsCmd->getCmds();
			case 'unreadble':
				$jsCmd->addMsg(g_l('backup', '[file_not_readable]'), we_base_util::WE_MESSAGE_ERROR);
				$jsCmd->addCmd('we_cmd', ['handle_event', 'previous']);
				return $hdnsBtnStates . $jsCmd->getCmds();
			case 'unknown':
				$jsCmd->addMsg(g_l('import', '[format_unknown]'), we_base_util::WE_MESSAGE_ERROR);
				$jsCmd->addCmd('we_cmd', ['handle_event', 'previous']);
				return $hdnsBtnStates . $jsCmd->getCmds();
		}

		$parts = new we_html_multiIconBoxContent();

		$rootDirID = we_users_util::get_def_ws();
		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('DocPath')
			->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT])
			->setInput('v[doc_dir]', ($v['doc_dir'] ?? id2path($rootDirID)), ['onfocus' => "WE().util.dialogTop(window).setFormField('_v[restore_doc_path]', false, 'checkbox');"])
			->setMaxResults(10)
			->setRequired(true)
			->setResult('v[doc_dir_id]', (string) (($v['doc_dir_id'] ?? $rootDirID)))
			->setSelector(we_gui_suggest::DirSelector)
			->setTable(FILE_TABLE)
			->setWidth(280)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:WE().util.dialogTop(window).wizbody.document.we_form.elements['v[doc_dir_id]'].value,table:'" . FILE_TABLE . "',targetID:'v[doc_dir_id]',targetName:'v[doc_dir]',rootDirID:'" . $rootDirID . "'})"));


		$docPath = $weSuggest->getHTML();

		$dir_table = new we_html_table(['id' => 'doc_table', 'style' => 'margin-left:20px;'], 3, 2);
		if((isset($v['import_docs']) && !$v['import_docs'])){
			$dir_table->setStyle('display', 'none');
		}
		$dir_table
			->setCol(0, 0, we_html_tools::alertAttentionBox(g_l('import', '[documents_desc]'), we_html_tools::TYPE_ALERT, 390, true, 50))
			->setCol(1, 0, $docPath)
			->setCol(2, 0, we_html_forms::checkboxWithHidden((!empty($v['restore_doc_path'])), 'v[restore_doc_path]', g_l('import', '[maintain_paths]'), false, "defaultfont", "self.document.we_form.elements['v[doc_dir]'].value='/';"));

		$tbl_extra = (new we_html_table([], 5, 1))

			// import documents
			->setCol(0, 0, we_html_forms::checkboxWithHidden((!empty($v['import_docs'])), 'v[import_docs]', g_l('import', '[import_docs]'), false, 'defaultfont', "WE().util.dialogTop(window).toggle(document,'doc_table')"))
			->setCol(1, 0, $dir_table->getHtml())

			// --------------
			// import templates
			->setCol(2, 0, we_html_forms::checkboxWithHidden((!empty($v['import_templ'])), 'v[import_templ]', g_l('import', '[import_templ]'), false, 'defaultfont', "WE().util.dialogTop(window).toggle(document,'tpl_table')"), [
			'colspan' => '2']);

		$rootDirID = we_users_util::get_def_ws(TEMPLATES_TABLE);
		$weSuggest->setAcId('TemplPath')
			->setContentType([we_base_ContentTypes::FOLDER_TEMPLATE])
			->setInput('v[tpl_dir]', ($v['tpl_dir'] ?? id2path($rootDirID, TEMPLATES_TABLE)), ['onfocus' => "WE().util.dialogTop(window).setFormField('_v[restore_tpl_path]', false, 'checkbox');"])
			->setMaxResults(10)
			->setRequired(true)
			->setResult('v[tpl_dir_id]', (string) ($v['tpl_dir_id'] ?? $rootDirID))
			->setSelector(we_gui_suggest::DirSelector)
			->setTable(TEMPLATES_TABLE)
			->setWidth(280)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:WE().util.dialogTop(window).wizbody.document.we_form.elements['v[tpl_dir_id]'].value,table:'" . TEMPLATES_TABLE . "',targetID:'v[tpl_dir_id]',targetName:'v[tpl_dir]',rootDirID:'" . $rootDirID . "'})"));

		$docPath = $weSuggest->getHTML();

		$dir_table = new we_html_table(['id' => 'tpl_table', 'style' => 'margin-left:20px;'], 3, 2);
		if((isset($v['import_templ']) && !$v['import_templ'])){
			$dir_table->setStyle('display', 'none');
		}
		$dir_table
			->setCol(0, 0, we_html_tools::alertAttentionBox(g_l('import', '[templates_desc]'), we_html_tools::TYPE_ALERT, 390, true, 50))
			->setCol(1, 0, $docPath)
			->setCol(2, 0, we_html_forms::checkboxWithHidden((!empty($v['restore_tpl_path'])), 'v[restore_tpl_path]', g_l('import', '[maintain_paths]'), false, 'defaultfont', "self.document.we_form.elements['v[tpl_dir]'].value='/';"));


		$tbl_extra
			->setCol(3, 0, $dir_table->getHtml())
			->setCol(4, 0, we_html_forms::checkboxWithHidden((!empty($v['import_thumbnails'])), "v[import_thumbnails]", g_l('import', '[import_thumbnails]'), false, "defaultfont"), [
				'colspan' => '2']);


		$parts->add((new we_html_multiIconBoxContentLine(g_l('import', '[handle_document_options]') . '<br/>' . g_l('import', '[handle_template_options]'), $tbl_extra->getHTML()))
				->space(we_html_multiIconBox::SPACE_MED)
		);

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$tbl_extra = (new we_html_table([], 2, 1))
				->setCol(0, 0, we_html_forms::checkboxWithHidden((!empty($v['import_objs'])), "v[import_objs]", g_l('import', '[import_objs]')))
				->setCol(1, 0, we_html_forms::checkboxWithHidden((!empty($v['import_classes'])), "v[import_classes]", g_l('import', '[import_classes]')));

			$parts->add((new we_html_multiIconBoxContentLine(g_l('import', '[handle_object_options]') . '<br/>' . g_l('import', '[handle_class_options]'), $tbl_extra->getHTML()))
					->space(we_html_multiIconBox::SPACE_MED)
			);
		}

		$tbl_extra = (new we_html_table([], 4, 1))
			->setCol(0, 0, we_html_forms::checkboxWithHidden((!empty($v['import_dt'])), "v[import_dt]", g_l('import', '[import_doctypes]')))
			->setCol(1, 0, we_html_forms::checkboxWithHidden((!empty($v['import_ct'])), "v[import_ct]", g_l('import', '[import_cats]')))
			->setCol(2, 0, we_html_forms::checkboxWithHidden((!empty($v['import_navigation'])), "v[import_navigation]", g_l('import', '[import_navigation]'), false, 'defaultfont', "WE().util.dialogTop(window).toggle(document,'navigation_table')"));

		// --
		$weSuggest->setAcId('NaviPath')
			->setContentType([we_base_ContentTypes::FOLDER_COMMON])
			->setInput("v[navigation_dir]", ($v['navigation_dir'] ?? id2path($rootDirID)))
			->setMaxResults(10)
			->setRequired(true)
			->setResult('v[navigation_dir_id]', (string) ($v['navigation_dir_id'] ?? $rootDirID))
			->setSelector(we_gui_suggest::DirSelector)
			->setTable(NAVIGATION_TABLE)
			->setWidth(280)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_navigation_dirSelector',{id:document.we_form.elements[\"v[navigation_dir_id]\"].value,targetID:'v[navigation_dir_id]',targetName:'v[navigation_dir]'});"));

		$docPath = $weSuggest->getHTML();

		$dir_table = new we_html_table(['id' => 'navigation_table', 'style' => 'margin-left:20px;'], 2, 1);
		if((isset($v['import_navigation']) && !$v['import_navigation'])){
			$dir_table->setStyle('display', 'none');
		}
		$dir_table
			->setCol(0, 0, we_html_tools::alertAttentionBox(g_l('import', '[navigation_desc]'), we_html_tools::TYPE_ALERT, 390))
			->setCol(1, 0, $docPath);

		$tbl_extra->setCol(3, 0, $dir_table->getHtml());

		$parts->add((new we_html_multiIconBoxContentLine(g_l('import', '[handle_doctype_options]', false, true) . '<br/>' . g_l('import', '[handle_category_options]', false, true), $tbl_extra->getHTML()))
				->space(we_html_multiIconBox::SPACE_MED)
		);

		$parts->add((new we_html_multiIconBoxContentLine(g_l('import', '[handle_file_options]'), we_html_forms::checkboxWithHidden((!empty($v['import_binarys'])), 'v[import_binarys]', g_l('import', '[import_files]'))))
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('import', '[rebuild]'), we_html_forms::checkboxWithHidden((!empty($v['rebuild'])), 'v[rebuild]', g_l('import', '[rebuild_txt]'))))
				->space(we_html_multiIconBox::SPACE_MED)
		);

		$header = we_base_file::loadPart(WE_DOCUMENT_ROOT . $v['import_from'], 0, 512);

		if(empty($header)){
			$hdnsBtnStates = we_html_element::hiddens([
					'v[btnState_next]' => 'disabled',
					'v[btnState_back]' => 'enabled'
			]);
			$parts = (new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('import', '[invalid_path]'), we_html_tools::TYPE_ALERT, 530))
			);
			return $hdns . $hdnsBtnStates . (new we_html_multiIconBox($parts, we_exim_ExIm::TYPE_WE))
					->marginLeft()
					->headline(g_l('import', '[warning]'))
					->get();
		}

		$show_owner_opt = str_contains($header, '<' . we_exim_ExIm::INFO_TAG . '>');

		if($show_owner_opt){
			$tbl_extra = (new we_html_table([], 2, 1))
				->setCol(0, 0, we_html_forms::checkboxWithHidden((!empty($v['import_owners'])), 'v[import_owners]', g_l('import', '[handle_owners]')))
				->setCol(1, 0, we_html_forms::checkboxWithHidden((!empty($v['owners_overwrite'])), 'v[owners_overwrite]', g_l('import', '[owner_overwrite]')));

			$tbl_extra2 = we_html_element::div(['style' => 'margin:20px 20px 0 0;'], $this->formWeChooser(USER_TABLE, 0, 'v[owners_overwrite_id]', (int) ($v['owners_overwrite_id'] ?? 0), 'v[owners_overwrite_path]', ($v['owners_overwrite_path'] ?? '/')));

			$parts->add((new we_html_multiIconBoxContentLine(g_l('import', '[handle_owners_option]'), we_html_tools::alertAttentionBox(g_l('import', '[notexist_overwrite]'), we_html_tools::TYPE_ALERT, 530) . $tbl_extra->getHTML() . $tbl_extra2))
					->space(we_html_multiIconBox::SPACE_MED)
			);
		} else {
			$hdns .= we_html_element::hiddens([
					'v[import_owners]' => '0',
					'v[owners_overwrite]' => '0',
					'v[owners_overwrite_id]' => '0'
			]);
		}

		$znr = -1;
		return $hdns . $hdnsBtnStates . (new we_html_multiIconBox($parts, we_exim_ExIm::TYPE_WE))->marginLeft()
				->fold($znr, false, g_l('weClass', '[moreProps]'), g_l('weClass', '[lessProps]'))
				->headline(g_l('import', '[import_options]'))
				->get();
	}

	private function getStep3(): string{
		$hdns = we_html_element::hiddens(['v[btnState_next]' => 'disabled', 'v[btnState_back]' => 'disabled']);
		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', we_html_element::div(['class' => 'blockWrapper', 'style' => 'width: 520px; height: 400px; border:1px #dce6f2 solid;',
					'id' => 'log'], '')
		));
		return $hdns . (new we_html_multiIconBox($parts, we_exim_ExIm::TYPE_WE))
				->marginLeft()
				->headline(g_l('import', '[log]'))
				->get();
	}

	private function formWeChooser(string $table = FILE_TABLE, int $rootDirID = 0, string $IDName = 'ID', int $IDValue = 0, string $Pathname = 'Path', string $Pathvalue = '/', string $cmd = ''): string{
		$Pathvalue = ($Pathvalue ?: fS('SELECT Path FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $IDValue));
		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_file',{id:document.we_form.elements['" . $IDName . "'].value,table:'" . $table . "',targetID:'" . $IDName . "',targetName:'" . $Pathname . "',jsCmd:'" . $cmd . "',rootDirID:'" . $rootDirID . "'})");
		return we_html_tools::formElementTable('', we_html_tools::textInput($Pathname, $Pathvalue, 0, ['readonly' => 'readonly']), we_html_element::hidden($IDName, $IDValue), $button);
	}

	protected function getWizCmdProgressStep(we_base_jsCmd $jsCmd, array $v, array $records, array $we_flds, array $attributes): string{
		$hiddens = $this->getHdns('v', $v);

		if(intval($v['cid']) == 0){
			// clear session data
			we_exim_ExIm::unsetPreserves();
		}

		if($v['cid'] >= $v['numFiles'] - 1){ // finish import
			$xmlExIm = new we_import_updater();
			$xmlExIm->loadPreserves();
			$this->xmlExImSetOpt($xmlExIm, $v);
			if($xmlExIm->RefTable->current == 0){
				$jsCmd->addCmd('addLog_buffered', [we_html_element::b(g_l('import', '[update_links]'))]);
			}

			while(($ref = $xmlExIm->RefTable->getNext()) !== null){
				if(isset($ref->ContentType) && isset($ref->ID)){
					$doc = we_exim_contentProvider::getInstance($ref->ContentType, $ref->ID, $ref->Table);
					$xmlExIm->updateObject($doc);
				}
			}

			if($ref){
				$xmlExIm->savePreserves();

				$jsCmd->addCmd('setProgressText_footer', 'pb1', g_l('import', '[update_links]') . $xmlExIm->RefTable->current . '/' . $xmlExIm->RefTable->getCount());
				$jsCmd->addCmd('setProgress_footer', (int) ((($v['cid'] + $xmlExIm->RefTable->current + 1) / ($xmlExIm->RefTable->getCount() + $v['numFiles'])) * 100));
				$jsCmd->addCmd('call_delayed', ['function' => 'we_import', 'delay' => 15, 'param_1' => 1, 'param_2' => $v['cid'], 'param_3' => false]);
			} else {
				//FIXME: if update needs more steps they must be handled here
				we_update_updater::doUpdate('internal');

				$jsCmd->addCmd('finish', $xmlExIm->options['rebuild']);
				$jsCmd->addCmd('call_delayed', ['function' => 'we_import', 'delay' => 15, 'param_1' => 1, 'param_2' => $v['numFiles'], 'param_3' => false]);
			}


			we_exim_ExIm::unsetPreserves();
			return we_html_element::form(['name' => we_base_constants::WE_FORM], $hiddens);
		}// do import
		$chunk = $v['uniquePath'] . basename($v['import_from']) . '_' . $v['cid'];
		if(!file_exists($chunk)){
			return '';
		}
		$xmlExIm = new we_exim_XMLImport();
		$xmlExIm->loadPreserves();
		$this->xmlExImSetOpt($xmlExIm, $v);
		$imported = $xmlExIm->import($chunk);
		$xmlExIm->savePreserves();
		if($imported){
			$status = g_l('import', '[import]');
			$ref = $xmlExIm->RefTable->getLast();

			switch($ref->ContentType){
				case we_base_ContentTypes::BACKUP_BINARY:
				case we_base_ContentTypes::CATEGORY:
				case we_base_ContentTypes::OBJECT_FILE:
					$path_info = $ref->Path;
					break;
				case we_base_ContentTypes::DOCTYPE:
					$path_info = fS('SELECT Text FROM ' . DB_WE::$inst->escape($ref->Table) . ' WHERE ID=' . intval($ref->ID));
					break;
				case we_base_ContentTypes::NAVIGATIONRULE:
					$path_info = fS('SELECT NavigationName FROM ' . DB_WE::$inst->escape($ref->Table) . ' WHERE ID=' . intval($ref->ID));
					break;
				case we_base_ContentTypes::THUMBNAIL:
					$path_info = fS('SELECT Name FROM ' . DB_WE::$inst->escape($ref->Table) . ' WHERE ID=' . intval($ref->ID));
					break;
				default:
					$path_info = id2path($ref->ID, $ref->Table);
					break;
			}
			try{
				$prog = g_l('contentTypes', '[' . $ref->ContentType . ']', true);
			} catch (we_exception_base $ex){
				$prog = g_l('import', '[' . $ref->ContentType . ']', true);
			}
			$progress_text = we_html_element::b($prog) . ' ' . $path_info;
			$jsCmd->addCmd('addLog_buffered', [$progress_text]);
		} else {
			$status = g_l('import', '[skip]');
			$jsCmd->addCmd('addLog_buffered', [g_l('import', '[skip]') . '<br/>']);
		}

		$counter_text = g_l('import', '[item]') . ' ' . $v['cid'] . '/' . ($v['numFiles'] - 2);

		$jsCmd->addCmd('setProgressText_footer', 'pb1', $status . ' - ' . $counter_text);
		$jsCmd->addCmd('setProgress_footer', (int) (((intval($v['cid']) + 1) / (2 * intval($v['numFiles']))) * 100));
		$jsCmd->addCmd('call_delayed', ['function' => 'we_import', 'delay' => 15, 'param_1' => 1, 'param_2' => ($v['cid'] + 1), 'param_3' => false]);

		return we_html_element::form(['name' => we_base_constants::WE_FORM], $hiddens);
	}

	protected function getWizCmdPrepareStep(we_base_jsCmd $jsCmd, array $v, array $records, array $we_flds, array $attrs, array $attributes): string{
		$jsCmd->addCmd('addLog_buffered', [
			we_html_element::b(g_l('import', '[start_import]') . ' - ' . date(g_l('weEditorInfo', '[date_format_sec]'))),
			we_html_element::b(g_l('import', '[prepare]')),
			we_html_element::b(g_l('import', '[import]'))
		]);

		$path = TEMP_PATH . we_base_file::getUniqueId() . '/';
		we_base_file::createLocalFolderByPath($path);
		we_base_file::insertIntoCleanUp($path, 3000);

		$num_files = (is_dir($path) ?
			we_exim_XMLImport::splitFile(WE_DOCUMENT_ROOT . $v['import_from'], $path, 1) + 1 : 0);

		return parent::getWizCmdPrepareStepBase($jsCmd, $this->getHdns('v', $v) .
				we_html_element::hiddens([
					'v[numFiles]' => (string) $num_files,
					'v[uniquePath]' => $path]));
	}

	private function xmlExImSetOpt(we_exim_ExIm $xmlExIm, array $v): void{
		$xmlExIm->setOptions([
			'handle_documents' => $v['import_docs'],
			'handle_templates' => $v['import_templ'],
			'handle_objects' => $v['import_objs'] ?? 0,
			'handle_classes' => $v['import_classes'] ?? 0,
			'handle_doctypes' => $v['import_dt'],
			'handle_categorys' => $v['import_ct'],
			'handle_binarys' => $v['import_binarys'],
			'document_path' => $v['doc_dir_id'],
			'template_path' => $v['tpl_dir_id'],
			'handle_collision' => $v['collision'],
			'restore_doc_path' => $v['restore_doc_path'],
			'restore_tpl_path' => $v['restore_tpl_path'],
			'handle_owners' => $v['import_owners'],
			'owners_overwrite' => $v['owners_overwrite'],
			'owners_overwrite_id' => $v['owners_overwrite_id'],
			'handle_navigation' => $v['import_navigation'],
			'navigation_path' => $v['navigation_dir_id'],
			'handle_thumbnails' => $v['import_thumbnails'],
			'rebuild' => $v['rebuild']
		]);
	}

}
