<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f52c91000392 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-23 12:54 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Definition of WebEdition Newsletter Block
 *
 */
class we_newsletter_block extends we_newsletter_base{
// Document based Newsletter Block

	const DOCUMENT = 0;
// Document field based Newsletter Block
	const DOCUMENT_FIELD = 1;
// Object based Newsletter Block
	const OBJECT = 2;
// Object field based Newsletter Block
	const OBJECT_FIELD = 3;
// File based Newsletter Block
	const FILE = 4;
//  Text based Newsletter Block
	const TEXT = 5;
//  Newsletter attachment
	const ATTACHMENT = 6;
//  URL based newsletter
	const URL = 7;

	/** @var int */
	protected $NewsletterID = 0;

	/** @var int[] */
	protected $Groups = [];

	/** @var int */
	var $Type = self::DOCUMENT;

	/** @var int */
	protected $LinkID = 0;

	/** @var string */
	protected $Field = '';

	/** @var string */
	protected $Source = '';

	/** @var string */
	protected $Html = '';

	/*	 * *****************************************************
	 * Default Constructor
	 * Can load or create new Newsletter depends of parameter
	 * ****************************************************** */

	function __construct(int $newsletterID = 0){
		parent::__construct(NEWSLETTER_BLOCK_TABLE);
		$this->persistent_slots = [
			'NewsletterID' => we_base_request::INT,
			'Groups' => we_base_request::INTLISTA,
			'Type' => we_base_request::INT,
			'LinkID' => we_base_request::INT,
			'Field' => we_base_request::STRING, //can be numeric or not, seems that paths can be transfered this way.
			'Source' => we_base_request::RAW,
			'Html' => we_base_request::RAW,
		];

		if($newsletterID){
			$this->load($newsletterID);
		}
	}

	//---------------------------------- STATIC FUNCTIONS -------------------------------

	/*	 * ****************************************************
	 * return all newsletter blocks for given newsletter id
	 *
	 * ***************************************************** */

	/**
	 * @param int $newsletterID
	 * @return we_newsletter_block[]
	 */
	static function getAllBlocks(int $newsletterID): array{
		$all = DB_WE::$inst->getAllq('SELECT ID FROM ' . NEWSLETTER_BLOCK_TABLE . ' WHERE NewsletterID=' . $newsletterID . ' ORDER BY ID', true);
		$ret = [];
		foreach($all as $id){
			$ret[] = new we_newsletter_block((int) $id);
		}
		return $ret;
	}

	public function setNewsletterID(int $id): void{
		$this->NewsletterID = $id;
	}

	public function getLinkID(): int{
		return $this->LinkID;
	}

	/**
	 * @return int[]
	 */
	public function getGroups(): array{
		return $this->Groups;
	}

	/**
	 * @param int[] $groups
	 * @return void
	 */
	public function setGroups(array $groups): void{
		$this->Groups = array_filter($groups);
	}

	public function getSource(): string{
		return $this->Source;
	}

	public function getHTML(): string{
		return $this->Html;
	}

	public function getField(): string{
		return $this->Field;
	}

	public function getHidden(int $counter): string{
		$vals = [];

		foreach($this->persistent_slots as $per => $type){
			$vals['block' . $counter . '_' . $per] = $this->convertToHiddenString($per, $type, $this->$per);
		}
		return we_html_element::hiddens($vals);
	}

	public function setFromRequest(int $id, int $skey): void{
		$this->setNewsletterID($id);

		foreach($this->persistent_slots as $per => $type){
			$this->$per = we_base_request::_($type, 'block' . $skey . '_' . $per, $this->$per);
		}
	}

}
