<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 818ead29ce48 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-20 02:08 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_checkForm extends we_weTag_tag{

	public static function parse(array $attribs, string $content): string{
		self::attributeMissing($attribs, ['match' => false, 'type' => false], __CLASS__);

		return '<?php ob_start();?>' . $content . '<?= ' . we_weTag_parser::printTag('checkForm', $attribs, 'ob_get_clean()?:\'\'', false, true) . '; ?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		//  dont make this in editMode
		if(we_base_WE::$editmode){
			return '';
		}

		//  check required Fields
		self::attributeMissing($attribs, ["match" => false, 'type' => false], __CLASS__);

		// get fields of $attribs
		$match = self::getAttribute('match', $attribs, '', we_base_request::STRING);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$mandatory = self::getAttribute('mandatory', $attribs, '', we_base_request::STRING);
		$email = self::getAttribute('email', $attribs, '', we_base_request::STRING);
		$password = self::getAttribute('password', $attribs, '', we_base_request::STRING);
		$onError = self::getAttribute('onError', $attribs, '', we_base_request::JS);
		$jsIncludePath = self::getAttribute('jsIncludePath', $attribs, '', we_base_request::RAW);

		//  Generate errorHandler:
		$jsOnError = ($onError ? '
if(self.' . $onError . '){' .
			$onError . '(formular,missingReq,wrongEmail,pwError);
} else {alert("' . $content . '");
}' :
			'alert("' . $content . '");'
			);

		//  Generate mandatory array
		if($mandatory){
			$fields = explode(',', $mandatory);
			$jsMandatory = '//  check mandatory
var required = ["' . implode('", "', $fields) . '"];
missingReq = weCheckFormMandatory(formular, required);';
		} else {
			$jsMandatory = '';
		}

		$jsEmail = ($email ? //  code to check Emails
			'//  validate emails
        var email = ["' . implode('", "', explode(',', $email)) . '"];
        wrongEmail = weCheckFormEmail(formular, email);' :
			'');


		if($password){
			$pwFields = explode(',', $password);
			if(count($pwFields) != 3){
				return self::parseError(g_l('parser', '[checkForm_password]'));
			}
			$jsPasword = '//  check passwords
var password = ["' . implode('", "', $pwFields) . '"];
pwError = weCheckFormPassword(formular, password);
';
		} else {
			$jsPasword = '';
		}

		//  deal with alwasy needed stuff - "class weCheckFormEvent"
		if($jsIncludePath){

			if(is_numeric($jsIncludePath)){
				$jsEventHandler = we_weTag_util::tag('js', ['id' => $jsIncludePath]);
				if(!$jsEventHandler){
					return self::parseError(g_l('parser', '[checkForm_jsIncludePath_not_found]'));
				}
			} else {
				$jsEventHandler = we_html_element::jsScript($jsIncludePath);
			}
		} else {
			$jsEventHandler = we_html_element::jsScript(JS_DIR . 'external/weCheckForm.js');
		}

		switch($type){
			case 'id': //  id of formular is given
				$function = we_html_element::jsElement('
weCheckFormEvent.addEvent(window, "load", function(){
	initWeCheckForm_by_id(weCheckForm_id_' . $match . ',"' . $match . '");
});
function weCheckForm_id_' . $match . '(ev){
	var missingReq = [],
	wrongEmail = [],
	pwError    = false;

	formular = document.forms.' . $match . ';
	' . $jsMandatory . '
	' . $jsEmail . '
	' . $jsPasword . '

	//  return true or false depending on errors
	if( (wrongEmail.length>0) || (missingReq.length>0) || pwError){
			' . $jsOnError . '
			weCheckFormEvent.stopEvent(ev);
			return false;
	}
	return true;
}');

				break;
			default:
			case 'name' : //  name of formular is given
				$function = we_html_element::jsElement('
weCheckFormEvent.addEvent( window, "load", function(){
	initWeCheckForm_by_name(weCheckForm_n_' . $match . ',"' . $match . '");
});
function weCheckForm_n_' . $match . '(ev){
	var missingReq = [];
	var wrongEmail = [];
	var pwError    = false;

	formular = document.forms.' . $match . ';
	' . $jsMandatory . '
	' . $jsEmail . '
	' . $jsPasword . '

	//  return true or false depending on errors
	if( wrongEmail.length || missingReq.length || pwError){
		' . $jsOnError . '
		weCheckFormEvent.stopEvent(ev);
		return false;
	}
	return true;
}');

				break;
		}

		return $jsEventHandler . $function;
	}

}
