<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: c564388e074d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-17 13:01 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_controlElement extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes @unused-param
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @return string parsed tag for php
	 */
	public static function parse(array $attribs, string $content): string{
		return '';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$hide = self::getAttribute('hide', $attribs, false, we_base_request::BOOL);
		if(!isset($attribs['doc'])){
			return '';
		}
		$doc = $attribs['doc'];

		if($type && $name){
			switch($type){
				case 'button': //	only look, if the button shall be hidden or not
					$doc->controlElement['button'][$name] = ['hide' => ( $hide ? 1 : 0 )];
					break;
				case 'checkbox':
					$doc->controlElement['checkbox'][$name] = [
						'hide' => ( $hide ? 1 : 0 ),
						'readonly' => ( self::getAttribute('readonly', $attribs, true, we_base_request::BOOL) ? 1 : 0 ),
						'checked' => ( self::getAttribute('checked', $attribs, false, we_base_request::BOOL) ? 1 : 0 )
					];
					break;
			}
		}

		return '';
	}

}
