<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d3fd95ad9ec4 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-10 11:02 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tree_base{
	const DefaultWidth = 300;
	const MinWidth = 200;
	const MaxWidth = 1000;
	const StepWidth = 20;
	const DeleteWidth = 420;
	const MoveWidth = 500;
	const AddToCollectionWidth = 508;
	const HiddenWidth = 40;
	const MinWidthModules = 120;
	const MaxWidthModules = 800;
	const TYP_FOLDER = 'group';
	const TYP_ITEM = 'item';
	const TYP_DOTS = 'threedots';
	const CT_UP = 'arrowup';
	const CT_DOWN = 'arrowdown';

	/** @var we_database_base */
	protected $db;

	/** @var string[] */
	var $styles = [];

	/** @var array<string,int> */
	var $tree_states = [
		'edit' => 0,
		'select' => 1,
		'selectitem' => 2,
		'selectgroup' => 3,
	];

	/** @var int */
	var $default_segment = 30;

	/** @var bool */
	protected $autoload = true;

	/** @var bool */
	protected $addSorted = true;

	/** @var string */
	protected $extraClasses = '';

	/** @var we_base_jsCmd */
	protected $jsCmd;

//Initialization

	public function __construct(we_base_jsCmd $jsCmd){
		$this->db = new DB_WE();
		$this->jsCmd = $jsCmd;
		$this->default_segment = intval(we_base_preferences::getUserPref('default_tree_count'));
	}

	/*
	  the functions prints tree javascript
	  should be placed in a frame which doesn't reloads

	 */

	public function getJSTreeCode(): string{
		if($this->autoload){
			$this->jsCmd->addCmd('loadTree', [
				'clear' => true,
				'items' => static::getItems(0, 0, $this->default_segment),
				'sorted' => $this->addSorted
			]);
		}

		return we_html_element::jsScript(JS_DIR . 'tree.js') . $this->customJSFile();
	}

	abstract protected function customJSFile(): string;

	public function getHTMLConstruct(): string{
		return
			we_html_element::cssLink(CSS_DIR . 'tree.css') .
			we_html_element::div(['id' => 'treetable',
				'class' => 'tree' . ($this->extraClasses ? ' ' . $this->extraClasses : ''),
				], ''
		);
	}

	/**
	 * @param $ParentID
	 * @param int $offset
	 * @param int $segment
	 * @param bool $sort
	 * @return array
	 */
	public static function getItems(int $ParentID, int $offset = 0, int $segment = 500, bool $sort = false): array{
		//overwrite
		return [];
	}

	/**
	 *
	 * @param string $path
	 * @return string[]
	 */
	public static function getParents(string $path): array{
		$out = [];
		$db = DB_WE::$inst;
		while($path != '/' && $path != '\\' && $path){
			$out[] = '"' . $db->escape($path) . '"';
			$path = dirname($path);
		}
		return $out;
	}

}
