<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6cc64d37551c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:32 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_tree_main extends we_tree_base{

	public function __construct(we_base_jsCmd $jsCmd){
		parent::__construct($jsCmd);
		$this->autoload = false;
		$this->extraClasses = 'withFooter';
	}

	/**
	 *
	 * @param we_contents_root $doc
	 * @param bool $select
	 * @param ?we_base_jsCmd $jsCmd
	 * @return array|array{string,int,array,array}
	 */
	public static function getJSUpdateTreeCmd(we_contents_root $doc, bool $select, we_base_jsCmd $jsCmd = null): array{
		switch($doc->ContentType){
			case we_base_ContentTypes::HTML:
			case we_base_ContentTypes::WEDOCUMENT:
			case we_base_ContentTypes::OBJECT_FILE:
				switch($doc->getDocumentState()){
					case we_contents_root::STATE_UNPUBLISHED:
						$published = 0;
						break;
					case we_contents_root::STATE_MODIFIED:
						$published = -1;
						break;
					default:
						$published = 1;
				}
				break;
			default:
				$published = 1;
		}

//	This is needed in SeeMode
		if(we_editor_functions::isSeeMode()){
			return [];
		}

		$hasSched = false;
		if($doc instanceof we_document_base){
			foreach($doc->schedArr as $sched){
				$hasSched = $hasSched || $sched->active;
			}
		}
		$main = [
			'id' => $doc->ID,
			'parentid' => $doc->ParentID,
			'text' => $doc->getText(),
			'published' => $published,
			'table' => $doc->Table,
			'inschedule' => intval($hasSched),
		];
		$adv = [
			'contenttype' => $doc->ContentType,
			'isclassfolder' => ($doc instanceof we_folder_objectFile ? $doc->isClassFolder() : false),
			'checked' => 0,
			'typ' => ($doc instanceof we_folder_base ? 'group' : 'item'),
			'open' => 0,
			'disabled' => 0,
			'tooltip' => $doc->ID,
		];
		if($jsCmd){
			$jsCmd->addCmd('updateMainTree', intval($select), $main, $adv);
		}
		return ['updateMainTree', intval($select), $main, $adv];
	}

	protected function customJSFile(): string{
		return we_html_element::jsScript(JS_DIR . 'main_tree.js', 'initTree();');
	}

	function getJSLoadTree(array $treeItems): array{
		$ret = [];

		foreach($treeItems as $item){
			$cur = [];
			foreach($item as $k => $v){
				$cur[strtolower($k)] = ($v === 1 || $v === 0 || $v === 'true' || $v === 'false' || is_int($v) ?
					intval($v) :
					$v);
			}
			$ret[] = $cur;
		}

		return $ret;
	}

	public static function getItems(int $ParentID, int $offset = 0, int $segment = 500, bool $sort = false): array{
		//unused
		return [];
	}

}
