<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: bea40b38ee5e $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-16 19:39 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_weTag_typeAttribute extends we_weTag_attribute{
	/**
	 * @var we_weTag_option[]
	 */
	private $Options;

	/**
	 * @param string $name
	 * @param we_weTag_option[] $options
	 * @param boolean $required
	 */
	function __construct(we_weTag_twbase $tag, string $name, array $options = [], bool $required = true, string $module = '', string $description = '', bool $deprecated = false){
		parent::__construct($tag, $name, '', $required, $module, $description, $deprecated);
		$this->Options = parent::getUseOptions($options);
		foreach($this->Options as $option){
			$option->addTypeAttribute($this);
		}
		// overwrite value if needed
		if($this->Value === false){
			$this->Value = '-';
		}
	}

	function getCodeForTagWizard(we_html_tr $row): void{
		$entries = ['' => g_l('taged', '[select_type]')];

		foreach($this->Options as $option){
			$entries[$option->Value] = ($option->getName() === '-' ? '' : $option->getName());
		}

		$select = (new we_html_select(['name' => $this->Name,
				'id' => $this->getIdName(),
				'onchange' => "we_cmd('switch_type', this.value);",
				'class' => 'defaultfont selectinput'
				]))
			->addOptions($entries);
		$row->td($this->getLabelCodeForTagWizard(), ['class' => 'attributeName']);
		$row->td($select->getHtml(), ['class' => 'attributeField']);
		$row->td('', ['class' => 'attributeButton']);
	}

	/**
	 * @return array
	 */
	function getOptions(): array{
		return $this->Options;
	}

	/**
	 * @return string
	 */
	protected function getLabelCodeForTagWizard(): string{
		return we_html_element::label([
				'id' => 'label_' . $this->getIdName(),
				'class' => 'defaultfont',
				'for' => $this->getIdName(),
				], $this->Name . ($this->Required ? '*' : ''));
	}

}
