<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0627c1498cde $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-02 12:07 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_widget_fdl extends we_widget_base{
	/** @var int */
	private $maxRows = 0;

	/** @var string */
	private $failedLoginHTML = '';

	/** @var string */
	private $newSCurrId = '';

	public function __construct(string $curID = '', array $aProps = []){
		if(!(we_modules_info::isActive(we_modules_info::CUSTOMER) && we_base_permission::hasPerm([we_base_perm::CUSTOMER_EDIT]))){
			return;
		}

		$failedLoginsTable = new we_html_table(['class' => 'default',], 1, 4);
		$ws = we_users_util::get_ws(CUSTOMER_TABLE);

		$queryFailedLogins = ' FROM ' .
			FAILED_LOGINS_TABLE . ' f LEFT JOIN ' .
			CUSTOMER_TABLE . ' c ON f.Username=c.Username
WHERE
	f.UserTable="tblWebUser" AND
	f.isValid="true" AND
	f.LoginDate>(NOW() - INTERVAL ' . intval(SECURITY_LIMIT_CUSTOMER_NAME_HOURS) . ' hour) ' .
			($ws ? ' AND ' . implode('', $ws) : '');


		if(($this->maxRows = fI('SELECT COUNT(DISTINCT f.Username) ' . $queryFailedLogins))){
			$failedLoginsTable->setColAttributes(0, 0, ['style' => 'width:25px;']);
			$failedLoginsTable->setCol(0, 1, we_html_element::b(g_l('cockpit', '[kv_failedLogins][username]')), ['class' => 'middlefont', 'style' => "text-align:left"]);
			$failedLoginsTable->setCol(0, 2, we_html_element::b(g_l('cockpit', '[kv_failedLogins][numberLogins]')), ['class' => 'middlefont', 'style' => "text-align:left"]);

			$cur = 0;
//	while($this->maxRows > $cur){
			$data = DB_WE::$inst->getAllq('SELECT f.Username, COUNT(f.isValid) AS numberFailedLogins,c.ID AS UID' . $queryFailedLogins . ' GROUP BY f.Username LIMIT ' . $cur . ',100');
			$i = 1;
			foreach($data as $cur){
				$webUserID = $cur['UID'];
				$prio = (intval($cur['numberFailedLogins']) >= SECURITY_LIMIT_CUSTOMER_NAME) || !$webUserID ? 'red' : 'green';
				$failedLoginsTable->setCol($i, 0, '<i class="fa fa-dot-circle-o" style="color:' . $prio . '"></i>', ['class' => 'middlefont', 'style' => 'text-align:center']);
				$failedLoginsTable->setCol($i, 1, $cur['Username'], ['class' => 'middlefont', 'style' => 'text-align:left']);
				$failedLoginsTable->setCol($i, 2, intval($cur['numberFailedLogins']) . ' / ' . SECURITY_LIMIT_CUSTOMER_NAME . ' ' . sprintf(g_l('cockpit', '[kv_failedLogins][logins]'), SECURITY_LIMIT_CUSTOMER_NAME_HOURS), [
					'class' => 'middlefont', 'style' => 'text-align:left']);

				$buttonJSFunction = 'WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR+"rpc.php?mod=customer&cmd=customerResetFailedLogins","custid=' . $webUserID . '", ajaxCallbackResetLogins );';
				$failedLoginsTable->setCol($i, 3, ((intval($cur['numberFailedLogins']) >= SECURITY_LIMIT_CUSTOMER_NAME && $webUserID) ? we_html_button::create_button(we_html_button::RESET, "javascript:" . $buttonJSFunction) : ''), [
					'class' => 'middlefont', 'style' => "text-align:right"]);
				$i++;
			}
			//$cur+=1000;
			//}
		} else {
			$this->maxRows = 0;
			$failedLoginsTable->setCol(1, 0, we_html_element::b(g_l('cockpit', '[kv_failedLogins][noFailedLogins]')), ['class' => 'middlefont', 'colspan' => "4", 'style' => "text-align:left;color:green;"]);
		}
		$this->newSCurrId = $curID ?: we_base_request::string('we_cmd', '', 5);

		$this->failedLoginHTML = $failedLoginsTable->getHtml();
	}

	/**
	 * @return array{string,array{string,string}}
	 */
	public function getInsertDiv(int $iCurrId, array $aProps, we_base_jsCmd $jsCmd): array{
		$cfg = self::getDefaultConfig();

		$oTblDiv = we_html_element::jsScript(JS_DIR . 'widgets/fdl.js', '', [
				'id' => 'loadVarFdl', 'data-fdl' => setDynamicVar([
					'refreshCmd' => 'm_' . $iCurrId
			])]) .
			we_html_element::div([
				'id' => 'm_' . $iCurrId . '_inline',
				'style' => 'width:100%;height:' . ($cfg['height'] - 25) . 'px;overflow:auto;'
				], we_html_element::div(['id' => 'fdl_data'], $this->failedLoginHTML)
		);
		$aLang = [g_l('cockpit', '[kv_failedLogins][headline]') . ' (' . $this->maxRows . ')', ""];
		return [$oTblDiv, $aLang];
	}

	/**
	 * @return array<string,mixed>
	 */
	public static function getDefaultConfig(): array{
		return [
			'width' => self::WIDTH_LARGE,
			'expanded' => 1,
			'height' => 210,
			'res' => 1,
			'cls' => 'orange',
			'csv' => '',
			'dlgHeight' => 435,
			'isResizable' => false
		];
	}

	public static function showDialog(): void{
		[$jsFile, $oSelCls] = self::getDialogPrefs();

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', $oSelCls->getHTML()));

		$sTblWidget = (new we_html_multiIconBox($parts, 'Props'))
			->marginLeft()
			->headline(g_l('cockpit', '[customer]'))
			->buttons(we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:save();"), we_html_button::create_button(we_html_button::PREVIEW, "javascript:preview();"), we_html_button::create_button(we_html_button::CLOSE, "javascript:exit_close();")))
			->get();

		echo we_html_tools::getHtmlTop(g_l('cockpit', '[customer]'), $jsFile .
			we_html_element::jsScript(JS_DIR . 'widgets/fdl.js', '', [
				'id' => 'loadVarFdl', 'data-fdl' => setDynamicVar([
					'refreshCmd' => we_base_request::string('we_cmd', '', 0)
			])])
			, we_html_element::body(
				['class' => 'weDialogBody', 'onload' => "init();"
				], we_html_element::form([], $sTblWidget)));
	}

	public function showPreview(): void{
		$jsCmd = new we_base_jsCmd();
		$jsCmd->addCmd('initPreview', [
			'id' => $this->newSCurrId,
			'type' => 'fdl',
			'tb' => g_l('cockpit', '[kv_failedLogins][headline]')
		]);
		echo we_html_tools::getHtmlTop(g_l('cockpit', '[kv_failedLogins][headline]') . ' (' . $this->maxRows . ')', $jsCmd->getCmds(), we_html_element::body([
				'style' => 'margin:10px 15px;',
				], we_html_element::div(['id' => 'fdl'], we_html_element::div(['id' => 'fdl_data'], $this->failedLoginHTML)
		)));
	}

}
