<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3f7c254b2b0c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-10-16 12:30 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_workflow_log{
	const TYPE_APPROVE = 'approve';
	const TYPE_APPROVE_FORCE = 'approve_force';
	const TYPE_DECLINE = 'decline';
	const TYPE_DECLINE_FORCE = 'decline_force';
	const TYPE_DOC_FINISHED = 'doc_finished';
	const TYPE_DOC_FINISHED_FORCE = 'doc_finished_force';
	const TYPE_DOC_INSERTED = 'doc_inserted';
	const TYPE_DOC_REMOVED = 'doc_removed';
	const NUMBER_LOGS = 8;

	/** @var int */
	public static $count = 0;

	static function logDocumentEvent(int $workflowDocID, int $userID, string $type, string $description = '', int $step = 0): void{
		DB_WE::$inst->query('INSERT INTO ' . WORKFLOW_LOG_TABLE . ' SET ' . we_database_base::arraySetter([
				'RefID' => $workflowDocID,
				'userID' => $userID,
				'docStepID' => $step,
				'Event' => $type,
				'Description' => $description
		]));
	}

	static function getLogForDocument(int $docID, string $order = 'DESC'): array{
		$offset = we_base_request::int('offset', 0);
		$db = new DB_WE();

		$q = 'FROM ' . WORKFLOW_LOG_TABLE . ' log JOIN ' . WORKFLOW_DOC_TABLE . ' doc ON log.RefID=doc.ID JOIN ' . WORKFLOW_TABLE . ' wf ON doc.workflowID=wf.ID LEFT JOIN ' . USER_TABLE . ' u ON log.userID=u.ID WHERE doc.documentID=' . $docID . ' ORDER BY log.Date ' . $db->escape($order) . ',doc.ID DESC';

		self::$count = fI('SELECT COUNT(1) ' . $q);

		$hash = $db->getAllq('SELECT log.*,DATE_FORMAT(log.Date, "' . g_l('date', '[format][mysql]') . '") AS Datum,u.First,u.Second,u.username ' . $q . ' LIMIT ' . $offset . ', ' . self::NUMBER_LOGS);

		foreach($hash as &$v){
			switch($v['Event']){
				case self::TYPE_APPROVE:
					$v['Event'] = g_l('modules_workflow', '[log_approve]');
					break;
				case self::TYPE_APPROVE_FORCE:
					$v['Event'] = g_l('modules_workflow', '[log_approve_force]');
					break;
				case self::TYPE_DECLINE:
					$v['Event'] = g_l('modules_workflow', '[log_decline]');
					break;
				case self::TYPE_DECLINE_FORCE:
					$v['Event'] = g_l('modules_workflow', '[log_decline_force]');
					break;
				case self::TYPE_DOC_FINISHED:
					$v['Event'] = g_l('modules_workflow', '[log_doc_finished]');
					break;
				case self::TYPE_DOC_FINISHED_FORCE:
					$v['Event'] = g_l('modules_workflow', '[log_doc_finished_force]');
					break;
				case self::TYPE_DOC_INSERTED:
					$v['Event'] = g_l('modules_workflow', '[log_insert_doc]');
					break;
				case self::TYPE_DOC_REMOVED:
					$v['Event'] = g_l('modules_workflow', '[log_remove_doc]');
					break;
			}
		}
		return $hash;
	}

	static function getLogForUser(int $userID): array{
		return getHash('SELECT * FROM ' . WORKFLOW_LOG_TABLE . ' WHERE userID=' . $userID);
	}

	static function clearLog(?DateTime $stamp): void{
		DB_WE::$inst->query('DELETE FROM ' . WORKFLOW_LOG_TABLE . ($stamp ? ' WHERE Date<"' . $stamp->format(we_database_base::DATE_TIME_FORMAT) . '"' : ''));
	}

}
