#!/usr/bin/php
<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/*
 * The script exports webEdition backup file to the given file
 * webEdition must be installed
 */
define('WE_DOCUMENT_ROOT', (string) realpath(__DIR__ . '/../../../'));
print("This script is currently unsupported due to changes on main core.");
exit(1);

// include needed libraries
require_once(WE_DOCUMENT_ROOT . '/webEdition/we/include/we.inc.php');

if(!defined('WE_ERROR_SHOW')){
	define('WE_ERROR_SHOW', 1);
}
if(!defined('WE_ERROR_LOG')){
	define('WE_ERROR_LOG', 1);
}

we_base_errorHandler::start(false);
// we want to see errors
ini_set('display_errors', '1');
error_reporting(E_ALL);

abstract class recoverBackup extends we_backup_cmd{
	static $backupFile = WE_DOCUMENT_ROOT . '/weBackup_daily.zip';

	const LONG_OPTS = [
		'all' => 'Backups everything except external files',
		we_modules_info::CORE => 'Documents and templates',
		we_backup_settings::OPTION_BACKUP_VERSIONS => 'Versions',
		we_backup_settings::OPTION_BACKUP_VERSION_BIN => 'Versions binary',
		we_backup_settings::OPTION_BACKUP_BINARY => 'Binary data',
		we_modules_info::USERS => 'User data',
		we_modules_info::CUSTOMER => 'Customer data',
		we_modules_info::SHOP => 'Shop data',
		we_modules_info::WORKFLOW => 'Workflow data',
		we_modules_info::NEWSLETTER => 'Newsletter data',
		we_modules_info::OBJECT => 'Objects and Classes',
		we_modules_info::SCHEDULER => 'Scheduler data',
		we_backup_settings::OPTION_BACKUP_SETTINGS => 'Settings',
		we_backup_settings::OPTION_BACKUP_CONFIG => 'Configuration Data',
		we_modules_info::EXPORT => 'webEdition-Export Data',
		we_modules_info::VOTING => 'Voting data',
		we_modules_info::GLOSSARY => 'Glossary data',
		we_modules_info::SEARCH => 'Search data',
		we_backup_settings::OPTION_BACKUP_HISTORY => 'History data',
		we_backup_settings::OPTION_BACKUP_HOOKS => 'Hooks',
		we_backup_settings::OPTION_BACKUP_CUSTOMTAGS => 'Custom tags',
		we_backup_settings::OPTION_BACKUP_EXTERN => 'Extern files',
		'verbose',
		'help',
	];
	const HELP = 'Usage: recoverBackup.php [options] [file]
Options:
  -v, --verbose              Verbosely list files processed

  --help                     Prints out this help

 Options what to recover:
';

	public static function _checkAll(bool $flag = true): void{
		foreach(array_keys(self::LONG_OPTS) as $key){
			$_REQUEST['handle_' . $key] = $flag;
		}
	}

	public static function getOptions($args){
// Short options
		$short_opts = 'v';

// Convert the arguments to options - check for the first argument
		return ($_SERVER['argv'] && realpath($_SERVER['argv'][0]) == __FILE__ ?
			Console_Getopt::getOpt($args, $short_opts, array_keys(self::LONG_OPTS)) :
			Console_Getopt::getOpt2($args, $short_opts, array_keys(self::LONG_OPTS)));
	}

	public static function showHelp(): void{
		fwrite(STDERR, self::HELP . "\n");
		foreach(self::LONG_OPTS as $key => $val){
			if(!$val){
				continue;
			}
			fwrite(STDERR, ' --' . str_pad((string) $key, 20, ' ') . $val . "\n");
		}
	}

	public static function doRecover(): void{
		// knock out identifiation and permissions
		$_SESSION['user']['isWeSession'] = true;
		$_SESSION['perms'] = ['ADMINISTRATOR' => true];
		$_SESSION['user']['Username'] = 1;


// Define exit codes for errors
		define('NO_ARGS', 10);
		define('INVALID_OPTION', 11);

// Reading the incoming arguments - same as $argv
		$args = Console_Getopt::readPHPArgv();


// Make sure we got them (for non CLI binaries)
		if(PEAR::isError($args)){
			fwrite(STDERR, $args->getMessage() . "\n");
			exit(NO_ARGS);
		}

		$options = self::getOptions($args);

// Check the options are valid
		if(PEAR::isError($options)){
			fwrite(STDERR, $options->getMessage() . "\n");
			self::showHelp();
			exit(INVALID_OPTION);
		}
		if($args){
			$_REQUEST['verbose'] = false;
			self::_checkAll(false);
			$_REQUEST['handle_extern'] = false;
		}

		foreach($options[0] as $opt){
			switch($opt[0]){
				case '--all':
					self::_checkAll(true);
					break;

				case 'v':
				case '--verbose':
					$_REQUEST['verbose'] = true;
					break;

				case '--help':
					self::showHelp();
					exit(0);

				default:
					$key = preg_replace('/^--/', '', $opt[0]);
					if(array_key_exists($key, self::LONG_OPTS)){
						$_REQUEST['handle_' . $key] = true;
					}
			}
		}


// check if no option is checked

		$optionsSelected = false;

		foreach($_REQUEST as $_k => $_v){
			if(str_starts_with($_k, "handle_")){
				if($_v){
					$optionsSelected = true;
					break;
				}
			}
		}

// if no option is checked, then check the dafaults
		if(!$optionsSelected){
			$_REQUEST['handle_core'] = true;
		}


		if(isset($options[1][0])){
			self::$backupFile = $options[1][0];
		}

		$_REQUEST['backup_select'] = basename(self::$backupFile);
		if(self::$backupFile != WE_DOCUMENT_ROOT . WE_BACKUP_DIR . $_REQUEST['backup_select']){
			copy(self::$backupFile, WE_DOCUMENT_ROOT . WE_BACKUP_DIR . $_REQUEST['backup_select']);
		}

		if(!we_backup_preparer::prepareImport()){
			print we_backup_preparer::getErrorMessage();
			return;
		}

		if($_REQUEST['verbose']){
			print "\nImporting from " . self::$backupFile . "...\n";
		}
		while(self::import()){

		}

		we_updater::doUpdate('internal');

		we_backup_settings::clearSessionData();
	}

	protected static function issueNext(string $type, string $description): void{
		static $lastDesc = '';
		if($lastDesc !== $description){
			$lastDesc = $description;
			echo "\n" . $description . "\n";
		}
		echo "\r" . we_backup_util::getImportPercent() . '  ';
	}

	protected static function issueLastImport(we_base_jsCmd $jsCmd): void{
		$jsCmd->clear();
		if($_REQUEST['verbose']){
			print "\nDone\n";
		}
	}

}

if(file_exists(WE_DOCUMENT_ROOT . '/webEdition/custom/conf/cliConfig.inc.php')){
	require_once(WE_DOCUMENT_ROOT . '/webEdition/custom/conf/cliConfig.inc.php'); // @phan-suppress-current-line PhanMissingRequireFile

	if(!isset($_SERVER['SERVER_NAME'])){
		$_SERVER['SERVER_NAME'] = $SERVER_NAME;
	}
	recoverBackup::doRecover();
} else {
	makeBackup::showHelp();
}