<?php
/**
 * webEdition CMS
 *
 * $Rev: 2195999f8065 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-19 08:54 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
defined('WE_DOCUMENT_ROOT') || define('WE_DOCUMENT_ROOT', (string) realpath(__DIR__ . '/../../../'));

function updateServerSettings(){
	$_SERVER['WE_ORG_ROOT'] = $_SERVER['DOCUMENT' . '_ROOT'];
	$_SERVER['DOCUMENT' . '_ROOT'] = WE_DOCUMENT_ROOT;

	foreach(['HTTP_USER_AGENT', 'PHP_SELF', 'REQUEST_URI', 'QUERY_STRING', 'REDIRECT_URL'] as $cur){
		if(isset($_SERVER[$cur])){
			$_SERVER[$cur] = strtr($_SERVER[$cur], ['<' => '%3C', '>' => '%3E', '"' => '%22', '\'' => '%27', '`' => '%60']);
		}
	}

//due to hoster bugs (1&1) we have to ensure servername is the called url. since http-host is not safe, we do some security additions.
	if(isset($_SERVER['HTTP_HOST']) && $_SERVER['SERVER_NAME'] != $_SERVER['HTTP_HOST']){
		//some security checks
		if(strlen($_SERVER['HTTP_HOST']) < 256 && strpos($_SERVER['HTTP_HOST'], $_SERVER['SERVER_NAME'])){
			$_SERVER['SERVER_NAME'] = rawurlencode($_SERVER['HTTP_HOST']);
		}
	}

// Set PHP flags
	ini_set('allow_url_fopen', '1');
	ini_set('file_uploads', '1');
	ini_set('session.use_trans_sid', '0');
	ini_set('max_input_vars', '9000');
//@ini_set("arg_separator.output","&");
//fix insecure cookies
	$https = isset($_SERVER['HTTPS']) && (strtoupper($_SERVER['HTTPS']) === 'ON' || $_SERVER['HTTPS'] == 1);
	if(version_compare(PHP_VERSION, '7.3.0') >= 0){
		session_set_cookie_params(['secure' => $https, 'httponly' => true, 'samesite' => 'strict']); //@phan-suppress-current-line PhanTypeMismatchArgumentInternalProbablyReal
	} else {
		$cookie = session_get_cookie_params();
		session_set_cookie_params($cookie['lifetime'], $cookie['path'], $cookie['domain'], $https, true);
	}
	/* if(!(defined('SYSTEM_WE_SESSION') && SYSTEM_WE_SESSION) && ini_get('session.gc_probability') != '0'){
	  ini_set('session.gc_probability', '0');
	  } */
	if(!defined('DATETIME_INITIALIZED')){// to prevent additional initialization if set somewhere else, i.e in autoload, this also allows later to make that an settings-item
		if(!date_default_timezone_set(@date_default_timezone_get())){
			date_default_timezone_set('Europe/Berlin');
		}
		define('DATETIME_INITIALIZED', 1);
	}
}

//prepare space for we-variables; $_SESSION['weS'] is set in we_session
//start autoloader!
require_once('we_autoload.inc.php');
updateServerSettings();
if(!we_base_WE::$Language){
	we_base_WE::$Language = WE_LANGUAGE;
	we_base_WE::$Langs = file_exists(WE_CUSTOM_PATH . 'conf/we_conf_language.inc.php') ? //FIXME: remove in cardada
		include(WE_CUSTOM_PATH . 'conf/we_conf_language.inc.php') :
		include(WE_INCLUDES_PATH . 'conf/we_conf_language.inc.php');
}

//register all used tables.
we_base_request::registerTables([
	'CAPTCHADEF_TABLE' => CAPTCHADEF_TABLE,
	'CAPTCHA_TABLE' => CAPTCHA_TABLE,
	'CLEAN_UP_TABLE' => CLEAN_UP_TABLE,
	'CONTENT_TABLE' => CONTENT_TABLE,
	'DOC_TYPES_TABLE' => DOC_TYPES_TABLE,
	'ERROR_LOG_TABLE' => ERROR_LOG_TABLE,
	'FAILED_LOGINS_TABLE' => FAILED_LOGINS_TABLE,
	'FILELINK_TABLE' => FILELINK_TABLE,
	'FILE_TABLE' => FILE_TABLE,
	'FORMMAIL_BLOCK_TABLE' => FORMMAIL_BLOCK_TABLE,
	'FORMMAIL_LOG_TABLE' => FORMMAIL_LOG_TABLE,
	'HISTORY_TABLE' => HISTORY_TABLE,
	'INDEX_TABLE' => INDEX_TABLE,
	'LANGLINK_TABLE' => LANGLINK_TABLE,
	'LOCK_TABLE' => LOCK_TABLE,
	'METADATA_TABLE' => METADATA_TABLE,
	'NOTEPAD_TABLE' => NOTEPAD_TABLE,
	'PREFS_TABLE' => PREFS_TABLE,
	'PWDRESET_TABLE' => PWDRESET_TABLE,
	'RECIPIENTS_TABLE' => RECIPIENTS_TABLE,
	'SEARCH_TABLE' => SEARCH_TABLE,
	'SESSION_TABLE' => SESSION_TABLE,
	'SETTINGS_TABLE' => SETTINGS_TABLE,
	'TEMPLATES_TABLE' => TEMPLATES_TABLE,
	'UPDATE_LOG_TABLE' => UPDATE_LOG_TABLE,
	'VALIDATION_SERVICES_TABLE' => VALIDATION_SERVICES_TABLE,
	'VERSIONSLOG_TABLE' => VERSIONSLOG_TABLE,
	'VERSIONS_TABLE' => VERSIONS_TABLE,
]);
//start db
$GLOBALS['DB_WE'] = DB_WE::getStatic(); //FIXME: remove global var
//start memcache
we_cache_we::setup();

we_modules_info::loadConfigs();

if(!(defined('NO_SESS') || defined('FROM_WE_SHOW_DOC'))){
	we_base_session::initSession();
}
we_base_WE::$Language = (!empty($_SESSION['prefs']['Language']) ?
	(is_dir(we_base_country::LANGDIR . $_SESSION['prefs']['Language']) ?
	$_SESSION['prefs']['Language'] :
	//  bugfix #4229
	($_SESSION['prefs']['Language'] = WE_LANGUAGE)) :
	WE_LANGUAGE);

if(!isset($GLOBALS['WE_IS_DYN'])){ //only true on dynamic frontend pages
	//send header?
	switch(isset($_REQUEST['we_cmd']) && !is_array($_REQUEST['we_cmd']) ? we_base_request::string('we_cmd', '__default__') : ''){
		case 'edit_link':
		case 'show_newsletter':
		case 'save_document':
			$header = false;
			break;
		case 'reload_hot_editpage':
		case 'reload_editpage':
			$header = (!in_array($_SESSION['weS']['EditPageNr'], [we_base_constants::WE_EDITPAGE_CONTENT, we_base_constants::WE_EDITPAGE_PREVIEW, we_base_constants::WE_EDITPAGE_PROPERTIES], true));
			break;
		case 'switch_edit_page':
			$header = (!in_array(we_base_request::int('we_cmd', -1, 1), [we_base_constants::WE_EDITPAGE_CONTENT, we_base_constants::WE_EDITPAGE_PREVIEW,
					we_base_constants::WE_EDITPAGE_PROPERTIES], true));
			break;
		case 'load_editor':
			$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction', '__NO_TRANS__');
			$doc = we_contents_root::getSessionDocument($we_transaction);
			$header = (!(
				$doc->Table === FILE_TABLE &&
				$_SESSION['weS']['EditPageNr'] == we_base_constants::WE_EDITPAGE_PREVIEW
				));
			break;
		case '__default__':
		default:
			$header = true;
	}

	if($header){
		we_html_tools::headerCtCharset('text/html');
	}
	unset($header);
}
