/* global top, WE */

/**
 * webEdition CMS
 *
 * $Rev: 393e00c8028a $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-16 14:27 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software, you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

function initMove(table) {
	top.treeData.setState(top.treeData.tree_states.select);
	if (top.treeData.table !== table) {
		top.treeData.table = table;
		we_cmd("load", table);
	} else {
		we_cmd("load", table);
		top.drawTree();
	}
}

function we_cmd() {
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window),
		args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments));
	//var url = WE().util.getWe_cmdArgsUrl(args);
	switch (args[0]) {
		case "moveInfo":
			(new (WE().layout.overlay)(window, 'top', WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=moveInfo", "we_moveinfo", WE().consts.size.dialog.small, WE().consts.size.dialog.small)).open();
			break;
		case "moveTreeEntries":
			moveTreeEntries(args[1]);
			break;
		case "start_multi_editor":
			top.we_cmd("start_multi_editor");
			break;
		case "close":
			window.close();
			break;
		case "moveCloseEditors":
			var open_move_editors = args[1],
				move_table = args[2];
			WE().layout.weEditorFrameController.closeAllEditors(open_move_editors, true);
			we_cmd('do_move', '', move_table);
			break;
		default:
			window.parent.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
	}
}

function moveTreeEntries(dontMoveClassFolders) {
	var obj = top.treeData.elements;
	var cont = new top.container();
	for (var i = 1; i <= obj.len; i++) {
		if (!obj[i].checked || (dontMoveClassFolders && !obj[i].parentid)) {
			if (obj[i].parentid) {
				if (!top.treeData.parentChecked(obj[i].parentid)) {
					cont.add(obj[i]);
				}
			} else {
				cont.add(obj[i]);
			}
		}
	}
	top.treeData.elements = cont.elements;
	top.treeData.len = cont.len;
	top.drawTree();
}

function we_submitForm(target, url) {
	var f = window.document.we_form;
	if (!f.checkValidity()) {
		WE().util.showMessage(WE().consts.g_l.main.save_error_fields_value_not_valid, WE().consts.message.WE_MESSAGE_ERROR, window);
		return false;
	}
	var selected = [];
	for (var i = 1; i <= top.treeData.len; i++) {
		if (top.treeData.elements[i].checked) {
			selected.push(top.treeData.elements[i].id);
		}
	}
	if (!selected) {
		WE().util.showMessage(WE().consts.g_l.main.nothing_to_move, WE().consts.message.WE_MESSAGE_ERROR, window);
		return false;
	}

	f.sel.value = selected.join(',');
	f.target = target;
	f.action = url;
	f.method = "post";
	f.submit();
	return true;
}

function press_ok_move(type) {
	var
		sel = [],
		open_move_editors = [],
		usedEditors = WE().layout.weEditorFrameController.getEditorsInUse(),
		move_table = top.treeData.table,
		openDocs_Str = "";

	for (var i = 1; i <= top.treeData.len; i++) {
		if (top.treeData.elements[i].checked) {
			sel.push(top.treeData.elements[i].id);
		}
	}
	if (!sel) {
		WE().util.showMessage(WE().consts.g_l.main.nothing_to_move, WE().consts.message.WE_MESSAGE_ERROR, window);
		return;
	}

	// check if selected target exists
	if (!WE().layout.weSuggest.checkRequired(window)) {
		WE().util.showMessage(WE().consts.g_l.main.notValidFolder, WE().consts.message.WE_MESSAGE_ERROR, window);
		return;
	}

	// close all documents before moving.

	// no open document can be moved
	// close all Editors with moved documents
	for (var frameId in usedEditors) {
		if (move_table === usedEditors[frameId].getEditorEditorTable() &&
			sel.indexOf(usedEditors[frameId].getEditorDocumentId()) > -1
			) {
			open_move_editors.push(usedEditors[frameId]);
		}
	}
	if (open_move_editors.length) {
		for (i = 0; i < open_move_editors.length; i++) {
			openDocs_Str += "<li>" + open_move_editors[i].getEditorDocumentPath() + "</li>";
		}
		WE().util.showConfirm(window, "", (WE().util.sprintf(WE().consts.g_l.alert.move_exit_open_docs_question, type, type) + "<ul>" + openDocs_Str + "</ul>" + WE().consts.g_l.alert.move_exit_open_docs_continue), [
			'moveCloseEditors', open_move_editors, move_table], ['close']);

	} else {
		WE().util.showConfirm(window, "", WE().consts.g_l.alert.move, ['do_move', '', move_table]);
	}
}
