/* global WE, errorHandler,treeData,drawTree, top */

/**
 * webEdition CMS
 *
 * $Rev: 6329b796be62 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-10 16:09 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
WE().util = {
	multi_editMulti: null,
	multi_edit: null,
	weSetCookie: function (doc, name, value, expires, path, domain) {
		doc.cookie = name + "=" + encodeURIComponent(value) +
			((expires === undefined) ? "" : "; expires=" + expires.toGMTString()) +
			((path === undefined) ? "" : "; path=" + path) +
			((domain === undefined) ? "" : "; domain=" + domain);
	},
	weGetCookie: function (doc, name) {
		var cname = name + "=";
		var dc = doc.cookie;
		if (dc.length > 0) {
			var begin = dc.indexOf(cname);
			if (begin !== -1) {
				begin += cname.length;
				var end = dc.indexOf(";", begin);
				if (end === -1) {
					end = dc.length;
				}
				return decodeURIComponent(dc.substring(begin, end));
			}
		}
		return null;
	},
	hashCode: function (s) {
		/*jshint bitwise:false */
		return s.split("").reduce(function (a, b) {
			a = ((a << 5) - a) + b.charCodeAt(0);
			return a & a;
		}, 0);
	},
	we_sbmtFrm: function (target, url, source) {
		if (source === undefined) {
			source = WE().layout.weEditorFrameController.getVisibleEditorFrame();
		}

		if (source) {
			try {
				if (source.we_submitForm !== undefined && source.we_submitForm) {
					return source.we_submitForm(target.name, url);
				}
				if (source.contentWindow && source.contentWindow.we_submitForm) {
					return source.contentWindow.we_submitForm(target.name, url);
				}
			} catch (e) {
				WE().t_e("Submit problem", source.name, target.name, url);
			}
		}
		return false;
	},
	hasPerm: function (perm) {
		return (WE().session.permissions.ADMINISTRATOR || WE().session.permissions[perm]);
	},
	/**
	 * This function sets incons inside elements of a given class. The element must have the property data-contenttype and data-extension set to determine the correct icon
	 * @param string classname the elements classname
	 * @returns noting
	 */
	setIconOfDocClass: function (doc, classname) {
		var elements = doc.getElementsByClassName(classname);
		for (var i = 0; i < elements.length; i++) {
			elements[i].innerHTML = this.getTreeIcon(elements[i].getAttribute("data-contenttype"), false, elements[i].getAttribute("data-extension"));
		}
	},
	/**
	 * Get a file icon out of a given type, used in tree, selectors & tabs
	 * @param {string} contentType
	 * @param {boolean} open
	 * @returns icon to be drawn as html-code
	 */
	getTreeIcon: null,
	sprintf: function (argum) {
		if (!arguments || arguments.length === 0) {
			return;
		}

		var regex = /([^%]*)%([%ds])(.*)/,
			arr,
			iterator = 0,
			ret = "",
			check = argum,
			type;

		while ((arr = regex.exec(check))) {
			ret += arr[1];
			type = arr[2];
			check = arr[3];

			switch (type) {
				case "d":
					iterator++;
					ret += parseInt(arguments[iterator]) ? parseInt(arguments[iterator]) : 0;
					break;
				case "s":
					iterator++;
					ret += arguments[iterator];
					break;
				case "%":
					ret += "%";
			}
		}
		return ret + check;
	},
	IsDigitPercent: function (e) {
		var key = e.keyCode;
		switch (key) {
			case 37://%
			case 0:
			case 46://.
			case 101://e
			case 109://m
			case 13:
			case 8://???
			case 63272://???
				return true;
		}
		return (((key >= 48) && (key <= 57))/*0-9*/ || (key <= 63235 && key >= 63232));
	},
	getWe_cmdArgsArray: function (arr) {
		if (arr.lenght > 0 && typeof arr[0] === "object") {
			return arr[0];
		}
		return arr;
	},
	getWe_cmdArgsUrl: function (args, base) {
		var
			url = (base === undefined ? WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?" : base),
			pos = 0;

		if (Array.isArray(args)) {
			for (var i = 0; i < args.length; i++) {
				if (typeof (args[i]) !== 'object') {
					url += "we_cmd[" + pos + "]=" + encodeURIComponent(args[i]) + (i < (args.length - 1) ? "&" : "");
					pos++;
				} else if (args[i] !== null && typeof (args[i]) === 'object') {
					//url += this.getWe_cmdArgsUrlIterate(args[i]);
					url += Object.keys(args[i]).map(this.getWe_cmdArgsUrlMap.bind(this, args[i])).join("&");
				}
			}
		} else {
			//url += this.getWe_cmdArgsUrlIterate(args[i]);
			url += Object.keys(args).map(this.getWe_cmdArgsUrlMap.bind(this, args)).join("&");
		}

		return url;
	},
	/*
	 getWe_cmdArgsUrlIterate: function(obj) {
	 var params = [];
	 for(var key in obj){
	 if(obj.hasOwnProperty(key)) {
	 params.push("we_cmd[" + key + "]=" + encodeURIComponent(obj[key]));
	 }
	 }
	 return params.join("&");
	 },
	 */
	getWe_cmdArgsUrlMap: function (obj, key) {
		return "we_cmd[" + key + "]=" + encodeURIComponent(obj[key]);
	},
	showConfirm: null,
	showMessage: null,
	clip: function (doc, unique, width) {
		var text = doc.getElementById("td_" + unique);
		var btn = doc.getElementById("btn_" + unique).firstChild;

		if (text.classList.contains("cutText")) {
			text.classList.remove("cutText");
			text.style.maxWidth = "";
			btn.classList.remove("fa-caret-right");
			btn.classList.add("fa-caret-down");
		} else {
			text.classList.add("cutText");
			text.style.maxWidth = "100%";
			btn.classList.remove("fa-caret-down");
			btn.classList.add("fa-caret-right");
		}
	},
	setTextareaCount: function (value, trgt) {
		var words = value.match(/\w+/g);
		trgt.innerHTML = (words ? words.length : 0) + " / " + value.length;

	},
	hasPermDelete: function (eTable, isFolder) {
		if (eTable === "") {
			return false;
		}
		if (WE().session.permissions.ADMINISTRATOR) {
			return true;
		}
		switch (eTable) {
			case WE().consts.tables.FILE_TABLE:
				return (isFolder ? WE().session.permissions.DELETE_DOC_FOLDER : WE().session.permissions.DELETE_DOCUMENT);
			case WE().consts.tables.TEMPLATES_TABLE:
				return (isFolder ? WE().session.permissions.DELETE_TEMP_FOLDER : WE().session.permissions.DELETE_TEMPLATE);
			case WE().consts.tables.OBJECT_FILES_TABLE:
				return (isFolder ? WE().session.permissions.DELETE_OBJECTFILE : WE().session.permissions.DELETE_OBJECTFILE);
			case WE().consts.tables.OBJECT_TABLE:
				return (isFolder ? false : WE().session.permissions.DELETE_OBJECT);
			case WE().consts.tables.VFILE_TABLE:
				return (isFolder ? WE().session.permissions.DELETE_COLLECTION_FOLDER : WE().session.permissions.DELETE_COLLECTION);
			default:
				return false;
		}
	},
	validate: {
		email: function (email) {
			email = email.replace(/".*"/g, "y");
			email = email.replace(/\\./g, "z");
			var parts = email.split("@");
			if (parts.length !== 2) {
				return false;
			}
			if (!WE().util.validate.domain(parts[1])) {
				return false;
			}
			return (parts[0].match(/(.+)/));
		},
		domain: function (domain) {
			var parts = domain.split(".");
			//if(parts.length>3 || parts.length<1) return false;
			//if(parts.length===1 && !WE().util.validate.domainname(parts[0])) return false;
			for (var i = 0; i < (parts.length - 1); i++) {
				if (!WE().util.validate.domainname(parts[i])) {
					return false;
				}
			}
			return (parts[parts.length - 1].match(/^[a-z][a-z]+$/i));
		},
		domainname: function (domainname) {
			var pattern = /^[^_\-\s/=?\*"'#!§$%&()\[\]\{\};:,°<>\|][^\s/=?\*"'#!§$%&()\[\]\{\};:,°<>\|]+$/i;
			return (domainname.match(pattern));
		},
		date: function () {
			// TODO
		},
		currency: function () {
			// TODO
		}
	},
	/**
	 * load consts if not present in global scope
	 * @param {HTMLDocument} doc reference to the current document, where we can load the consts
	 * @param {string} check if WE().consts doesn't have "check" as property, we load the consts with a request
	 * @returns {nothing}
	 */
	loadConsts: function (doc, check) {
		var cur = WE().consts;
		var found = true;
		var what = check.split(".");
		for (var i = 0; i < what.length; i++) {
			if (cur[what[i]] === undefined || !cur[what[i]]) {
				found = false;
				break;
			}
			cur = cur[what[i]];
		}
		if (found) {
			return;
		}
		$.ajax({
			async: false,
			type: "GET",
			cache: true,
			url: WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=loadJSConsts&we_cmd[1]=" + check,
			success: function (data) {
				cur[what[i]] = data;
			},
			dataType: "json"
				//timeout: 2000
		}).fail(function (jqxhr, textStatus, error) {
			WE().t_e('JS rpc (loadConsts) failed', textStatus, error, jqxhr.responseText, this.url);
		});
	},
	/**
	 * load variable data from a tag
	 * @param {document} doc the document containing the tag
	 * @param {string} id element containing the data
	 * @param {string} dataname the attribute of the element containing the data
	 * @returns {Array|Object} the decoded data
	 */
	getDynamicVar: function (doc, id, dataname) {
		var el = doc.getElementById(id);
		return (el ?
			this.decodeDynamicVar(el, dataname) :
			null
			);
	},
	/**
	 * internal for decoding the data of a tag
	 */
	decodeDynamicVar: function (el, dataname) {
		var data = el.getAttribute(dataname);
		return data ? JSON.parse(window.atob(data)) : null;
	},
	/**
	 * @param {string} url
	 * @param {string} data string of data to post, e.g. JSON.stringify(array)
	 * @param {function} success function with data, function (weResponse)  => weResponse.DataArray.data holds the data by WE
	 * @param {string} type default json
	 * @returns {nothing}
	 */
	rpc: function (url, data, success, type) {
		return $.ajax({
			type: "POST",
			url: url,
			data: data,
			success: success,
			dataType: (type ? type : "json")
				//timeout: 2000
		}).fail(function (jqxhr, textStatus, error) {
			if (jqxhr.status > 0) {//it is only an error if status > 0 (e.g. site moved, ...)
				WE().t_e('JS rpc failed', textStatus, error, jqxhr.responseText, this.url, jqxhr);
			}
		});
	},
	processJSCmd: function (cmd, cmdData) {
		var caller = (this && this.window === this ? this : window);

		//Keep switch clean, only few statements should be here, everthing else goes to specific js file
		switch (cmd) {
			case 'msg':
				WE().util.showMessage(cmdData.msg, cmdData.prio ? cmdData.prio : WE().consts.message.WE_MESSAGE_NOTICE, caller);
				break;
			case 'close':
				caller.top.close();
				break;
			case 'close_dialog':
				WE().util.dialogTop(caller).doClose();
				break;
			case 'close_delayed':
				window.setTimeout(caller.top.close.bind(caller.top), cmdData);
				break;
			case 'close_dialog_delayed':
				window.setTimeout(WE().util.dialogTop(caller).doClose.bind(WE().util.dialogTop(caller).doClose), cmdData);
				break;
			case 'history.back':
				history.back();
				break;
			case 'we_cmd':
				if (caller.we_cmd) {
					caller.we_cmd.apply(caller, cmdData);
				} else if (caller.parent.we_cmd) {
					caller.parent.we_cmd.apply(caller, cmdData);
				} else if (caller.parent.parent.we_cmd) {
					caller.parent.parent.we_cmd.apply(caller, cmdData);
				} else if (caller.top.we_cmd) {
					caller.top.we_cmd.apply(caller, cmdData);
				} else {
					top.we_cmd.apply(caller, cmdData);
				}
				break;
			case 'location':
				switch (cmdData.doc) {
					case 'document':
						caller.document.location = cmdData.loc;
						break;
					case 'body':
						caller.top.body.document.location = cmdData.loc;
						break;
				}
				break;
			default:
				//if nothing matched, we set arg[0]=cmd & pass the whole argument to we_cmd
				if (caller.we_cmd) {//direct match
					caller.we_cmd.call(caller, cmd, cmdData);
				} else if (caller.parent.we_cmd) {//match content frame
					caller.parent.we_cmd.call(caller, cmd, cmdData);
				} else if (caller.parent.parent.we_cmd) {//in view component (on modules)
					caller.parent.parent.we_cmd.call(caller, cmd, cmdData);
				} else if (caller.top.we_cmd) {
					caller.top.we_cmd.call(caller, cmd, cmdData);
				} else { // at last try the main we_cmd
					top.we_cmd.call(window, cmd, cmdData);
				}
		}
	},
	dialogTop: function (win) {
		var
			frameElement,
			cur = win;

		for (var i = 0; i < 10; i++) {
			cur = (i === 0) ? cur : cur.parent;
			if (cur === cur.parent) { // reached top and not found weOverlay_frameElement
				cur.doClose = cur.close.bind(cur);

				return cur;
			}

			if ((frameElement = cur.frameElement) && frameElement.getAttribute('data-overlayframe')) {
				if (!cur.overlay) {
					var overlay = frameElement.parentNode.getElementsByClassName('adminFrame')[0].overlay;

					cur.opener = overlay.opener;
					cur.doClose = overlay.doClose.bind(overlay);
					cur.toggleFullscreen = overlay.toggleFullscreen.bind(overlay);
					cur.resizeTo = overlay.resizeTo.bind(overlay);
					cur.overlay = overlay;
				}

				return cur;
			}
		}
	}
};

WE().consts = WE().util.getDynamicVar(document, 'loadWEData', 'data-consts');
WE().session = WE().util.getDynamicVar(document, 'loadWEData', 'data-session');
//finally load language files
WE().util.loadConsts(document, 'g_l');
WE().consts.browserDetect.small = window.matchMedia("(max-device-width: 800px)").matches || window.matchMedia("(max-device-height: 600px)").matches;
