/* global WE */

/**
 * webEdition CMS
 *
 * $Rev: 2cb1d7d5c6fa $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-21 01:53 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

var ImageDialog = {
	tinymcePlugin: window.tinyMCEPopup,
	editableSelects: window.TinyMCE_EditableSelects,
	tinymce: window.tinymce,

	init: function (ed) {
		var form = document.forms.we_form;
		var elems = form.elements;

		ed = this.tinymcePlugin.editor;
		var dom = ed.dom;
		var node = ed.selection.getNode();

		document.getElementById('we_dialog_args[cssclass]').innerHTML = this.tinymcePlugin.getClassOptions('img');
		this.editableSelects.init();

		if (node.nodeName === 'IMG' && !ed.isWeDataInitialized) {
			var imgWidth, imgHeight, longdesc;
			var src = dom.getAttrib(node, 'src'),
				isThumbnail = src.indexOf('?thumb=') !== -1;

			// load attributes into form
			imgWidth = isThumbnail ? 0 : parseInt(dom.getAttrib(node, 'width'));
			imgHeight = isThumbnail ? 0 : parseInt(dom.getAttrib(node, 'height'));

			elems["we_dialog_args[width]"].value = imgWidth;
			elems["we_dialog_args[height]"].value = imgHeight;
			elems["we_dialog_args[rendered_width]"].value = imgWidth;
			elems["we_dialog_args[rendered_height]"].value = imgHeight;
			elems["we_dialog_args[vspace]"].value = parseInt(dom.getStyle(node, 'margin-top')) || parseInt(dom.getStyle(node, 'margin-bottom')) || dom.getAttrib(node, 'vspace');
			elems["we_dialog_args[hspace]"].value = parseInt(dom.getStyle(node, 'margin-left')) || parseInt(dom.getStyle(node, 'margin-right')) || dom.getAttrib(node, 'hspace');
			elems["we_dialog_args[border]"].value = parseInt(dom.getStyle(node, 'border-width')) || dom.getAttrib(node, 'border');
			elems["we_dialog_args[alt]"].value = dom.getAttrib(node, 'alt');
			elems["we_dialog_args[title]"].value = dom.getAttrib(node, 'title');
			ImageDialog.selectOptionByValue(form, "we_dialog_args[align]", dom.getAttrib(node, 'align'));
			longdesc = dom.getAttrib(node, 'longdesc');
			elems["we_dialog_args[longdescSrc]"].value = longdesc.split('?id=', 2)[1] ? longdesc.split('?id=', 2)[0] : '';
			elems["we_dialog_args[longdescID]"].value = longdesc.split('?id=', 2)[1] ? longdesc.split('?id=', 2)[1] : '';
			ImageDialog.selectOptionByValue(form, "we_dialog_args[cssclass]", dom.getAttrib(node, 'class'));
			if (imgHeight * imgWidth) {
				elems.tinyMCEInitRatioH.value = imgWidth / imgHeight;
				elems.tinyMCEInitRatioW.value = imgHeight / imgWidth;
			}
		}
	},

	writeBack: function (attributes, thumbdata) {
		if (!attributes) {
			WE().util.dialogTop(window).doClose();
			return;
		}

		var ed = this.tinymcePlugin.editor,
			attribs = attributes,
			v,
			args = {},
			el;

		// remove <img> if src=""
		if (attributes.src === '' || attributes.src === 'http://') {
			if (ed.selection.getNode().nodeName === 'IMG') {
				ed.dom.remove(ed.selection.getNode());
				ed.execCommand('mceRepaint');
			}
			WE().util.dialogTop(window).doClose();
			return;
		}

		if (false && this.tinymcePlugin.getParam("accessibility_warnings", 1)) {
			if (!attributes.alt) {
				//this.tinymcePlugin.confirm(this.tinymcePlugin.getLang('advimage_dlg.missing_alt'), function(s) {
				//if (s){
				//t.insertAndClose();
				//}
				//});
				return;
			}
		}

		attribs.vspace = parseInt(attribs.vspace);
		attribs.hspace = parseInt(attribs.hspace);
		let vspace = attribs.vspace ? (attribs.vspace + 'px') : 0,
			hspace = attribs.hspace ? (attribs.hspace + 'px') : 0;

		this.tinymce.extend(args, {
			src: attribs.src.replace(/ /g, '%20'),
			width: attribs.width,
			height: attribs.height,
			hspace: '',
			vspace: '',
			border: '',
			alt: attribs.alt,
			align: attribs.align,
			name: attribs.name,
			'class': parseInt(attribs.cssclass) !== -1 ? attribs.cssclass : '',
			title: attribs.title,
			longdesc: attribs.longdesc,
			style: (attribs.border && parseInt(attribs.border) !== 0 ? 'border:' + attribs.border + 'px  solid black;' : '') +// Tiny4: HTML5 // TODO: add color!
				(vspace || hspace ? ('margin:' + vspace + ' ' + hspace + ';') : '')
				//id : attribs.id,
				//dir : attribs.dir,
				//lang : attribs.lang,
				//usemap : attribs.usemap,
		});

		el = ed.selection.getNode();

		if (el && el.nodeName === 'IMG') {
			ed.dom.setAttribs(el, args);
		} else {
			ed.execCommand('mceInsertContent', false, '<img id="__mce_tmp" />', {skip_undo: 1});
			//ed.execCommand('mceInsertContent', false, '<img />', {skip_undo : 1});
			ed.dom.setAttribs('__mce_tmp', args);
			ed.dom.setAttrib('__mce_tmp', 'id', '');
			ed.undoManager.add();
		}

		this.tinymcePlugin.editor.execCommand('mceRepaint');
		this.tinymcePlugin.editor.focus();
		WE().util.dialogTop(window).doClose();
	},

	selectOptionByValue: function (form, selName, val) {
		if (form === undefined || form.elements[selName] === undefined && val === undefined) {
			return;
		}

		var i;
		if (!val) {
			form.elements[selName].options[0].selected = true;
			for (i = 1; i < form.elements[selName].options.length; i++) {
				form.elements[selName].options[i].selected = false;
			}
		} else {
			var found = false;
			for (i = 1; i < form.elements[selName].options.length; i++) {
				if (form.elements[selName].options[i].value === val) {
					form.elements[selName].options[i].selected = true;
					found = true;
				} else {
					form.elements[selName].options[i].selected = false;
				}
			}
			if (!found) {
				form.elements[selName].options[i] = new Option('--------------------------------------', '');
				form.elements[selName].options[i + 1] = new Option(val, val);
				form.elements[selName].options[i + 1].selected = true;
			}
		}
	},

	getAttrib: function (e, at) {
		var ed = this.tinymcePlugin.editor, dom = ed.dom, v, v2;

		if (ed.settings.inline_styles) {
			switch (at) {
				case 'align':
					if ((v = dom.getStyle(e, 'float'))) {
						return v;
					}

					if ((v = dom.getStyle(e, 'vertical-align'))) {
						return v;
					}

					break;

				case 'hspace':
					v = dom.getStyle(e, 'margin-left');
					v2 = dom.getStyle(e, 'margin-right');

					if (v && v === v2) {
						return parseInt(v.replace(/[^0-9]/g, ''));
					}

					break;

				case 'vspace':
					v = dom.getStyle(e, 'margin-top');
					v2 = dom.getStyle(e, 'margin-bottom');
					if (v && v === v2) {
						return parseInt(v.replace(/[^0-9]/g, ''));
					}

					break;

				case 'border':
					v = 0;

					this.tinymce.each(['top', 'right', 'bottom', 'left'], function (sv) {
						sv = dom.getStyle(e, 'border-' + sv + '-width');

						// False or not the same as prev
						if (!sv || (sv !== v && v !== 0)) {
							v = 0;
							return false;
						}

						if (sv) {
							v = sv;
						}
					});

					if (v) {
						return parseInt(v.replace(/[^0-9]/g, ''));
					}

					break;
			}
		}

		if ((v = dom.getAttrib(e, at))) {
			return v;
		}

		return '';
	}

	// lots of tinyMCE-functions removed

};

function weTinyDialog_doOk() {
	// obsolete?
}

$(function () {
	ImageDialog.init();
});