/* global tinymce */
/**
 * webEdition CMS
 *
 * $Rev: 6329b796be62 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-10 16:09 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

(function () {
	var weselects = (function () {
		var global = tinymce.util.Tools.resolve('tinymce.PluginManager');
		var initSelects = function (ed) {
			initSizeselect(ed);
			initBlockselect(ed);
			initStyleselect(ed);
		};

		var initSizeselect = function (ed) {
			var fontsizeBtn = ed.getParam('weWin').document.getElementsByName(ed.getParam('weName'))[0].nextSibling.querySelector('[title="' + ed.translate('Font Sizes') + '"]') ||
						ed.getParam('weWin').document.getElementsByName(ed.getParam('weName'))[0].nextSibling.querySelector('[title="Font sizes"]');
			if (fontsizeBtn) {
				var replaceBtn = fontsizeBtn.cloneNode(true);
				var menuState = ed.settings.weIsFontsizeSelectOpen;
				fontsizeBtn.style.width = 0;
				fontsizeBtn.style.padding = 0;
				replaceBtn.title = ed.translate('Font Sizes');
				replaceBtn.childNodes[0].innerHTML = ed.translate('Font Sizes');
				replaceBtn.addEventListener("click", handleclick, true);
				fontsizeBtn.parentNode.insertBefore(replaceBtn, fontsizeBtn.nextSibling);
				ed.on('NodeChange', function (e) {
					setText();
				});
			}
			function setText() {
				var node = ed.selection.getNode();
				var value = '';
				if (node.nodeName.toLowerCase() !== 'body') {
					do {
						if ((node && node.getAttribute('style') && node.getAttribute('style').indexOf('font-size') !== -1)) {
							if (node.style.fontSize && ed.settings.weSelects.fontsizeselect.formatted.length && ed.settings.weSelects.fontsizeselect.formatted.indexOf(node.style.fontSize) !== -1) {
								value = fontsizeBtn.firstChild.innerHTML;
							}
							break;
						}
					} while ((node = node.parentNode) && node.nodeName.toLowerCase() !== 'body');
				}
				replaceBtn.firstChild.innerHTML = value || ed.translate('Font Sizes');
			}
			function toggleState(e) {
				e.stopPropagation();
				menuState = !menuState;
				return false;
			}
			function handleclick() {
				if (!menuState) {
					fontsizeBtn.click();
					ed.getParam('weWin').addEventListener('click', toggleState);
					ed.iframeElement.contentWindow.addEventListener('click', toggleState);
				} else {
					menuState = false;
					ed.getParam('weWin').removeEventListener('click', toggleState);
					ed.iframeElement.contentWindow.removeEventListener('click', toggleState);
				}
			}
		};

		var initBlockselect = function (ed) {
			if (ed.settings.weSelects.blockselect.formatted.length && ed.settings.weSelects.blockselect.formatted.indexOf('p') === -1) {
				var btnTitle = WE().consts.g_l.tinyMceTranslationObject.selectsFormatsTitle;
				var blocksBtn = ed.getParam('weWin').document.getElementsByName(ed.getParam('weName'))[0].nextSibling.querySelector('[title="' + ed.translate('Blocks') + '"]') ||
						ed.getParam('weWin').document.getElementsByName(ed.getParam('weName'))[0].nextSibling.querySelector('[title="Blocks"]');
				if (blocksBtn) {
					var replaceBtn = blocksBtn.cloneNode(true);
					var menuState = ed.settings.weIsFormatSelectOpen;
					blocksBtn.style.width = 0;
					blocksBtn.style.padding = 0;
					replaceBtn.title = btnTitle;
					replaceBtn.childNodes[0].innerHTML = btnTitle;
					replaceBtn.addEventListener("click", handleclick, true);
					blocksBtn.parentNode.insertBefore(replaceBtn, blocksBtn.nextSibling);
					ed.on('NodeChange', function (e) {
						setText();
					});
				}
			}
			function setText() {
				var text = blocksBtn.firstChild.innerHTML;
				replaceBtn.firstChild.innerHTML = text === ed.translate('Paragraph') || text === ed.translate('Blocks') ||
					text === 'Paragraph' || text === 'Blocks' ? btnTitle : text;
			}
			function toggleState(e) {
				e.stopPropagation();
				menuState = !menuState;
				return false;
			}
			function handleclick() {
				if (!menuState) {
					blocksBtn.click();
					ed.getParam('weWin').addEventListener('click', toggleState);
					ed.iframeElement.contentWindow.addEventListener('click', toggleState);
				} else {
					menuState = false;
					ed.getParam('weWin').removeEventListener('click', toggleState);
					ed.iframeElement.contentWindow.removeEventListener('click', toggleState);
				}
			}

		};

		var initStyleselect = function (ed) {
			var btnTitle = WE().consts.g_l.tinyMceTranslationObject.selectsStylesTitle;
			var classesBtn = ed.getParam('weWin').document.getElementsByName(ed.getParam('weName'))[0].nextSibling.querySelector('[title="' + ed.translate('Formats') + '"]') ||
						ed.getParam('weWin').document.getElementsByName(ed.getParam('weName'))[0].nextSibling.querySelector('[title="Formats"]');
			if (classesBtn) {
				var replaceBtn = classesBtn.cloneNode(true);
				var menuState = ed.settings.weIsStyleSelectOpen;
				classesBtn.style.width = 0;
				classesBtn.style.padding = 0;
				replaceBtn.title = btnTitle;
				replaceBtn.childNodes[0].innerHTML = btnTitle;
				replaceBtn.addEventListener("click", handleclick, true);
				classesBtn.parentNode.insertBefore(replaceBtn, classesBtn.nextSibling);

				if (ed.plugins.importcss) {
					ed.on('NodeChange', function (e) {
						setText(e);
					});
				} else {
					ed.on('NodeChange', function (e) {
						syncText();
					});
				}
			}
			function setText() {
				var matches = ed.formatter.matchAll(ed.settings.weProcessedFormats.formats);
				switch (matches.length) {
					case 0:
						replaceBtn.firstChild.innerHTML = btnTitle;
						return;
					case 1:
						replaceBtn.firstChild.innerHTML = ed.settings.weProcessedFormats.byFormat[matches[0]];
						return;
					default:
						var node = ed.selection.getNode();
						do {
							for (var i = 0; i < matches.length; i++) {
								if (ed.formatter.matchNode(node, matches[i])) {
									replaceBtn.firstChild.innerHTML = ed.settings.weProcessedFormats.byFormat[matches[i]];
									return;
								}
							}
						} while ((node = node.parentNode) && node.nodeName.toLowerCase() !== 'body');
				}
			}
			function syncText() {
				var text = classesBtn.firstChild.innerHTML;
				replaceBtn.firstChild.innerHTML = text === ed.translate('Paragraph') || text === 'Paragraph' ? btnTitle : text;
			}
			function toggleState(e) {
				e.stopPropagation();
				menuState = !menuState;
				return false;
			}
			function handleclick() {
				if (!menuState) {
					classesBtn.click();
					ed.getParam('weWin').addEventListener('click', toggleState);
					ed.iframeElement.contentWindow.addEventListener('click', toggleState);
				} else {
					menuState = false;
					ed.getParam('weWin').removeEventListener('click', toggleState);
					ed.iframeElement.contentWindow.removeEventListener('click', toggleState);
				}
			}
		};

		global.add('weselects', function () {
			return {
				init: function (ed) {
					ed.on('Init', initSelects.bind(this, ed));
				}
			};
		});
		function Plugin() {}

		return Plugin;
	}());
})();