/* global tinyMCEPopup,SXE, tinyMCE */

/**
 * webEdition CMS
 *
 * $Rev: 6329b796be62 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-10 16:09 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

var tinymcePlugin = window.tinyMCEPopup;

function initCommonAttributes(elm) {
	var formObj = document.forms[0], dom = tinymcePlugin.editor.dom;

	// Setup form data for common element attributes
	setFormValue('title', dom.getAttrib(elm, 'title'));
	setFormValue('id', dom.getAttrib(elm, 'id'));
	selectByValue(formObj, 'class', dom.getAttrib(elm, 'class'), true);
	//setFormValue('style', dom.getAttrib(elm, 'style'));
	//selectByValue(formObj, 'dir', dom.getAttrib(elm, 'dir'));
	setFormValue('lang', dom.getAttrib(elm, 'lang'));
}

function setFormValue(name, value) {
	if (document.forms[0].elements[name]) {
		document.forms[0].elements[name].value = value;
	}
}

function insertDateTime(id) {
	document.getElementById(id).value = getDateTime(new Date(), "%Y-%m-%dT%H:%M:%S");
}

function getDateTime(d, fmt) {
	fmt = fmt.replace("%D", "%m/%d/%y");
	fmt = fmt.replace("%r", "%I:%M:%S %p");
	fmt = fmt.replace("%Y", "" + d.getFullYear());
	fmt = fmt.replace("%y", "" + d.getYear());
	fmt = fmt.replace("%m", addZeros(d.getMonth() + 1, 2));
	fmt = fmt.replace("%d", addZeros(d.getDate(), 2));
	fmt = fmt.replace("%H", "" + addZeros(d.getHours(), 2));
	fmt = fmt.replace("%M", "" + addZeros(d.getMinutes(), 2));
	fmt = fmt.replace("%S", "" + addZeros(d.getSeconds(), 2));
	fmt = fmt.replace("%I", "" + ((d.getHours() + 11) % 12 + 1));
	fmt = fmt.replace("%p", "" + (d.getHours() < 12 ? "AM" : "PM"));
	fmt = fmt.replace("%%", "%");

	return fmt;
}

function addZeros(value, len) {
	var i;

	value = "" + value;

	if (value.length < len) {
		for (i = 0; i < (len - value.length); i++) {
			value = "0" + value;
		}
	}

	return value;
}

function setAttrib(elm, attrib, value) {
	var formObj = document.forms[0];
	var valueElm = formObj.elements[attrib.toLowerCase()];
	value = attrib === 'class' && parseInt(value) === -1 ? '' : value;
	tinymcePlugin.editor.dom.setAttrib(elm, attrib, value || valueElm.value);
}

function setAllCommonAttribs(elm) {
	setAttrib(elm, 'title');
	setAttrib(elm, 'id');
	setAttrib(elm, 'class');
//	setAttrib(elm, 'style');
//	setAttrib(elm, 'dir');
	setAttrib(elm, 'lang');
}

var SXE = {
	currentAction: "insert",
	inst: tinymcePlugin.editor,
	updateElement: null
};

SXE.focusElement = SXE.inst.selection.getNode();

SXE.initElementDialog = function (element_name) {
	document.getElementById('class').innerHTML = tinymcePlugin.getClassOptions('element_name');
	TinyMCE_EditableSelects.init();

	element_name = element_name.toLowerCase();
	var elm = SXE.inst.dom.getParent(SXE.focusElement, element_name.toUpperCase());
	if (elm !== null && elm.nodeName.toUpperCase() == element_name.toUpperCase()) {
		SXE.currentAction = "update";
	}

	if (SXE.currentAction === "update") {
		initCommonAttributes(elm);
		SXE.updateElement = elm;
	}

	//document.forms[0].insert.value = tinymcePlugin.getLang(SXE.currentAction, 'Insert', true);
};

SXE.insertElement = function (element_name) {
	var elm = SXE.inst.dom.getParent(SXE.focusElement, element_name.toUpperCase()), h, tagName;

	if (elm === null) {
		var s = SXE.inst.selection.getContent();
		if (s.length > 0) {
			tagName = element_name;

			insertInlineElement(element_name);
			var elementArray = window.tinymce.grep(SXE.inst.dom.select(element_name));
			for (var i = 0; i < elementArray.length; i++) {
				elm = elementArray[i];

				if (SXE.inst.dom.getAttrib(elm, 'data-mce-new')) {
					elm.id = '';
					elm.setAttribute('id', '');
					elm.removeAttribute('id');
					elm.removeAttribute('data-mce-new');

					setAllCommonAttribs(elm);
				}
			}
		}
	} else {
		setAllCommonAttribs(elm);
	}
	SXE.inst.nodeChanged();
	tinymcePlugin.execCommand('mceEndUndoLevel');
};

SXE.removeElement = function (element_name) {
	element_name = element_name.toLowerCase();
	var elm = SXE.inst.dom.getParent(SXE.focusElement, element_name.toUpperCase());
	if (elm && elm.nodeName.toUpperCase() === element_name.toUpperCase()) {
		tinyMCE.execCommand('mceRemoveNode', false, elm);
		SXE.inst.nodeChanged();
		tinymcePlugin.execCommand('mceEndUndoLevel');
	}
};

SXE.disableRemoveButton = function (disabled) {
	document.getElementById('btn_function_trash').disabled = disabled;
};

function insertInlineElement(en) {
	var ed = tinymcePlugin.editor, dom = ed.dom;

	ed.getDoc().execCommand('FontName', false, 'mceinline');
	window.tinymce.each(dom.select('span,font'), function (n) {
		if (n.style.fontFamily === 'mceinline' || n.face === 'mceinline') {
			dom.replace(dom.create(en, {'data-mce-new': 1}), n, 1);
		}
	});
}
